Package: AbSolution
Title: Interactive Feature-Based Analysis of AIRR-Seq Data
Version: 1.0.1
Authors@R: c(
    person("Rodrigo", "García-Valiente", , "r.garciavaliente@amsterdamumc.nl", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0003-0444-5587", email2 = "rodrigo.garcia.valiente+absolution@email.com")),
    person("Charisios", "Triantafyllou", , "charisios.triantafyllou@genyo.es", role = "aut",
           comment = c(ORCID = "0000-0002-0283-141X")),
    person("Antoine", "van Kampen", , "a.h.c.vankampen@amsterdamumc.nl", role = c("aut", "ths"),
           comment = c(ORCID = "0000-0003-1025-7232"))
  )
Description: An interactive framework for the exploration and analysis of
    adaptive immune receptor repertoire sequencing (AIRR-seq) data. It
    enables large-scale computation and integrated analysis of
    sequence-derived features, including physicochemical properties, amino
    acid descriptor sets, sequence motifs, compositional patterns, and
    somatic hypermutation metrics. The application supports multiscale
    analysis across sequences, clones, and repertoires, with interactive
    visualizations and statistical feature selection. 'AbSolution' also
    facilitates reproducible research by enabling structured export of
    data, code, parameters, and computational environments.  See
    <https://github.com/EDS-Bioinformatics-Laboratory/AbSolution> for more
    details.
License: GPL (>= 3)
URL: https://github.com/EDS-Bioinformatics-Laboratory/AbSolution,
        https://eds-bioinformatics-laboratory.github.io/AbSolution/
BugReports: https://github.com/EDS-Bioinformatics-Laboratory/AbSolution/issues
Depends: R (>= 4.4.0)
Imports: alakazam, attachment, benchmarkme, bigassertr, bigparallelr,
        bigstatsr, Biostrings, bs4Dash, callr, colourpicker, config (>=
        0.3.2), dashboardthemes, data.table, dockerfiler, doParallel,
        dplyr, DT, foreach, fresh, fs, ggplot2, golem (>= 0.4.1),
        grDevices, htmlwidgets, IRanges, iterors, knitr, parallel,
        Peptides, plotly, pwalign, reactable, rlang, rmarkdown, seqinr,
        shiny (>= 1.7.4), shinycssloaders, shinyFiles, shinyjs,
        shinymanager, shinymeta, shinythemes, shinyWidgets, sortable,
        stats, stringdist, stringr, sunburstR, tools, umap, upsetjs,
        utils, viridis, xfun
Additional_repositories: https://bioconductor.org/packages/release/bioc
Suggests: testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-26 10:46:27 UTC; rgarcia
Author: Rodrigo García-Valiente [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-0444-5587>, email2:
    rodrigo.garcia.valiente+absolution@email.com),
  Charisios Triantafyllou [aut] (ORCID:
    <https://orcid.org/0000-0002-0283-141X>),
  Antoine van Kampen [aut, ths] (ORCID:
    <https://orcid.org/0000-0003-1025-7232>)
Maintainer: Rodrigo García-Valiente <r.garciavaliente@amsterdamumc.nl>
Repository: CRAN
Date/Publication: 2026-04-27 08:50:09 UTC
Built: R 4.5.2; ; 2026-05-01 19:35:06 UTC; unix

Package: AbSolution
Title: Interactive Feature-Based Analysis of AIRR-Seq Data
Version: 1.0.1
Authors@R: c(
    person("Rodrigo", "García-Valiente", , "r.garciavaliente@amsterdamumc.nl", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0003-0444-5587", email2 = "rodrigo.garcia.valiente+absolution@email.com")),
    person("Charisios", "Triantafyllou", , "charisios.triantafyllou@genyo.es", role = "aut",
           comment = c(ORCID = "0000-0002-0283-141X")),
    person("Antoine", "van Kampen", , "a.h.c.vankampen@amsterdamumc.nl", role = c("aut", "ths"),
           comment = c(ORCID = "0000-0003-1025-7232"))
  )
Description: An interactive framework for the exploration and analysis of
    adaptive immune receptor repertoire sequencing (AIRR-seq) data. It
    enables large-scale computation and integrated analysis of
    sequence-derived features, including physicochemical properties, amino
    acid descriptor sets, sequence motifs, compositional patterns, and
    somatic hypermutation metrics. The application supports multiscale
    analysis across sequences, clones, and repertoires, with interactive
    visualizations and statistical feature selection. 'AbSolution' also
    facilitates reproducible research by enabling structured export of
    data, code, parameters, and computational environments.  See
    <https://github.com/EDS-Bioinformatics-Laboratory/AbSolution> for more
    details.
License: GPL (>= 3)
URL: https://github.com/EDS-Bioinformatics-Laboratory/AbSolution,
        https://eds-bioinformatics-laboratory.github.io/AbSolution/
BugReports: https://github.com/EDS-Bioinformatics-Laboratory/AbSolution/issues
Depends: R (>= 4.4.0)
Imports: alakazam, attachment, benchmarkme, bigassertr, bigparallelr,
        bigstatsr, Biostrings, bs4Dash, callr, colourpicker, config (>=
        0.3.2), dashboardthemes, data.table, dockerfiler, doParallel,
        dplyr, DT, foreach, fresh, fs, ggplot2, golem (>= 0.4.1),
        grDevices, htmlwidgets, IRanges, iterors, knitr, parallel,
        Peptides, plotly, pwalign, reactable, rlang, rmarkdown, seqinr,
        shiny (>= 1.7.4), shinycssloaders, shinyFiles, shinyjs,
        shinymanager, shinymeta, shinythemes, shinyWidgets, sortable,
        stats, stringdist, stringr, sunburstR, tools, umap, upsetjs,
        utils, viridis, xfun
Additional_repositories: https://bioconductor.org/packages/release/bioc
Suggests: testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-26 10:46:27 UTC; rgarcia
Author: Rodrigo García-Valiente [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-0444-5587>, email2:
    rodrigo.garcia.valiente+absolution@email.com),
  Charisios Triantafyllou [aut] (ORCID:
    <https://orcid.org/0000-0002-0283-141X>),
  Antoine van Kampen [aut, ths] (ORCID:
    <https://orcid.org/0000-0003-1025-7232>)
Maintainer: Rodrigo García-Valiente <r.garciavaliente@amsterdamumc.nl>
Repository: CRAN
Date/Publication: 2026-04-27 08:50:09 UTC
Built: R 4.6.0; ; 2026-04-27 09:56:49 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Ab_palette)
export(Feature_1)
export(Feature__dataset)
export(big_PCA)
export(calculate_clone)
export(draw_feature_violinplot)
export(draw_sharedclonesplot)
export(draw_upsetplot)
export(draw_violinplots)
export(filter_merged)
export(merge_FBMs)
export(parse_AIRRSeq_file)
export(run_app)
export(validate_before_run)
import(Peptides)
import(attachment)
import(bigassertr)
import(bigparallelr)
import(bigstatsr)
import(dashboardthemes)
import(dplyr)
import(foreach)
import(fresh)
import(golem)
import(grDevices)
import(parallel)
import(plotly)
import(reactable)
import(shiny)
import(shinyFiles)
import(shinycssloaders)
import(shinymanager)
import(shinymeta)
import(shinythemes)
import(sortable)
import(stringdist)
import(stringr)
import(tools)
import(upsetjs)
import(utils)
import(viridis)
importFrom(Biostrings,GENETIC_CODE)
importFrom(Biostrings,subseq)
importFrom(data.table,":=")
importFrom(dockerfiler,dock_from_desc)
importFrom(golem,activate_js)
importFrom(golem,add_resource_path)
importFrom(golem,bundle_resources)
importFrom(golem,favicon)
importFrom(golem,with_golem_options)
importFrom(shiny,HTML)
importFrom(shiny,column)
importFrom(shiny,shinyApp)
importFrom(shiny,tagAppendAttributes)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(stringr,str_locate_all)
importFrom(stringr,str_split)
importFrom(utils,adist)
importFrom(utils,glob2rx)

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/AbSolution)](https://cran.r-project.org/package=AbSolution)
[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
<!-- badges: end -->

<p align="center">
  <img src="man/figures/AbSolution_logo.png" alt="AbSolution logo" width="300"/>
</p>

# AbSolution <a id="top"></a>

**An interactive and scalable Shiny application for analyzing sequence features of B-cell and T-cell receptor repertoires.**

AbSolution supports identification and visualization of variable region characteristics, clonotypes, and mutation patterns from immunoglobulin and TCR sequence data. It has been designed under the principles of accessibility, scalability, flexibility, interactivity, and reproducibility.

---

## Features

- **AIRR-Seq parsing**: import and parse AIRR-formatted sequence data with automatic germline reconstruction
- **Sequence feature extraction**: nucleotide/amino acid properties, mutation analysis, physicochemical descriptors (Kidera factors, Atchley factors, etc.)
- **Scalable computation**: backed by file-backed big matrices ([bigstatsr](https://github.com/privefl/bigstatsr)) for large repertoires
- **Interactive visualization**: PCA/UMAP projections, violin plots, clonal frequency distributions, UpSet plots for shared clones
- **Differential analysis**: variable selection between user-defined groups with univariate logistic regression
- **Clonal exploration**: multiple clonotype definitions, shared clone detection, dominance analysis
- **Reproducible exports**: ENCORE-formatted output bundles with code, data, Docker environment, and rendered reports

---

## Installation

### From CRAN - Recommended

AbSolution is available on CRAN: https://cran.r-project.org/package=AbSolution

```r
if (!requireNamespace("AbSolution", quietly = TRUE))
    install.packages("AbSolution")
```

### From GitHub

```r
# Install Bioconductor dependencies first
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install(c("Biostrings", "IRanges", "pwalign"))

# Install AbSolution
if (!requireNamespace("remotes", quietly = TRUE))
    install.packages("remotes")
remotes::install_github("EDS-Bioinformatics-Laboratory/AbSolution")
```

### From source

```r
install.packages("AbSolution_1.0.1.tar.gz", repos = NULL, type = "source")
```

### From a shared exported proyect

#### Docker image

A Docker setup is available in the `0_SoftwareEnvironment/R/` directory of the exported project:

```bash
cd deploy
docker build -f Dockerfile_base --progress=plain -t absolution_base .
docker build -f Dockerfile --progress=plain -t absolution:latest .
docker run -p 3838:3838 absolution:latest
# Open http://127.0.0.1:3838
```

#### renv file

A renv.lock file is available in the `0_SoftwareEnvironment/R/` directory of the exported project:

```R
renv::restore(lockfile = "0_SoftwareEnvironment/R/renv.lock")
```

#### Install AbSolution exported source file

```r
install.packages("0_SoftwareEnvironment/R/AbSolution_1.0.0.tar.gz", repos = NULL, type = "source")
```
---


## Quick start

```r
library(AbSolution)
run_app()
```

This launches the Shiny application. A built-in test dataset is available to explore all features without needing your own data.

For a complete guide, see the package vignette:

```r
vignette("AbSolution")
```

---

## Workflow overview

<p align="center">
  <img src="man/figures/AbSolution_scheme.png" alt="AbSolution workflow" width="700"/>
</p>

AbSolution guides you through a step-by-step analysis:

1. **Project information** — define your samples, groups, and metadata
2. **AIRR-Seq conversion** — parse sequences and reconstruct germlines
3. **Feature calculation** — calculate sequence-based features for each sequence
4. **Dataset exploration** — filter variables, compare groups, explore PCA/UMAP projections
5. **Feature exploration** — examine individual sequence features across groups
6. **Clonal exploration** — analyze clonotype distributions, shared clones, and dominance
7. **Export** — generate reproducible reports in ENCORE format

---

## Input format

AbSolution accepts [AIRR-Seq](https://docs.airr-community.org/en/stable/) formatted TSV files as produced by tools such as [IMGT/HighV-QUEST](https://www.imgt.org/HighV-QUEST/), [IgBLAST](https://www.ncbi.nlm.nih.gov/igblast/), [MiXCR](https://github.com/milaboratory/mixcr), or [10x Genomics Cell Ranger](https://www.10xgenomics.com/support/software/cell-ranger).

---

## Citation

If you use AbSolution in your research, please cite:

> In preparation!

---

## Contributing

Contributions are welcome. Please open an [issue](https://github.com/EDS-Bioinformatics-Laboratory/AbSolution/issues) to report bugs or suggest features, or submit a pull request.

---

## License

AbSolution is released under the [GPL-3](https://github.com/EDS-Bioinformatics-Laboratory/AbSolution/blob/main/LICENSE.md) license.

---

## Acknowledgments

Developed at the [Bioinformatics Laboratory](https://bioinformaticslaboratory.eu/), Department of Epidemiology and Data Science, Amsterdam UMC, University of Amsterdam, the Netherlands.

<p align="center">
  <img src="man/figures/BioinformaticsLaboratory.png" alt="Bioinformatics Laboratory" height="60"/>
  &nbsp;&nbsp;&nbsp;
  <img src="man/figures/UMC_logo.png" alt="Amsterdam UMC" height="60"/>
</p>

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup_AbS, warning = FALSE, message = FALSE, eval = FALSE, results="hide"----
#  # Load required package
#  library(AbSolution)
#  
#  # Load Shiny app
#  AbSolution::run_app()
#  

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup_AbS, warning = FALSE, message = FALSE, eval = FALSE, results="hide"----
# # Load required package
# library(AbSolution)
# 
# # Load Shiny app
# AbSolution::run_app()
# 

#' 5_Feature_exploration
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import plotly
#' @keywords internal
#' @export
draw_feature_violinplot <- function(
    values, name_values, sequence_info_df, group_info,
    additional_group_info = "Patient",
    show_reconstructed, compare_opposites=FALSE, selected_rows,
    selected_subclones = NULL, selected_clones = NULL,hide_dots = FALSE,
    really_hide_dots=FALSE, width=1400, height=1000, img_type= "png", scale=4,
    seed=1234, source="feature_violinplot", primary_color="#09BC8A"
) {
  set.seed(seed)
  # print("Feature violin")
  # devtools::install_github('karthik/wesanderson') pal <-
  # wes_palette(length(unique(seq_df[,..group]))*2, name = 'Zissou1', type =
  # 'continuous')
  fig <- plot_ly(type = "violin") %>%
    config(
      toImageButtonOptions = list(
        format = img_type,
        filename = paste("Violin_plot_features",name_values, Sys.time(),sep="_"),
        width = width,
        height = height,
        scale=scale
      )
    )%>%
    layout(
      xaxis = list(showline = TRUE,
                   zeroline = FALSE),
      yaxis = list(showline = TRUE,
                   zeroline = FALSE)
    )


  if (is.null(group_info) ||
      group_info == "") {
    group_info <- "Group"
  }
  if (show_reconstructed || length(unique(sequence_info_df$Seq_type[selected_rows])) >
      2) {
    i <- 0
    # print("Reconstr")
    if (show_reconstructed && !("Reconstructed germline" %in% unique(sequence_info_df$Seq_type[selected_rows]))) {
      selected_rows <- sort(c(selected_rows, selected_rows - 1))
    } else if (show_reconstructed && !("Repertoire" %in% unique(sequence_info_df$Seq_type[selected_rows]))) {
      selected_rows <- sort(c(selected_rows, selected_rows + 1))
    }

    if (compare_opposites) {
      meta_summary<- data.frame(Group=  as.character(),
                                Subgroup= as.character() ,
                                Seq_ID = as.character(),
                                Value = as.numeric())
    }

    for (single_group in sort(unique(sequence_info_df[selected_rows, get(group_info)]),decreasing = T)) {
      # print(single_group)
      index <- intersect(
        selected_rows, which(
          sequence_info_df[, get(group_info)] ==
            single_group
        )
      )
      subset_seq_df <- sequence_info_df[intersect(
        selected_rows, which(
          sequence_info_df[, get(group_info)] ==
            single_group
        )
      ),
      ]
      subset_group <- sequence_info_df[intersect(
        selected_rows, which(
          sequence_info_df[, get(group_info)] ==
            single_group
        )
      ),
      get("additional_group_info")]

      for (seq_type in c("Repertoire", "Reconstructed_germline")) {
        i <- i + 1
        # print(seq_type)
        sub_index <- intersect(index, which(sequence_info_df$Sequence_type == seq_type))
        subsubset_seq_df <- subset_seq_df[which(subset_seq_df$Sequence_type == seq_type),
        ]
        subsubset_group <- subset_group[which(subset_seq_df$Sequence_type == seq_type)]
        if (nrow(subsubset_seq_df) >
            0) {

          # if(seq_type =='Reconstructed_germline') {
          # print(head(subsubset_seq_df)) }
          tmp_summary <- data.frame(Group= single_group,
                                    Subgroup=seq_type ,
                                    Seq_ID = subsubset_seq_df$ID,
                                    Value = values[sub_index])

          if(compare_opposites) {
            meta_summary <- rbind(meta_summary, tmp_summary)
          }

          index_nonselected <- which(!(tmp_summary$Seq_ID %in% c(selected_subclones,selected_clones)))
          index_selected_sub <- which((tmp_summary$Seq_ID %in% selected_subclones))
          index_selected_clones <- which((tmp_summary$Seq_ID %in% selected_clones))
          index_selected<- sort(unique(c(index_selected_sub,index_selected_clones)))
          index_colors_sub<- sapply(index_selected, function(z) if (z %in%index_selected_clones && z %in% index_selected_sub) {"#C07100"} else if(z %in%index_selected_clones){"#FFA500"}else {"#000000"})

          fig <- add_trace(
            fig, x = paste(single_group, sep = ""),
            y = tmp_summary$Value[index_nonselected], hoveron = "points",
            text = paste(
              tmp_summary$Seq_ID[index_nonselected], " Value=", tmp_summary$Value[index_nonselected],
              sep = ""
            ),
            key = tmp_summary$Seq_ID[index_nonselected], legendgroup = seq_type,
            scalegroup = single_group, selectedpoints = if (any(tmp_summary$Seq_ID[index_nonselected] %in% c(selected_subclones, selected_clones))) {
              ((which(tmp_summary$Seq_ID[index_nonselected] %in% c(selected_subclones, selected_clones)) -
                  1))
            } else {
              NULL
            }, name = seq_type, side = if (seq_type == "Reconstructed_germline") {
              "positive"
            } else {
              "negative"
            }, box = list(visible = TRUE, line = list(color = "#2D2926")),
            points = if(really_hide_dots){FALSE}else{"all"}, pointpos = if (seq_type == "Reconstructed_germline") {
              0.5
            } else {
              -0.5
            }, jitter = 0.25, scalemode = "count", meanline = list(visible = T),
            color = I(
              if (i%%2 != 0) {
                primary_color
              } else {
                "#707A6C"
              }
            ),
            marker = list(
              opacity = if (hide_dots) {
                0
              } else {
                0.75
              }, line = list(
                width = 2, color = if (i%%2 != 0) {
                  primary_color
                } else {
                  "#707A6C"
                }
              )
            ),
            unselected = list(
              marker = list(
                size = 2, opacity = if (hide_dots) {
                  0
                } else {
                  0.1
                }, color = if (i%%2 != 0) {
                  primary_color
                } else {
                  "#707A6C"
                }
              )
            ),
            selected = list(
              marker = list(
                size = 10, opacity = if (hide_dots) {
                  0.5
                } else {
                  1
                }, color = "#2D2926", zorder = 2
              )
            ),
            showlegend = if (i > 2) {
              F
            } else {
              T
            }
          )



          if (length(index_selected) !=
              0) {
            fig <- add_trace(
              fig, x = paste(single_group, sep = ""),
              y = tmp_summary$Value[index_selected], hoveron = "points",
              text = paste(
                tmp_summary$Seq_ID[index_selected], " Value=", tmp_summary$Value[index_selected],
                sep = ""
              ),
              key = tmp_summary$Seq_ID[index_selected], legendgroup = seq_type,
              scalegroup = single_group, selectedpoints = if (any(tmp_summary$Seq_ID[index_selected]  %in% c(selected_subclones, selected_clones))) {
                ((which(tmp_summary$Seq_ID[index_selected] %in% c(selected_subclones, selected_clones)) -
                    1))
              } else {
                NULL
              }, name = seq_type, side = if (seq_type == "Reconstructed_germline") {
                "positive"
              } else {
                "negative"
              }, box = list(visible = TRUE, line = list(color = "#2D2926")),
              points = if(really_hide_dots){FALSE}else{"all"}, pointpos = if (seq_type == "Reconstructed_germline") {
                0.5
              } else {
                -0.5
              }, jitter = 0.25, scalemode = "count", meanline = list(visible = T),
              color = I(
                if (i%%2 != 0) {
                  primary_color
                } else {
                  "#707A6C"
                }
              ),
              marker = list(
                opacity = if (hide_dots) {
                  0
                } else {
                  0.75
                }, line = list(
                  width = 2, color = if (i%%2 != 0) {
                    primary_color
                  } else {
                    "#707A6C"
                  }
                )
              ),
              unselected = list(
                marker = list(
                  size = 2, opacity = if (hide_dots) {
                    0
                  } else {
                    0.1
                  }, color = if (i%%2 != 0) {
                    primary_color
                  } else {
                    "#707A6C"
                  }
                )
              ),
              selected = list(
                marker = list(
                  size = 10, opacity = if (hide_dots) {
                    0.5
                  } else {
                    1
                  }, color = index_colors_sub, zorder = 2
                )
              ),
              showlegend = FALSE
            )
          }

        }

      }

    }

  } else {
    i <- 1
    for (single_group in unique(sequence_info_df[, get(group_info)])) {
      index <- intersect(
        selected_rows, which(
          sequence_info_df[, get(group_info)] ==
            single_group
        )
      )
      subset_seq_df <- sequence_info_df[intersect(
        selected_rows, which(
          sequence_info_df[, get(group_info)] ==
            single_group
        )
      ),
      ]
      subset_group <- sequence_info_df[intersect(
        selected_rows, which(
          sequence_info_df[, get(group_info)] ==
            single_group
        )
      ),
      get("additional_group_info")]

      tmp_summary <- data.frame(Seq_ID = subset_seq_df$ID, Value = values[index])

      index_nonselected <- which(!(tmp_summary$Seq_ID %in% c(selected_subclones,selected_clones)))
      index_selected_sub <- which((tmp_summary$Seq_ID %in% selected_subclones))
      index_selected_clones <- which((tmp_summary$Seq_ID %in% selected_clones))
      index_selected <- sort(unique(c(index_selected_sub,index_selected_clones)))
      index_colors_sub<- sapply(index_selected, function(z) if (z %in%index_selected_clones && z %in% index_selected_sub) {"#C07100"} else if(z %in%index_selected_clones){"#FFA500"}else {"#000000"})
      fig <- add_trace(
        fig, x = paste(single_group, sep = ""),
        y = tmp_summary$Value[index_nonselected], hoveron = "points", text = paste(
          tmp_summary$Seq_ID[index_nonselected], " Value=", tmp_summary$Value[index_nonselected],
          sep = ""
        ),
        key = tmp_summary$Seq_ID[index_nonselected], legendgroup = single_group,
        scalegroup = single_group, selectedpoints = if (any(tmp_summary$Seq_ID[index_nonselected] %in% c(selected_subclones, selected_clones))) {
          ((which(tmp_summary$Seq_ID[index_nonselected] %in% c(selected_subclones, selected_clones)) -
              1))
        } else {
          NULL
        }, name = single_group, box = list(visible = TRUE, line = list(color = "#2D2926")),
        points = if(really_hide_dots){FALSE}else{"all"}, pointpos = 0, jitter = 0.25, scalemode = "count",
        meanline = list(visible = T),
        color = I(
          if (i%%2 != 0) {
            primary_color
          } else {
            "#707A6C"
          }
        ),
        marker = list(
          opacity = if (hide_dots) {
            0
          } else {
            0.75
          }, line = list(
            width = 2, color = if (i%%2 != 0) {
              primary_color
            } else {
              "#707A6C"
            }
          )
        ),
        unselected = list(
          marker = list(
            size = 2, opacity = if (hide_dots) {
              0
            } else {
              0.1
            }, color = if (i%%2 != 0) {
              primary_color
            } else {
              "#707A6C"
            }
          )
        ),
        selected = list(
          marker = list(
            size = 10, opacity = if (hide_dots) {
              0.5
            } else {
              1
            }, color = "#000000", zorder = 2
          )
        ),
        showlegend = FALSE
      )



      if (length(index_selected) !=
          0) {

        fig <- add_trace(
          fig, x = paste(single_group, sep = ""),
          y = tmp_summary$Value[index_selected], hoveron = "points", text = paste(
            tmp_summary$Seq_ID[index_selected], " Value=", tmp_summary$Value[index_selected],
            sep = ""
          ),
          key = tmp_summary$Seq_ID[index_selected], legendgroup = single_group,
          scalegroup = single_group, selectedpoints = if (any(tmp_summary$Seq_ID[index_selected] %in% c(selected_subclones, selected_clones))) {
            ((which(tmp_summary$Seq_ID[index_selected] %in% c(selected_subclones, selected_clones)) -
                1))
          } else {
            NULL
          }, name = single_group, box = list(visible = T, line = list(color = "#2D2926")),
          points =if(really_hide_dots){FALSE}else{"all"}, pointpos = 0, jitter = 0.25, scalemode = "count",
          meanline = list(visible = TRUE),
          color = I(
            if (i%%2 != 0) {
              primary_color
            } else {
              "#707A6C"
            }
          ),
          marker = list(
            opacity = if (hide_dots) {
              0
            } else {
              0.75
            }, line = list(
              width = 2, color = if (i%%2 != 0) {
                primary_color
              } else {
                "#707A6C"
              }
            )
          ),
          unselected = list(
            marker = list(
              size = 2, opacity = if (hide_dots) {
                0
              } else {
                0.1
              }, color = if (i%%2 != 0) {
                primary_color
              } else {
                "#707A6C"
              }
            )
          ),
          selected = list(
            marker = list(
              size = 10, opacity = if (hide_dots) {
                0.5
              } else {
                1
              }, color = index_colors_sub, zorder = 2
            )
          ),
          showlegend = FALSE
        )
      }


    }

  }

  base_font <- 14
  font_size <- round(base_font * scale)


  fig <- layout(
    fig, title = list(font = list(size = font_size + 6),
                      text=paste(c(
                        "Violin plot of the <i>feature ",
                        name_values,
                        "</i> <br> between the groups ",
                        group_info
                      ),
                      collapse = ""
                      )),
    font = list(size = font_size),

    xaxis = list(
      tickfont = list(size = font_size)
    ),

    yaxis = list(
      tickfont = list(size = font_size)
    ),

    legend = list(
      font = list(size = font_size - 1),
      tracegroupgap = 0
    ),

    hoverlabel = list(
      font = list(size = font_size)
    ),
    margin = list(
      l = 80,
      r = 80,
      t = 90 + font_size,
      b = 40 ),
    # violingap = 0, violingroupgap = 0, violinmode = "overlay",
    dragmode = "lasso"
  ) %>%
    config(
      displaylogo = FALSE, modeBarButtonsToRemove = c("autoScale2d", "pan2d", "hoverCompareCartesian")
    )


  if ((show_reconstructed || length(unique(sequence_info_df$Seq_type[selected_rows])) >
       2) && compare_opposites)
  {

    lab_from_p <- function(p){
      if (is.na(p)) return("ns")
      if (p < 0.001) return("***")
      if (p < 0.01)  return("**")
      if (p < 0.05)  return("*")
      "ns"
    }

    cats <- (unique(meta_summary$Group))
    p_by_cat <- stats::setNames(numeric(length(cats)), cats)
    lab_by_cat <- stats::setNames(character(length(cats)), cats)
    ypos_by_cat <- stats::setNames(numeric(length(cats)), cats)

    for (cc in cats){
      dcc <- meta_summary[meta_summary$Group == cc, ]
      pa <- dcc$Value[dcc$Subgroup == "Repertoire"]
      pb <- dcc$Value[dcc$Subgroup != "Repertoire"]
      tt <- stats::t.test(pa, pb, paired=T)
      # print(cc)
      # print(tt$p.value)
      p_by_cat[cc] <- tt$p.value
      d <- (mean(pa) - mean(pb)) / stats::sd(c(pa,pb))
      lab_by_cat[cc] <- if(d<=0.2){""}else{lab_from_p(tt$p.value)}
      ypos_by_cat[cc] <- max(dcc$Value, na.rm = TRUE) + 0.4  # altura de la barra
    }
    n <- length(cats)


    centers <- stats::setNames(if (n > 1) (seq_len(n) - 0.6) / n else 0.5, cats)


    bar_frac <- 0.6
    half_width <- if (n > 1) (bar_frac / 2) * (1 / n) else 0.2  # para n=1, ancho razonable


    bump_by_cat <- stats::setNames(numeric(n), cats)
    for (cc in cats) {

      y_range_cc <- range(ypos_by_cat[[cc]], na.rm = TRUE)

      bump_by_cat[[cc]] <- 0.04 * diff(range(values, na.rm = TRUE))
    }


    shapes_list <- list()
    ann_list <- list()

    for (cc in cats){
      cx <- centers[[cc]]
      y0 <- ypos_by_cat[[cc]]
      bump <- bump_by_cat[[cc]]

      if(lab_by_cat[[cc]]!="") {
        shapes_list[[length(shapes_list) + 1]] <- list(
          type = "line",
          xref = "x domain", yref = "y",
          x0 = cx - half_width, x1 = cx + half_width,
          y0 = y0, y1 = y0,
          line = list(color = "black", width = 2)
        )


        ann_list[[length(ann_list) + 1]] <- list(
          xref = "x domain", yref = "y",
          x = cx, y = y0 + bump,
          text = lab_by_cat[[cc]],
          showarrow = FALSE,
          font = list(size = 19)
        )
      }


    }


    fig <- fig %>% layout(shapes = shapes_list, annotations = ann_list)
  }


  fig <- suppressWarnings(fig %>%
    layout(plot_bgcolor = "rgba(0, 0, 0, 0)",
           paper_bgcolor = "rgba(0, 0, 0, 0)",
           yaxis = list(  showgrid = TRUE,
                          gridcolor = "#495057",
           minor = list(dtick = 0.1, ticklen = 3)) ) %>%
    toWebGL())

  return(fig)
}

#' Ab_palette: color ranges adapted to the BCR/TCR gene structure
#'
#' @description
#'    #' @description
#'    **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'    Returns a vector of colors according to the input.
#'    For V/D/J/VJ/VDJ inputs, the colors are produced in a way that genes and
#'    combination of genes from the same family are assigned a similar color.
#' @param list_values For V(D)J combinations, a list of lists of the families of
#'     each segment and the genes within them. For V/D/J segments, a list of the
#'      families and the genes within them. For cuantitative or cualitative
#'      values, a list of values.
#' @param vect_genes_comb The vector of the V(D)J combinations present in the
#'    dataset if type_values is 'VJ' or 'VDJ'. Otherwise, NA.
#' @param type_values One of 'V','D','J','VJ','VDJ', 'cuantitative' or
#'    'cualitative'.  'V','D','J','VJ','VDJ',
#' @param colorblind If TRUE, the output is a colorblind-friendly vector of
#'    colors from the viridis package. The similarity of the V-D-J colors is
#'    lost.
#'
#' @return palette_colors: a (named) vector of colors
#' @import grDevices
#' @import viridis
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
Ab_palette <- function(
    list_values, vect_genes_comb = NA, type_values = c("V", "D", "J", "VJ", "VDJ", "cuantitative", "cualitative"),
    colorblind = FALSE, seed=1234
) {

    palette_colors <- c()
    set.seed(seed)
    if (colorblind) {
        if (type_values == "cuantitative") {
            palette_colors <- viridis(
                max(list_values) -
                  min(list_values)
            )
        } else if (type_values == "cualitative" || type_values %in% c("V", "D", "J")) {
            palette_colors <- viridis(length(unlist(unlist(list_values))))
            names(palette_colors) <- sort(unlist(unlist(list_values)))
        } else {
            palette_colors <- viridis(length(unlist(unlist(vect_genes_comb))))
            names(palette_colors) <- sort(unlist(unlist(vect_genes_comb)))
        }
    } else {
        if (type_values == "VJ" || type_values == "VDJ") {
            n_genes <- length(list_values)
            RGB_list <- list()

            for (k in c(1:n_genes)) {
                n_family <- length(list_values[[k]])
                divisions <- seq(127, 255, length.out = n_family)  ###FORMER 0, 255
                # RGB_list[[names(list_values)[k]]]=list()

                for (i in c(1:n_family)) {
                  # RGB_list[[names(list_values)[k]]][[as.character(i)]]=list()
                  for (j in c(1:length(list_values[[k]][[i]]))) {
                    RGB_combn_exists <- T
                    RGB_val <- divisions[i]
                    tmp_RGB_val <- RGB_val
                    while (RGB_combn_exists) {

                      tmp_inc <- stats::rnorm(1, mean = 5.5, sd = 10)
                      if (tmp_RGB_val %in% unlist(RGB_list)) {
                        tmp_RGB_val <- RGB_val
                        if (sample(1:2)[1] ==
                          1) {
                          tmp_RGB_val <- max(
                            min(tmp_RGB_val + tmp_inc, 255),
                            0
                        )

                        } else {
                          tmp_RGB_val <- min(
                            max(tmp_RGB_val - tmp_inc, 0),
                            255
                        )
                        }

                      } else {
                        RGB_combn_exists <- F
                      }

                    }

                    RGB_list[[as.character(sort(list_values[[k]][[i]])[[j]])]] <- tmp_RGB_val
                  }

                }

            }

            if (type_values == "VJ") {
                palette_colors <- sapply(
                  vect_genes_comb, function(z) grDevices::rgb(
                    RGB_list[[strsplit(z, split = "_")[[1]][1]]],
                    122, RGB_list[[strsplit(z, split = "_")[[1]][2]]],
                    max = 255
                )
              )
                # seecol(usecol(pal = palette_colors, n = 'all'))

            } else {
                palette_colors <- sapply(
                  vect_genes_comb, function(z) if (grepl("__", z)) {
                    grDevices::rgb(
                      RGB_list[[strsplit(z, split = "_")[[1]][1]]],
                      RGB_list[[which(
                        names(RGB_list) ==
                          ""
                    )]],
                      RGB_list[[strsplit(z, split = "_")[[1]][3]]],
                      max = 255
                  )
                  } else {
                    grDevices::rgb(
                      RGB_list[[strsplit(z, split = "_")[[1]][1]]],
                      RGB_list[[strsplit(z, split = "_")[[1]][2]]],
                      RGB_list[[strsplit(z, split = "_")[[1]][3]]],
                      max = 255
                  )
                  }
              )
                # seecol(usecol(pal = palette_colors, n = 'all'))

            }


            # sapply(vect_genes_comb, function(z) RGB_list[[strsplit(z,
            # split='__')[[1]][1]]] )


        } else if (type_values == "cualitative") {
            some_colors <- list(
                blue = "#00798c", lavender = "#9e51bd", rosedust = "#9D5568", red = "#d1495b",
                orange = "#DF7C52", yellow = "#edae49", green = "#66a182", greenmoss = "#AAA866",
                greeeeen = "#2e853b", grey = "#333840", pinkputi = "#F72585", azulon = "#3A0CA3",
                azulin = "#4CC9F0", raro = "#D81159", naranja = "#f86624", gold = "#ff7d00",
                green = "#007200", darkgreen = "#004b23", d81159 = "#d81159", verypink = "#ff0a54",
                goldie = "#fbb13c", softmalva = "#9381ff", softyellow = "#fee440",
                lightgreen = "#abff4f", keyred = "#e5383b", brown = "#78290f", brownred = "#ad2831",
                verybrown = "#941b0c", darkbrown = "#220901", opaqueblack = "#242423",
                lightlessblue = "#003d5b", verylightbrown = "#d69f7e", veryldarkblue = "#04052e",
                clodublue = "#50d8d7", purpura = "#4f0147", aqua = "#037971", odoperdido = "#76520e",
                cloudgreen = "#7bf1a8", darkmagenta = "#290025", powerpink = "#ff70a6",
                lightgrey = "#d5bdaf"
            )

            for (element in c(1:length(list_values))) {
                if (element <= length(some_colors)) {

                  new_palette <- some_colors[[element]]
                } else {
                  is_in_palette <- T
                  while (is_in_palette) {
                    RGB <- round(stats::runif(3, min = -0.49, max = 255.49))
                    new_palette <- grDevices::rgb(RGB[1], RGB[2], RGB[3], maxColorValue = 255)
                    if (new_palette %!in% palette_colors && sum(
                      sapply(
                        RGB, function(z) 255 -
                          z
                    )
                  ) >
                      15 && sum(RGB) >
                      20) {
                      is_in_palette <- F
                    }
                  }


                }
                palette_colors <- c(palette_colors, new_palette)
            }
            names(palette_colors) <- sort(list_values)

        } else if (type_values == "cuantitative") {
            some_colors <- list(red = "#d1495b", orange = "#DF7C52", yellow = "#edae49", green = "#66a182")
            palette_colors <- colorRampPalette(some_colors)(max(list_values) -
                min(list_values))
        } else {
            n_family <- length(list_values)
            divisions <- seq(127, 255, length.out = n_family)   ### Former: 0, 255
            RGB_list <- list()
            for (i in c(1:n_family)) {
                RGB_combn_exists <- T

                while (RGB_combn_exists) {
                  RGB_combn <- round(stats::runif(3, min = 0.51, max = (n_family + 0.49)))
                  RGB_combn[1:3] <- RGB_combn[1]
                  RGB_combn[round(stats::runif(1, min = 0.51, max = 3.49))] <- NA
                  RGB_combn <- paste(RGB_combn, collapse = "_")
                  if (RGB_combn %!in% RGB_list) {
                    RGB_combn_exists <- F
                  }
                }

                RGB_list[[length(RGB_list) +
                  1]] <- RGB_combn
            }
            suppressWarnings(
                {
                  for (i in c(1:n_family)) {
                    tmp_palette_colors <- c()

                    # fam_divisions=seq(0, 255,
                    # length.out=length(list_values[[i]]))
                    fam_divisions <- round(stats::runif(1, min = -0.49, max = 255.49))
                    for (j in c(1:length(list_values[[i]]))) {
                      tmp_RGB <- as.numeric(strsplit(RGB_list[[i]], split = "_")[[1]])
                      tmp_RGB <- divisions[tmp_RGB]
                      NA_index <- which(is.na(tmp_RGB))
                      tmp_RGB[NA_index] <- fam_divisions


                      tmp_RGB_exists <- T

                      while (tmp_RGB_exists) {

                        if (sum(
                          sapply(
                            tmp_RGB, function(z) 255 -
                              z
                        )
                      ) <
                          15 || sum(tmp_RGB) <
                          45 || grDevices::rgb(tmp_RGB[1], tmp_RGB[2], tmp_RGB[3], maxColorValue = 255) %in%
                          palette_colors || grDevices::rgb(tmp_RGB[1], tmp_RGB[2], tmp_RGB[3], maxColorValue = 255) %in%
                          tmp_palette_colors) {

                          tmp_inc <- stats::rnorm(1, mean = 10, sd = 15)
                          if (sample(1:2)[1] ==
                            1) {
                            tmp_RGB[NA_index] <- max(
                              min(fam_divisions + tmp_inc, 255),
                              0
                          )

                          } else {
                            tmp_RGB[NA_index] <- min(
                              max(fam_divisions - tmp_inc, 0),
                              255
                          )
                          }
                        } else {
                          tmp_RGB_exists <- F
                          tmp_palette_colors <- c(
                            tmp_palette_colors, grDevices::rgb(tmp_RGB[1], tmp_RGB[2], tmp_RGB[3], maxColorValue = 255)
                        )

                        }
                      }
                    }

                    names(tmp_palette_colors) <- list_values[[i]]

                    palette_colors <- c(palette_colors, tmp_palette_colors)
                  }
                }
            )



        }
    }

    return(palette_colors)
}



#' Total variance of the PCA
#'
#' @description (squared) Frobenius norm (which is the total variance of the
#'    matrix) https://github.com/privefl/bigstatsr/issues/83
#' @author Florian Privé
#' @param X FBM object used in the bigstatsr::big_randomSVD calculation
#' @param ind.row FBM rows used in the bigstatsr::big_randomSVD calculation
#' @param ind.col FBM cols used in the bigstatsr::big_randomSVD calculation
#' @param center centering vector used in bigstatsr::big_randomSVD
#' @param scale scaling vector used in bigstatsr::big_randomSVD
#' @return (squared) Frobenius norm
#' @import bigstatsr
#' @import bigassertr
#' @noRd
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
big_norm <- function(
    X, ind.row = rows_along(X),
    ind.col = cols_along(X),
    center = rep(0, length(ind.col)),
    scale = rep(1, length(ind.col))
) {
    bigassertr::assert_lengths(center, scale, ind.col)
    stats <- bigstatsr::big_colstats(X, ind.row, ind.col)
    n <- length(ind.row)
    return(sum(((n - 1) * stats$var + n * (stats$sum/n - center)^2)/scale^2))
}


#' TEMPORAL
#'
#' @description colorss
#' @return color
#' @noRd

branded_colors <- list(
    blue = "#00798c", lavender = "#9e51bd", rosedust = "#9D5568", red = "#d1495b",
    orange = "#DF7C52", yellow = "#edae49", green = "#66a182", greenmoss = "#AAA866",
    navy = "#2e853b", grey = "#333840", pinkputi = "#F72585", azulon = "#3A0CA3",
    azulin = "#4CC9F0", raro = "#D81159", naranja = "#f86624", gold = "#ff7d00",
    green = "#007200", darkgreen = "#004b23", d81159 = "#d81159", verypink = "#ff0a54",
    goldie = "#fbb13c", softmalva = "#9381ff", softyellow = "#fee440", lightgreen = "#abff4f",
    keyred = "#e5383b", brown = "#78290f", brownred = "#ad2831", verybrown = "#941b0c",
    darkbrown = "#220901", opaqueblack = "#242423", lightlessblue = "#003d5b", verylightbrown = "#d69f7e",
    veryldarkblue = "#04052e", clodublue = "#50d8d7", purpura = "#4f0147", aqua = "#037971",
    odoperdido = "#76520e", cloudgreen = "#7bf1a8", darkmagenta = "#290025", powerpink = "#ff70a6",
    lightgrey = "#d5bdaf"
)

# Helper: compare hex color vectors allowing minor rounding differences
expect_colors_equal <- function(actual, expected, tolerance = 2) {
  actual_rgb <- t(grDevices::col2rgb(actual))
  expected_rgb <- t(grDevices::col2rgb(expected))
  expect_equal(actual_rgb, expected_rgb, tolerance = tolerance)
}

test_that("Ab_palette works for VDJ combinations", {

  library(alakazam)
  data("ExampleDb")
  vect_V_g=ExampleDb$v_call[1:10]
  vect_D_g=ExampleDb$d_call[1:10]
  vect_J_g=ExampleDb$j_call[1:10]

  vect_genes_comb=paste(vect_V_g,
                        vect_D_g,
                        vect_J_g,
                        sep="_")
  ##Example: Homsap IGHV3-11*05 F_Homsap IGHD3-10*01 F_Homsap IGHJ4*03 F

  list_values=list()
  list_values[["V"]]=list()
  list_values[["D"]]=list()
  list_values[["J"]]=list()
  for (com in c(1:length(vect_V_g))) {

    list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com],
                                                   split="*",
                                                   fixed=T)[[1]][1],
                                          split="/")[[1]][1],
                                 split="-")[[1]][1]]]= sort(unique(c(list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_V_g[com])))
    list_values[["D"]][[strsplit(strsplit(strsplit(vect_D_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["D"]][[strsplit(strsplit(strsplit(vect_D_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_D_g[com])))
    list_values[["J"]][[strsplit(strsplit(strsplit(vect_J_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["J"]][[strsplit(strsplit(strsplit(vect_J_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_J_g[com])))
  }

  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=vect_genes_comb,
                                        type_values="VDJ", colorblind=F)),
                      c("#7F837A", "#75A9BF", "#87ACFE", "#87D4FE", "#FFFA79",
                        "#7E8EFE", "#7EFCFE", "#7ED4FE", "#7EACFE", "#7EA9FE"))

  expect_equal(names(Ab_palette(list_values=list_values,
                                vect_genes_comb=vect_genes_comb,
                                type_values="VDJ",
                                colorblind=F)),
               vect_genes_comb)
})




test_that("Ab_palette works for VJ combinations", {

  library(alakazam)
  data("ExampleDb")
  vect_V_g=ExampleDb$v_call[1:10]
  vect_J_g=ExampleDb$j_call[1:10]

  vect_genes_comb=paste(vect_V_g,
                        vect_J_g,
                        sep="_")
  ##Example: Homsap IGHV3-11*05 F_Homsap IGHJ4*03 F

  list_values=list()
  list_values[["V"]]=list()
  list_values[["J"]]=list()
  for (com in c(1:length(vect_V_g))) {
    list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_V_g[com])))
    list_values[["J"]][[strsplit(strsplit(strsplit(vect_J_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["J"]][[strsplit(strsplit(strsplit(vect_J_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_J_g[com])))
  }

  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=vect_genes_comb,
                                        type_values="VJ",
                                        colorblind=F)),
                      c( "#7F7A8E", "#757ABF", "#877AF8", "#877AF8", "#FF7A83",
                         "#7E7AF8", "#7E7AF8", "#7E7AF8", "#7E7AF8", "#7E7AF8"))

  expect_equal(names(Ab_palette(list_values=list_values,
                                vect_genes_comb=vect_genes_comb,
                                type_values="VJ",
                                colorblind=F)),
               vect_genes_comb)
})

test_that("Ab_palette works for V (ergo also D or J) gene sets", {

  library(alakazam)
  data("ExampleDb")
  vect_V_g=ExampleDb$v_call[1:10]


  list_values=list()
  list_values[["V"]]=list()
  for (com in c(1:length(vect_V_g))) {
    list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_V_g[com])))
  }

  expect_colors_equal(unname(Ab_palette(list_values=list_values[["V"]],
                                        vect_genes_comb=NA,
                                        type_values="V",
                                        colorblind=F)),
                      c("#7FAA7F","#7F9F7F", "#7F8A7F", "#7FA97F","#4DFFFF" ))

  expect_equal(names(Ab_palette(list_values=list_values[["V"]],
                                vect_genes_comb=NA,
                                type_values="V",
                                colorblind=F)),
               unname(unlist(list_values[["V"]])))
})

test_that("Ab_palette works for quantitative", {
  list_values=c(1,10,4,5,0)
  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=NA,
                                        type_values="cuantitative",
                                        colorblind=F)),
                      c("#D1495B", "#D55958", "#DA6A55", "#DF7C52", "#E38C4F",
                        "#E89D4C", "#EDAE49", "#C0A95B", "#93A56E", "#66A182"))
})

test_that("Ab_palette works for qualitative", {
  list_values=c("Group_1","Group_2")
  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=NA,
                                        type_values="cualitative",
                                        colorblind=F)),
                      c("#00798c", "#9e51bd"))
})

test_that("Ab_palette colorblind mode works for V(D)J combinations/quantitative/qualitative", {
  list_values=c("Group_1","Group_2")
  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=NA,
                                        type_values="cualitative",
                                        colorblind=T)),
                      c("#440154FF", "#FDE725FF"))

  list_values=c(1,10,4,5,0)
  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=NA,
                                        type_values="cuantitative",
                                        colorblind=T)),
                      c("#440154FF", "#482878FF", "#3E4A89FF", "#31688EFF",
                        "#26828EFF", "#1F9E89FF", "#35B779FF", "#6DCD59FF",
                        "#B4DE2CFF", "#FDE725FF"))


  library(alakazam)
  data("ExampleDb")
  vect_V_g=ExampleDb$v_call[1:10]
  vect_D_g=ExampleDb$d_call[1:10]
  vect_J_g=ExampleDb$j_call[1:10]

  vect_genes_comb=paste(vect_V_g,
                        vect_D_g,
                        vect_J_g,
                        sep="_")
  ##Example: Homsap IGHV3-11*05 F_Homsap IGHD3-10*01 F_Homsap IGHJ4*03 F

  list_values=list()
  list_values[["V"]]=list()
  list_values[["D"]]=list()
  list_values[["J"]]=list()
  for (com in c(1:length(vect_V_g))) {

    list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com],
                                                   split="*",
                                                   fixed=T)[[1]][1],
                                          split="/")[[1]][1],
                                 split="-")[[1]][1]]]= sort(unique(c(list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_V_g[com])))
    list_values[["D"]][[strsplit(strsplit(strsplit(vect_D_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["D"]][[strsplit(strsplit(strsplit(vect_D_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_D_g[com])))
    list_values[["J"]][[strsplit(strsplit(strsplit(vect_J_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["J"]][[strsplit(strsplit(strsplit(vect_J_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_J_g[com])))
  }

  expect_colors_equal(unname(Ab_palette(list_values=list_values,
                                        vect_genes_comb=vect_genes_comb,
                                        type_values="VDJ", colorblind=T)),
                      c("#440154FF", "#482878FF", "#3E4A89FF", "#31688EFF",
                        "#26828EFF",  "#1F9E89FF", "#35B779FF",  "#6DCD59FF",
                        "#B4DE2CFF", "#FDE725FF"))


  vect_V_g=ExampleDb$v_call[1:10]


  list_values=list()
  list_values[["V"]]=list()
  for (com in c(1:length(vect_V_g))) {
    list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]]= sort(unique(c(list_values[["V"]][[strsplit(strsplit(strsplit(vect_V_g[com], split="*", fixed=T)[[1]][1], split="/")[[1]][1], split="-")[[1]][1]]], vect_V_g[com])))
  }

  expect_colors_equal(unname(Ab_palette(list_values=list_values[["V"]],
                                        vect_genes_comb=NA,
                                        type_values="V",
                                        colorblind=T)),
                      c("#440154FF", "#3B528BFF", "#21908CFF",
                        "#5DC863FF", "#FDE725FF"))
})

#' 4_Clonal_exploration
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'
#' @return The return value, if any, from executing the utility.
#'
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
calculate_clone <- function(
    seq_df, clonotype, AA_or_NT = "NT", region = "CDR3", percentage = 100, calculate_shared_clones
) {

    if(clonotype == "Reconstructed_germline"){
      region="Whole"
    }

    if (!(paste(
      "Clone", AA_or_NT, region, clonotype, percentage, if (calculate_shared_clones) {
        "shared"
      }else {"non-shared"}, "simil", sep = "_"
    ) %in%
        colnames(seq_df))) {
        # print("calculate_clones")
        if (clonotype == "VCDR3J") {

            # tmp=paste( seq_df[selected_rows,'V_and_J'],
            # seq_df[selected_rows,paste(AA_or_NT,
            # 'CDR3',sep='_')],sep='_______')

            tmp <- paste(
                seq_df[, get("V_and_J")],
                seq_df[, get(paste(AA_or_NT, "CDR3", sep = "_"))],
                sep = "_______"
            )



        }

        if (clonotype == "Reconstructed_germline") {
            # tmp_selected_rows=sapply(selected_rows, function(z)
            # intersect(which(seq_df$Sequence_type=='Reconstructed_germline'),
            # which(seq_df[,'ID'] %in% seq_df[z,'ID'])) ) tmp=paste(
            # seq_df[tmp_selected_rows,'V_and_J'],
            # seq_df[tmp_selected_rows,paste(AA_or_NT,
            # 'Whole',sep='_')],sep='_______')

            tmp_selected_rows <- sapply(
                c(1:nrow(seq_df)),
                function(z) intersect(
                  which(seq_df$Sequence_type == "Reconstructed_germline"),
                  which(
                    seq_df[, get("ID")] %in%
                      seq_df[z, get("ID")]
                )
              )
            )
            tmp <- paste(
                seq_df[tmp_selected_rows, get("V_and_J")],
                seq_df[tmp_selected_rows, get(paste(AA_or_NT, region, sep = "_"))],
                sep = "_______"
            )

        }

        if (!calculate_shared_clones) {
            tmp <- paste(tmp, seq_df$Patient_Sample, sep = "__")
        }

        seq_df[, paste(
            "Clone", AA_or_NT, region, clonotype, percentage, if (calculate_shared_clones) {
                "shared"
            }else {"non-shared"}, "simil", sep = "_"
        )] <- tmp
        # print("calculate_clones_DONE")
        # print(
        #     paste(
        #         "Clone", AA_or_NT, region, clonotype, percentage, if (calculate_shared_clones) {
        #           "shared"
        #         } else {"non-shared"}, "simil", sep = "_"
        #     )
        # )

    }



    return(seq_df)
}

#' 4_Clonal_exploration
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'
#' @return The return value, if any, from executing the utility.
#' @import plotly
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
draw_violinplots <- function(
    seq_df, group = "Patient_Sample", selected_rows, clonotype, AA_or_NT = "AA", region = "CDR3",
    percentage = 100, freq_filter = 0, Selected_clones = NULL, dominance_threshold, seed=1234,
    really_hide_dots=FALSE, width=1400, height=1000,img_type= "png", scale=4,
    source="clone_violinplot"
) {
    set.seed(seed)
    # devtools::install_github('karthik/wesanderson') pal <-
    # wes_palette(length(unique(seq_df[,..group]))*2, name = 'Zissou1', type =
    # 'continuous')
    fig <- plot_ly(type = "violin") %>%
      config(
        toImageButtonOptions = list(
          format = img_type,
          filename = paste("Violin_plot_clones",clonotype, Sys.time(),sep="_"),
          width = width,
          height = height,
          scale=scale
        )
      )

    i <- 0
    min_value=0
    max_value=0
    for (single_group in unique(seq_df[selected_rows, get(group)])) {

        # subset_seq_df=seq_df[intersect(selected_rows,
        # which(seq_df[,get(group)]==single_group)),]
        # subset_group=unique(seq_df[intersect(selected_rows,
        # which(seq_df[,get(group)]==single_group)), get('Group')])

        og_subset_seq_df <- seq_df[intersect(
            selected_rows, which(
                seq_df[, get(group)] ==
                  single_group
            )
        ),
            ]
        subset_seq_df <- seq_df[intersect(
            which(seq_df$Sequence_type %in% unique(seq_df$Sequence_type[selected_rows])),
            which(
                seq_df[, get(group)] ==
                  single_group
            )
        ),
            ]
        subset_group <- unique(
            seq_df[intersect(
                which(seq_df$Sequence_type %in% unique(seq_df$Sequence_type[selected_rows])),
                which(
                  seq_df[, get(group)] ==
                    single_group
              )
            ),
                get("Group")]
        )
        # print(single_group)


        for (seq_type in c("Repertoire", "Reconstructed_germline")) {
            i <- i + 1
            # print(seq_type)
            subsubset_seq_df <- subset_seq_df[which(subset_seq_df$Sequence_type == seq_type),
                ]
            og_subsubset_seq_df <- og_subset_seq_df[which(og_subset_seq_df$Sequence_type == seq_type),
                ]
            if (nrow(subsubset_seq_df) >
                0) {

                # if (seq_type == "Reconstructed_germline") {
                #   print(subsubset_seq_df)
                # }
                tmp_clones <- table(subsubset_seq_df[, get(clonotype)])

                tmp_clones=tmp_clones[order(names(tmp_clones))]


                tmp_subclones <- table(
                  paste(
                    subsubset_seq_df[, get(clonotype)],
                    subsubset_seq_df[, get("NT_Whole")],
                    sep = "_&_"
                )


              )
                tmp_subclones_num=strsplit(names(tmp_subclones), split="_&_")
                tmp_subclones_num=unlist(tmp_subclones_num)[2*(1:length(tmp_subclones))-1]
                tmp_subclones_num=table(tmp_subclones_num)
                tmp_subclones_num=tmp_subclones_num[order(names(tmp_subclones_num))]

                tmp_frequencies <- data.frame(
                  Clone_ID = names(tmp_clones),
                  Frequency = (100 * as.numeric(tmp_clones))/(sum(tmp_clones)),
                #   Number_of_unique_subclones = sapply(
                #     names(tmp_clones),
                #     function(z) length(
                #       which(
                #         startsWith(
                #           names(tmp_subclones),
                #           paste(z, "_&_", sep = "")
                #       )
                #     )
                #   )
                # ),
                Number_of_unique_subclones = as.numeric(unname(tmp_subclones_num)),
                  Number_of_sequences = as.numeric(tmp_clones)
              )
                tmp_frequencies <- tmp_frequencies[which(tmp_frequencies$Clone_ID %in% unique(og_subsubset_seq_df[, get(clonotype)])),
                  ]

                max_value=max(max_value, log10(max(tmp_frequencies$Frequency)))
                min_value=min(min_value, log10(min(tmp_frequencies$Frequency)))

                # print(Selected_clones$key)
                # if (any(tmp_frequencies$Clone_ID %in% Selected_clones$key)) {
                #   print("Selected clones:")
                #   print(
                #     tmp_frequencies$Clone_ID[(which(tmp_frequencies$Clone_ID %in% Selected_clones$key))]
                # )
                # }
                # if (any(tmp_frequencies$Clone_ID %in% Selected_clones$key)) {
                #   print(
                #     list(
                #       (which(tmp_frequencies$Clone_ID %in% Selected_clones$key) -
                #         1)
               #    )
               #  )
               #  }
                # if (any(tmp_frequencies$Clone_ID %in% Selected_clones$key)) {
                #   print("YESSSCLONES")
                # }
                # print("MMmmmMMm?")

                # FIX #### writing tmp_frequencies to a file and checking later

                # saving also everything in a RData???
                fig <- add_trace(
                  fig, x = paste(
                    single_group, " (", paste(subset_group, collapse = "&"),
                    ")", sep = ""
                ),
                  y = tmp_frequencies$Frequency[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))],
                  hoveron = "points", text = paste(
                    tmp_frequencies$Clone_ID[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))],
                    " Number of unique Fab regions=", tmp_frequencies$Number_of_unique_subclones[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))],
                    " Number of sequences=", tmp_frequencies$Number_of_sequences[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))],
                    sep = ""
                ),
                  key = tmp_frequencies$Clone_ID[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))],
                  legendgroup = seq_type, scalegroup = single_group, selectedpoints = if (any(
                    tmp_frequencies$Clone_ID[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))] %in%
                      Selected_clones$key
                )) {
                    ((which(
                      tmp_frequencies$Clone_ID[which(!(tmp_frequencies$Clone_ID %in% Selected_clones$key))] %in%
                        Selected_clones$key
                  ) -
                      1))
                  } else {
                    NULL
                  }, name = seq_type, side = if (seq_type == "Reconstructed_germline") {
                    "positive"
                  } else {
                    "negative"
                  }, box = list(visible = TRUE, line = list(color = "#2D2926")),
                  points = if(really_hide_dots){FALSE}else{"all"}, pointpos = if (seq_type == "Reconstructed_germline") {
                    0.5
                  } else {
                    -0.5
                  }, jitter = 0.25, scalemode = "count", meanline = list(visible = T),
                  color = I(
                    if (i%%2 != 0) {
                      "#E85D75"
                    } else {
                      "#707A6C"
                    }
                ),
                  marker = list(
                    opacity = 0.75, line = list(
                      width = 2, color = if (i%%2 != 0) {
                        "#E85D75"
                      } else {
                        "#707A6C"
                      }
                  )
                ),
                  unselected = list(
                    marker = list(
                      size = 2, opacity = 0.1, color = if (i%%2 != 0) {
                        "#E85D75"
                      } else {
                        "#707A6C"
                      }
                  )
                ),
                  selected = list(marker = list(size = 10, opacity = 1, color = "#FFA500", zorder = 2)),
                  showlegend = if (i > 2) {
                    F
                  } else {
                    T
                  }
              )

                if (length(which(tmp_frequencies$Clone_ID %in% Selected_clones$key)) !=
                  0) {

                  fig <- add_trace(
                    fig, x = paste(
                      single_group, " (", paste(subset_group, collapse = "&"),
                      ")", sep = ""
                  ),
                    y = tmp_frequencies$Frequency[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)],
                    hoveron = "points", text = if (length(which(tmp_frequencies$Clone_ID %in% Selected_clones$key)) ==
                      0) {
                      NULL
                    } else {
                      paste(
                        tmp_frequencies$Clone_ID[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)],
                        " Number of unique Fab regions=", tmp_frequencies$Number_of_unique_subclones[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)],
                        " Number of sequences=", tmp_frequencies$Number_of_sequences[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)],
                        sep = ""
                    )
                    }, key = tmp_frequencies$Clone_ID[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)],
                    legendgroup = seq_type, scalegroup = single_group, selectedpoints = if (any(
                      tmp_frequencies$Clone_ID[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)] %in%
                        Selected_clones$key
                  )) {
                      ((which(
                        tmp_frequencies$Clone_ID[which(tmp_frequencies$Clone_ID %in% Selected_clones$key)] %in%
                          Selected_clones$key
                    ) -
                        1))
                    } else {
                      NULL
                    }, name = seq_type, side = if (seq_type == "Reconstructed_germline") {
                      "positive"
                    } else {
                      "negative"
                    }, box = list(visible = TRUE, line = list(color = "#2D2926")),
                    points = if(really_hide_dots){FALSE}else{"all"}, pointpos = if (seq_type == "Reconstructed_germline") {
                      0.2
                    } else {
                      -0.2
                    }, jitter = 0.03, scalemode = "count", meanline = list(visible = T),
                    color = I(
                      if (i%%2 != 0) {
                        "#E85D75"
                      } else {
                        "#707A6C"
                      }
                  ),
                    marker = list(
                      opacity = 0.75, size=10,zorder = 2,
                      line = list(
                        width = 4,color = "#FFA500"
                      )
                  ),
                  #   unselected = list(
                  #     marker = list(
                  #       size = 2, opacity = 0.1, color = if (i%%2 != 0) {
                  #         "#A8577E"
                  #       } else {
                  #         "#707A6C"
                  #       }
                  #   )
                  # ),
                  #   selected = list(marker = list(size = 10, opacity = 1, color = "#000000", zorder = 2)),
                    showlegend = FALSE
                )
                }
            }


        }

    }

    fig <- layout(
        fig, title = paste(
            c(
                "Relative log10 frequency distribution of clones<br><i>using the criteria ",
                clonotype, "</i>"
            ),
            collapse = ""
        ),
        yaxis = list(
          title = "Log10 clonal frequency",
          type = "log", exponentformat = "power", showexponent = "all",
          range = c( min_value-0.1,
                    max(2, max_value+0.1))
        ),
        # violingap = 0, violingroupgap = 0, violinmode = "overlay",
        legend = list(tracegroupgap = 0),
        dragmode = "lasso", shapes = list(hline((dominance_threshold))),
        xaxis = list(showline = TRUE,
                     zeroline = FALSE),
        yaxis = list(showline = TRUE,
                     zeroline = FALSE)
    ) %>%
        config(
            displaylogo = FALSE, modeBarButtonsToRemove = c("autoScale2d", "pan2d", "hoverCompareCartesian")
        )


    fig <- suppressWarnings(fig %>%
        layout(plot_bgcolor = "rgba(0, 0, 0, 0)", paper_bgcolor = "rgba(0, 0, 0, 0)",
               xaxis = list(showline = TRUE,
                            zeroline = FALSE),
               yaxis = list(showline = TRUE,
                            zeroline = FALSE)) %>%
        toWebGL())

    return(fig)
}

#' 4_Clonal_exploration
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'
#' @return The return value, if any, from executing the utility.
#' @import upsetjs
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
draw_upsetplot <- function(
    seq_df, group = "Patient", selected_rows, clonotype, AA_or_NT = "AA", region = "CDR3",
    percentage = 100, freq_filter = 0, Selected_clones = NULL
) {

    set_list <- list()

    for (single_group in unique(seq_df[, get(group)])) {
        subset_seq_df <- seq_df[intersect(
            selected_rows, which(
                seq_df[, get(group)] ==
                  single_group
            )
        ),
            ]
        subset_group <- unique(
            seq_df[intersect(
                selected_rows, which(
                  seq_df[, get(group)] ==
                    single_group
              )
            ),
                get("Group")]
        )
        # print(single_group)
        subsubset_seq_df <- subset_seq_df[which(subset_seq_df$Sequence_type == "Repertoire"),
            ]
        tmp_clones <- table(subsubset_seq_df[, get(clonotype)])
        set_list[[length(set_list) +
            1]] <- unique(names(tmp_clones))

    }

    names(set_list) <- unique(seq_df[, get(group)])



    upset_plot <- upsetjs::upsetjs() %>%
        upsetjs::fromList(set_list) %>%
        chartStyleFlags(export.buttons =T)

    return(upset_plot)
}

#' 4_Clonal_exploration
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#'
#' @noRd
hline <- function(y = 0, color = "#2D2926") {
    list(
        type = "line", x0 = 0, x1 = 100, xref = "paper", y0 = y, y1 = y, line = list(color = color, width = 0.75, dash = "dot")
    )
}

#' 4_Clonal_exploration
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#'
#' @noRd
vline <- function(x = 0, color = "#2D2926") {
    list(
        type = "line", x0 = x, x1 = x, xref = "paper", y0 = 0, y1 = 100, line = list(color = color, width = 0.75, dash = "dot")
    )
}

#' 4_Clonal_exploration
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'
#' @return The return value, if any, from executing the utility.
#' @import plotly
#' @import utils
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
draw_sharedclonesplot <- function(
    seq_df, sets = NULL, group = "Patient", selected_rows, clonotype, AA_or_NT = "AA",
    region = "CDR3", percentage = 100, freq_filter = 0, Selected_clones = NULL, dominance_threshold,
    color_by="Clonal sharing", colorblind, Big_mem_color_values,
    seed=1234, width=1400, height=1000,img_type= "png",  scale=4
) {
    set.seed(seed)
    list_figs <- list()
    set_vector <- c()
    Shared_intersected_clones=c()

    if(color_by!= "Clonal sharing" && color_by !="VJ usage") {
      colors <- AbSolution::Ab_palette(
        list_values = unique(seq_df[,get(color_by)]),
        vect_genes_comb = NA, type_values = "cualitative", colorblind = colorblind,
        seed=seed
      )
      names(colors)=unique(seq_df[,get(color_by)])
    }

    for (group_A in unlist(sets)) {
      for (group_B in unlist(sets)) {
        if (group_A != group_B) {
          tmp_comparison <- paste(
            sort(c(group_A, group_B)),
            collapse = "    vs    "
          )
          if (!(tmp_comparison %in% set_vector)) {
            set_vector <- c(set_vector, tmp_comparison)
            group1 <- sort(c(group_A, group_B))[1]
            group2 <- sort(c(group_A, group_B))[2]
            subset1_seq_df <- seq_df[intersect(
              selected_rows, which(
                seq_df[, get(group)] ==
                  group1
              )
            ),
            ]
            subset1_seq_df <- subset1_seq_df[which(subset1_seq_df$Sequence_type == "Repertoire"),
            ]
            subset2_seq_df <- seq_df[intersect(
              selected_rows, which(
                seq_df[, get(group)] ==
                  group2
              )
            ),
            ]
            subset2_seq_df <- subset2_seq_df[which(subset2_seq_df$Sequence_type == "Repertoire"),
            ]

            if(length(Shared_intersected_clones)==0) {
              Shared_intersected_clones <- intersect(unique(subset1_seq_df[, get(clonotype)]),
                                                     unique(subset2_seq_df[, get(clonotype)]))
            } else {
              Shared_intersected_clones <- intersect(Shared_intersected_clones,
                                                     intersect(unique(subset1_seq_df[, get(clonotype)]),
                                                               unique(subset2_seq_df[, get(clonotype)])))
            }


          }
        }
      }}

    set_vector <- c()
    first_iteration=T
    for (group_A in unlist(sets)) {
        for (group_B in unlist(sets)) {
            if (group_A != group_B) {
                tmp_comparison <- paste(
                  sort(c(group_A, group_B)),
                  collapse = "    vs    "
              )
                if (!(tmp_comparison %in% set_vector)) {
                  set_vector <- c(set_vector, tmp_comparison)
                  group1 <- sort(c(group_A, group_B))[1]
                  group2 <- sort(c(group_A, group_B))[2]
                  subset1_seq_df <- seq_df[intersect(
                    selected_rows, which(
                      seq_df[, get(group)] ==
                        group1
                  )
                ),
                    ]
                  subset1_seq_df <- subset1_seq_df[which(subset1_seq_df$Sequence_type == "Repertoire"),
                    ]
                  subset2_seq_df <- seq_df[intersect(
                    selected_rows, which(
                      seq_df[, get(group)] ==
                        group2
                  )
                ),
                    ]
                  subset2_seq_df <- subset2_seq_df[which(subset2_seq_df$Sequence_type == "Repertoire"),
                    ]

                  tmp_clones1 <- table(subset1_seq_df[, get(clonotype)])

                  tmp_frequencies1 <- data.frame(
                    Clone_ID = names(tmp_clones1),
                    Frequency = (100 * as.numeric(tmp_clones1))/((nrow(subset1_seq_df))),
                    Number_of_subclones = as.numeric(tmp_clones1)
                )

                  tmp_clones2 <- table(subset2_seq_df[, get(clonotype)])

                  tmp_frequencies2 <- data.frame(
                    Clone_ID = names(tmp_clones2),
                    Frequency = (100 * as.numeric(tmp_clones2))/((nrow(subset2_seq_df))),
                    Number_of_subclones = as.numeric(tmp_clones2)
                )

                  tmp_summary_df <- merge(tmp_frequencies1, tmp_frequencies2, by = "Clone_ID", all = T)
                  tmp_summary_df[(is.na(tmp_summary_df))] <- 0



                  tmp_summary_df$Freq_X <- sapply(
                    tmp_summary_df$Frequency.x, function(z) if (z ==
                      0) {
                      5e-04
                    } else {
                      z
                    }
                )
                  tmp_summary_df$Freq_Y <- sapply(
                    tmp_summary_df$Frequency.y, function(z) if (z ==
                      0) {
                      5e-04
                    } else {
                      z
                    }
                )

                  tmp_vector <- tmp_summary_df$Freq_X
                  for (tmp_value in unique(tmp_vector[which(duplicated(tmp_vector))])) {
                    tmp_summary_df$Freq_X[which(tmp_vector == tmp_value)] <- jitter(tmp_summary_df$Freq_X[which(tmp_vector == tmp_value)])
                  }

                  tmp_vector <- tmp_summary_df$Freq_Y
                  for (tmp_value in unique(tmp_vector[which(duplicated(tmp_vector))])) {
                    tmp_summary_df$Freq_Y[which(tmp_vector == tmp_value)] <- jitter(tmp_summary_df$Freq_Y[which(tmp_vector == tmp_value)])
                  }
                  tmp_summary_df$info <- paste(
                    paste(
                      tmp_summary_df$Clone_ID, paste(tmp_summary_df$Frequency.x, tmp_summary_df$Frequency.y, sep = " , "),
                      sep = " ("
                  ),
                    ")", sep = ""
                )
                  if(color_by == "Clonal sharing") {
                    tmp_summary_df$shared <- sapply(
                      1:nrow(tmp_summary_df),
                      function(z) if (tmp_summary_df$Frequency.x[z] !=
                                      0 && tmp_summary_df$Frequency.y[z] != 0) {
                        if(tmp_summary_df$Clone_ID[z] %in% Shared_intersected_clones) {
                          "Shared - intersection"
                        } else {
                          "Shared"
                        }

                      } else {
                        "Unique"
                      }
                    )
                  } else {
                    if (color_by =="VJ usage"){
                      columna_color="V_and_J"
                    } else {
                      columna_color=color_by
                    }
                    # tmp_summary_df$shared <- sapply(
                    #   tmp_summary_df$Clone_ID,
                    #   function(z) names(which.max(table(c(subset1_seq_df[subset1_seq_df[, get(clonotype)]==z, get(columna_color)],
                    #                                       subset2_seq_df[subset2_seq_df[, get(clonotype)]==z, get(columna_color)]))))
                    # )

                    DT1 <- data.table::as.data.table(subset1_seq_df)[, .(Clone_ID = get(clonotype), color = get(columna_color))]
                    DT2 <- data.table::as.data.table(subset2_seq_df)[, .(Clone_ID = get(clonotype), color = get(columna_color))]

                    DT <- data.table::rbindlist(list(DT1, DT2), use.names = TRUE)
                    DT <- DT[!is.na(color)]

                    cnt <- DT[, .N, by = .(Clone_ID, color)]

                    best <- cnt[order(Clone_ID, -N)][, .SD[1], by = Clone_ID]
                    tmp_summary_df$shared <- best$color[match(tmp_summary_df$Clone_ID, best$Clone_ID)]
                  }

                  # tmp_summary_df$selected <- rep("#2D2926", nrow(tmp_summary_df))
                  # tmp_summary_df$selected[which(startsWith(tmp_summary_df$shared, "Shared"))] <- "#E63946"
                  # tmp_summary_df$selected[which(tmp_summary_df$Clone_ID %in% Selected_clones)] <- "#FFBC0A"
                  tmp_summary_df$selected <- rep("rgba(0,0,0,0)", nrow(tmp_summary_df))
                  # tmp_summary_df$selected[which(startsWith(tmp_summary_df$shared, "Shared"))] <- "#E63946"
                  tmp_summary_df$selected[which(tmp_summary_df$Clone_ID %in% Selected_clones)] <- "#FFBC0A"


                  # if(first_iteration) {print("Iteration first")} else {print("No first")}

                  tmp_summary_df$shared <- factor(tmp_summary_df$shared, levels = sort(unique(tmp_summary_df$shared)))


                  # tmp_summary_df$selected
                  # tmp_summary_df$selected[which(tmp_summary_df$Clone_ID %in% Selected_clones)] <- "Selected"
                  # tmp_summary_df$selected <- factor(tmp_summary_df$selected, levels = c("Unselected", "Selected"))
                  # print(head(tmp_summary_df))
                  tmp_fig <- plot_ly()

                  for (sub in unique(tmp_summary_df$shared)){

                    color_plot=if(color_by=="Clonal sharing"){
                            if (sub=="Unique") {
                              "#2D2926"
                            } else if ( sub=="Shared - intersection") {
                              "#FFBC0A"
                            } else {
                              "#E63946"
                            }
                    } else if (color_by=="VJ usage") {
                      Big_mem_color_values$VJ[which(names(Big_mem_color_values$VJ)==sub)]
                    } else {
                      colors[which(names(colors)==sub)]
                    }
                    tmp_fig <- add_trace(tmp_fig,
                                         data = tmp_summary_df[which(tmp_summary_df$shared==sub),],
                                         type="scattergl", mode="markers",
                                         x = ~Freq_X, y = ~Freq_Y, text = ~info,
                                         hoverinfo = "text",
                                         key = tmp_summary_df$Clone_ID[which(tmp_summary_df$shared==sub)],
                                         legendgroup=sub,name=sub,
                                         showlegend = if(first_iteration) {T} else {F},
                                         marker = list(size = 15,opacity = 0.75,
                                                       color=color_plot,
                                                       line = list(width = 3,
                                                                   color = ~selected)))
                  }
                  tmp_fig%>%
                    config(
                      toImageButtonOptions = list(
                        format = img_type,
                        filename = paste("Shared_plot",clonotype,Sys.time(),sep="_"),
                        width = width,
                        height = height,
                        scale=scale
                      )
                    )
                  tmp_fig <- layout(
                    tmp_fig, xaxis = list(
                      title = paste("Log10 clonal frequency", group1, sep = " - "),
                      type = "log", exponentformat = "power",
                      range = c(min(-5, log10(min(tmp_summary_df$Freq_X))-0.1),
                                max(2, log10(max(tmp_summary_df$Freq_X))+0.1))
                  ),
                    yaxis = list(
                      title = paste("Log10 clonal frequency", group2, sep = " - "),
                      type = "log", exponentformat = "power", showexponent = "all",
                      range = c(min(-5, log10(min(tmp_summary_df$Freq_Y))-0.1),
                                max(2, log10(max(tmp_summary_df$Freq_Y)+0.1)))
                  ),
                    shapes = list(
                      hline((dominance_threshold)),
                      vline((dominance_threshold))
                  ),
                  xaxis = list(showline = TRUE,
                               zeroline = FALSE),
                  yaxis = list(showline = TRUE,
                               zeroline = FALSE)
                )
                  list_figs[[length(list_figs) +
                    1]] <- tmp_fig

                  first_iteration=F
                }
            }
        }


    }

    sharedclonesplot <- subplot(
        list_figs, nrows = ceiling(length(list_figs)/3),
        margin = 0.05, titleY = TRUE, titleX = TRUE
    )%>%
      config(
        toImageButtonOptions = list(
          format = img_type,
          filename = paste("Shared_clones_plot",Sys.time(),sep="_"),
          width = width,
          height = height,
          scale=scale
        )
      ) %>%
      layout( paper_bgcolor = "transparent",
              xaxis = list(showline = TRUE,
                           zeroline = FALSE),
              yaxis = list(showline = TRUE,
                           zeroline = FALSE))%>%
      config(
        displaylogo = FALSE,
        modeBarButtonsToRemove = c("zoomIn2d", "zoomOut2d",
                                   "autoScale2d",
                                   "hoverCompareCartesian")
      )

    return(sharedclonesplot)
}

test_that("Clonal grouping works for VCDR3J grouping with non-shared clones", {
  expect_equal(c("VJ1_______A__Pat1","VJ1_______A__Pat2","VJ2_______A__Pat1"),
               AbSolution:::calculate_clone(
                 seq_df=data.table::as.data.table(data.frame(V_and_J=c("VJ1","VJ1","VJ2"),
                                                             AA_CDR3=c("A","A","A"),
                                                             Patient_Sample=c("Pat1","Pat2","Pat1"))),
                 clonotype="VCDR3J",
                 AA_or_NT = "AA",
                 region="CDR3",
                 percentage = 100,
                 calculate_shared_clones = F)$'Clone_AA_CDR3_VCDR3J_100_non-shared_simil')
})


test_that("Clonal grouping works for VCDR3J grouping with shared clones", {
  expect_equal(c("VJ1_______A","VJ1_______A","VJ2_______A"),
               AbSolution:::calculate_clone(
                 seq_df=data.table::as.data.table(data.frame(V_and_J=c("VJ1","VJ1","VJ2"),
                                                             AA_CDR3=c("A","A","A"),
                                                             Patient_Sample=c("Pat1","Pat2","Pat1"))),
                 clonotype="VCDR3J",
                 AA_or_NT = "AA",
                 region="CDR3",
                 percentage = 100,
                 calculate_shared_clones = T)$'Clone_AA_CDR3_VCDR3J_100_shared_simil')
})


test_that("Clonal grouping works for Reconstructed_germline grouping with shared clones", {
  expect_equal(rep(c("VJ1_______B","VJ1_______B","VJ2_______B"), each=2),
               AbSolution:::calculate_clone(
                 seq_df=data.table::as.data.table(data.frame(ID=rep(c("1","2","3"), each = 2),
                                                             Sequence_type=rep(c("Repertoire","Reconstructed_germline"),3),
                                                             V_and_J=rep(c("VJ1","VJ1","VJ2"), each = 2),
                                                             AA_Whole=rep(c("A","B"), 3),
                                                             Patient_Sample=rep(c("Pat1","Pat2","Pat1"), each = 2))),
                 clonotype="Reconstructed_germline",
                 AA_or_NT = "AA",
                 region="Whole",
                 percentage = 100,
                 calculate_shared_clones = T)$'Clone_AA_Whole_Reconstructed_germline_100_shared_simil')
})


test_that("Clonal grouping works for Reconstructed_germline grouping with shared clones", {
  expect_equal(rep(c("VJ1_______B__Pat1","VJ1_______B__Pat2","VJ2_______B__Pat1"), each=2),
               AbSolution:::calculate_clone(
                 seq_df=data.table::as.data.table(data.frame(ID=rep(c("1","2","3"), each = 2),
                                                             Sequence_type=rep(c("Repertoire","Reconstructed_germline"),3),
                                                             V_and_J=rep(c("VJ1","VJ1","VJ2"), each = 2),
                                                             AA_Whole=rep(c("A","B"), 3),
                                                             Patient_Sample=rep(c("Pat1","Pat2","Pat1"), each = 2))),
                 clonotype="Reconstructed_germline",
                 AA_or_NT = "AA",
                 region="Whole",
                 percentage = 100,
                 calculate_shared_clones = F)$'Clone_AA_Whole_Reconstructed_germline_100_non-shared_simil')
})

#' Access files in the current app
#'
#' NOTE: If you manually change your package name in the DESCRIPTION,
#' don't forget to change it here too, and in the config file.
#' For a safer name change mechanism, use the `golem::set_golem_name()` function.
#'
#' @param ... character vectors, specifying subdirectory and file(s)
#' within your package. The default, none, returns the root of the app.
#'
#' @noRd
app_sys <- function(...) {
  system.file(..., package = "AbSolution")
}


#' Read App Config
#'
#' @param value Value to retrieve from the config file.
#' @param config GOLEM_CONFIG_ACTIVE value. If unset, R_CONFIG_ACTIVE.
#' If unset, 'default'.
#' @param use_parent Logical, scan the parent directory for config file.
#' @param file Location of the config file
#'
#' @noRd
get_golem_config <- function(
    value, config = Sys.getenv("GOLEM_CONFIG_ACTIVE", Sys.getenv("R_CONFIG_ACTIVE", "default")),
    use_parent = TRUE, file = app_sys("golem-config.yml")
) {
    config::get(value = value, config = config, file = file, use_parent = use_parent)
}

#' Turn an R list into an HTML list
#'
#' @param list An R list
#' @param class a class for the list
#'
#' @return an HTML list
#' @noRd
#'
#' @examples
#' list_to_li(c('a', 'b'))
#' @importFrom shiny tags tagAppendAttributes tagList
list_to_li <- function(list, class = NULL) {
    if (is.null(class)) {
        tagList(lapply(list, tags$li))
    } else {
        res <- lapply(list, tags$li)
        res <- lapply(
            res, function(x) {
                tagAppendAttributes(x, class = class)
            }
        )
        tagList(res)
    }
}
#' Turn an R list into corresponding HTML paragraph tags
#'
#' @param list an R list
#' @param class a class for the paragraph tags
#'
#' @return An HTML tag
#' @noRd
#'
#' @examples
#' list_to_p(c('This is the first paragraph', 'this is the second paragraph'))
#' @importFrom shiny tags tagAppendAttributes tagList
#'
list_to_p <- function(list, class = NULL) {
    if (is.null(class)) {
        tagList(lapply(list, tags$p))
    } else {
        res <- lapply(list, tags$p)
        res <- lapply(
            res, function(x) {
                tagAppendAttributes(x, class = class)
            }
        )
        tagList(res)
    }
}

#' @importFrom shiny tags tagAppendAttributes tagList
named_to_li <- function(list, class = NULL) {
    if (is.null(class)) {
        res <- mapply(
            function(x, y) {
                tags$li(HTML(sprintf("<b>%s:</b> %s", y, x)))
            }, list, names(list),
            SIMPLIFY = FALSE
        )
        tagList(res)
    } else {
        res <- mapply(
            function(x, y) {
                tags$li(HTML(sprintf("<b>%s:</b> %s", y, x)))
            }, list, names(list),
            SIMPLIFY = FALSE
        )
        res <- lapply(
            res, function(x) {
                tagAppendAttributes(x, class = class)
            }
        )
        tagList(res)
    }
}

#' Remove a tag attribute
#'
#' @param tag the tag
#' @param ... the attributes to remove
#'
#' @return a new tag
#' @noRd
#'
#' @examples
#' a <- shiny::tags$p(src = 'plop', 'pouet')
#' tagRemoveAttributes(a, 'src')
tagRemoveAttributes <- function(tag, ...) {
    attrs <- as.character(list(...))
    for (i in seq_along(attrs)) {
        tag$attribs[[attrs[i]]] <- NULL
    }
    tag
}

#' Hide or display a tag
#'
#' @param tag the tag
#'
#' @return a tag
#' @noRd
#'
#' @examples
#' ## Hide
#' a <- shiny::tags$p(src = 'plop', 'pouet')
#' undisplay(a)
#' b <- shiny::actionButton('go_filter', 'go')
#' undisplay(b)
#' @importFrom shiny tagList
undisplay <- function(tag) {
    # if not already hidden
    if (!is.null(tag$attribs$style) &&
        !grepl("display:\\s+none", tag$attribs$style)) {
        tag$attribs$style <- paste("display: none;", tag$attribs$style)
    } else {
        tag$attribs$style <- "display: none;"
    }
    tag
}

#' @importFrom shiny tagList
display <- function(tag) {
    if (!is.null(tag$attribs$style) &&
        grepl("display:\\s+none", tag$attribs$style)) {
        tag$attribs$style <- gsub("(\\s)*display:(\\s)*none(\\s)*(;)*(\\s)*", "", tag$attribs$style)
    }
    tag
}

#' Hide an elements by calling jquery hide on it
#'
#' @param id the id of the element to hide
#'
#' @noRd
#'
#' @importFrom shiny tags
jq_hide <- function(id) {
    tags$script(sprintf("$('#%s').hide()", id))
}

#' Add a red star at the end of the text
#'
#' Adds a red star at the end of the text
#' (for example for indicating mandatory fields).
#'
#' @param text the HTLM text to put before the red star
#'
#' @return an html element
#' @noRd
#'
#' @examples
#' with_red_star('Enter your name here')
#' @importFrom shiny tags HTML
with_red_star <- function(text) {
    shiny::tags$span(HTML(paste0(text, shiny::tags$span(style = "color:red", "*"))))
}



#' Repeat tags$br
#'
#' @param times the number of br to return
#'
#' @return the number of br specified in times
#' @noRd
#'
#' @examples
#' rep_br(5)
#' @importFrom shiny HTML
rep_br <- function(times = 1) {
    HTML(rep("<br/>", times = times))
}

#' Create an url
#'
#' @param url the URL
#' @param text the text to display
#'
#' @return an a tag
#' @noRd
#'
#' @examples
#' enurl('https://www.thinkr.fr', 'ThinkR')
#' @importFrom shiny tags
enurl <- function(url, text) {
    tags$a(href = url, text)
}

#' Columns wrappers
#'
#' These are convenient wrappers around
#' `column(12, ...)`, `column(6, ...)`, `column(4, ...)`...
#'
#' @noRd
#'
#' @importFrom shiny column
col_12 <- function(...) {
    column(12, ...)
}

#' @importFrom shiny column
col_10 <- function(...) {
    column(10, ...)
}

#' @importFrom shiny column
col_8 <- function(...) {
    column(8, ...)
}

#' @importFrom shiny column
col_6 <- function(...) {
    column(6, ...)
}


#' @importFrom shiny column
col_4 <- function(...) {
    column(4, ...)
}


#' @importFrom shiny column
col_3 <- function(...) {
    column(3, ...)
}


#' @importFrom shiny column
col_2 <- function(...) {
    column(2, ...)
}


#' @importFrom shiny column
col_1 <- function(...) {
    column(1, ...)
}



#' Make the current tag behave like an action button
#'
#' Only works with compatible tags like button or links
#'
#' @param tag Any compatible tag.
#' @param inputId Unique id. This will host the input value to be used
#' on the server side.
#'
#' @return The modified tag with an extra id and the action button class.
#' @noRd
#'
#' @examples
#' if (interactive()) {
#'   library(shiny)
#'
#'   link <- a(href = '#', 'My super link', style = 'color: lightblue;')
#'
#'   ui <- fluidPage(
#'     make_action_button(link, inputId = 'mylink')
#'   )
#'
#'   server <- function(input, output, session) {
#'     observeEvent(input$mylink, {
#'       showNotification('Pouic!')
#'     })
#'   }
#'
#'   shinyApp(ui, server)
#' }
make_action_button <- function(tag, inputId = NULL) {
    # some obvious checks
    if (!inherits(tag, "shiny.tag"))
        stop("Must provide a shiny tag.")
    if (!is.null(tag$attribs$class)) {
        if (grep("action-button", tag$attribs$class)) {
            stop("tag is already an action button")
        }
    }
    if (is.null(inputId) &&
        is.null(tag$attribs$id)) {
        stop(
            "tag does not have any id. Please use inputId to be able to
           access it on the server side."
        )
    }

    # handle id
    if (!is.null(inputId)) {
        if (!is.null(tag$attribs$id)) {
            warning(
                paste(
                  "tag already has an id. Please use input$", tag$attribs$id, "to access it from the server side. inputId will be ignored."
              )
            )
        } else {
            tag$attribs$id <- inputId
        }
    }

    # handle class
    if (is.null(tag$attribs$class)) {
        tag$attribs$class <- "action-button"
    } else {
        tag$attribs$class <- paste(tag$attribs$class, "action-button")
    }
    # return tag
    tag
}


# UNCOMMENT AND USE attachment::att_amend_desc() To use this part of the UI
#' #' Include Content From a File
#' #'
#' #' Load rendered RMarkdown from a file and turn into HTML.
#' #'
#' #' @rdname includeRMarkdown
#' #' @export
#' #'
#' #' @importFrom rmarkdown render
#' #' @importFrom markdown markdownToHTML
#' #' @importFrom shiny HTML
#' includeRMarkdown <- function(path){
#'
#'   md <- tempfile(fileext = '.md')
#'
#'   on.exit(unlink(md),add = TRUE)
#'
#'   rmarkdown::render(
#'     path,
#'     output_format = 'md_document',
#'     output_dir = tempdir(),
#'     output_file = md,quiet = TRUE
#'     )
#'
#'   html <- markdown::markdownToHTML(md, fragment.only = TRUE)
#'
#'   Encoding(html) <- 'UTF-8'
#'
#'   return(HTML(html))
#' }

test_that("Test with_red_star works", {
  expect_s3_class(with_red_star("golem"), "shiny.tag")
  expect_equal(
    as.character(with_red_star("Enter your name here")),
    '<span>Enter your name here<span style="color:red">*</span></span>'
  )
})

test_that("Test list_to_li works", {
  expect_s3_class(list_to_li(c("a", "b")), "shiny.tag.list")
  expect_equal(
    as.character(list_to_li(c("a", "b"))),
    "<li>a</li>\n<li>b</li>"
  )
  expect_equal(
    as.character(list_to_li(c("a", "b"), class = "my_li")),
    '<li class="my_li">a</li>\n<li class="my_li">b</li>'
  )
})

test_that("Test list_to_p works", {
  expect_s3_class(
    list_to_p(c(
      "This is the first paragraph",
      "this is the second paragraph"
    )),
    "shiny.tag.list"
  )
  expect_equal(
    as.character(
      list_to_p(c(
        "This is the first paragraph",
        "this is the second paragraph"
      ))
    ),
    "<p>This is the first paragraph</p>\n<p>this is the second paragraph</p>"
  )
  expect_equal(
    as.character(
      list_to_p(
        c(
          "This is the first paragraph",
          "this is the second paragraph"
        ),
        class = "my_li"
      )
    ),
    '<p class="my_li">This is the first paragraph</p>\n<p class="my_li">this is the second paragraph</p>'
  )
})

test_that("Test named_to_li works", {
  expect_s3_class(named_to_li(list(a = "a", b = "b")), "shiny.tag.list")
  expect_equal(
    as.character(named_to_li(list(a = "a", b = "b"))),
    "<li><b>a:</b> a</li>\n<li><b>b:</b> b</li>"
  )
  expect_equal(
    as.character(named_to_li(list(a = "a", b = "b"), class = "mylist")),
    '<li class="mylist"><b>a:</b> a</li>\n<li class="mylist"><b>b:</b> b</li>'
  )
})

test_that("Test tagRemoveAttributes works", {
  a_with_tag <- shiny::tags$p(src = "plop", "pouet")
  expect_s3_class(a_with_tag, "shiny.tag")
  expect_equal(
    as.character(a_with_tag),
    '<p src="plop">pouet</p>'
  )

  a_without_tag <- tagRemoveAttributes(a_with_tag, "src")
  expect_s3_class(a_without_tag, "shiny.tag")
  expect_equal(
    as.character(a_without_tag),
    "<p>pouet</p>"
  )
})

test_that("Test undisplay works", {
  a <- shiny::tags$p(src = "plop", "pouet")
  expect_s3_class(a, "shiny.tag")
  expect_equal(
    as.character(a),
    '<p src="plop">pouet</p>'
  )
  a_undisplay <- undisplay(a)
  expect_s3_class(a_undisplay, "shiny.tag")
  expect_equal(
    as.character(a_undisplay),
    '<p src="plop" style="display: none;">pouet</p>'
  )

  ## Commented after Shiny 1.12.0 change due to changes in actionButton
  # b <- shiny::actionButton("go_filter", "go")
  # expect_s3_class(b, "shiny.tag")
  # expect_equal(
  #   as.character(b),
  #   '<button id="go_filter" type="button" class="btn btn-default action-button">go</button>'
  # )
  # b_undisplay <- undisplay(b)
  # expect_s3_class(b, "shiny.tag")
  # expect_equal(
  #   as.character(b_undisplay),
  #   '<button id="go_filter" type="button" class="btn btn-default action-button" style="display: none;">go</button>'
  # )

  c <- shiny::tags$p(src = "plop", style = "some_style", "pouet")
  expect_s3_class(c, "shiny.tag")
  expect_equal(
    as.character(c),
    '<p src="plop" style="some_style">pouet</p>'
  )
  c_undisplay <- undisplay(c)
  expect_s3_class(c_undisplay, "shiny.tag")
  expect_equal(
    as.character(c_undisplay),
    '<p src="plop" style="display: none; some_style">pouet</p>'
  )
})

test_that("Test display works", {
  a_undisplay <- shiny::tags$p(src = "plop", "pouet", style = "display: none;")
  expect_s3_class(a_undisplay, "shiny.tag")
  expect_equal(
    as.character(a_undisplay),
    '<p src="plop" style="display: none;">pouet</p>'
  )
  a_display <- display(a_undisplay)
  expect_s3_class(a_display, "shiny.tag")
  expect_equal(
    as.character(a_display),
    '<p src="plop" style="">pouet</p>'
  )
})

test_that("Test jq_hide works", {
  expect_s3_class(jq_hide("golem"), "shiny.tag")
  expect_equal(
    as.character(jq_hide("golem")),
    "<script>$('#golem').hide()</script>"
  )
})

test_that("Test rep_br works", {
  expect_s3_class(rep_br(5), "html")
  expect_equal(
    as.character(rep_br(5)),
    "<br/> <br/> <br/> <br/> <br/>"
  )
})

test_that("Test enurl works", {
  expect_s3_class(enurl("https://www.thinkr.fr", "ThinkR"), "shiny.tag")
  expect_equal(
    as.character(enurl("https://www.thinkr.fr", "ThinkR")),
    '<a href="https://www.thinkr.fr">ThinkR</a>'
  )
})

test_that("Test columns wrappers works", {
  expect_s3_class(col_12(), "shiny.tag")
  expect_s3_class(col_10(), "shiny.tag")
  expect_s3_class(col_8(), "shiny.tag")
  expect_s3_class(col_6(), "shiny.tag")
  expect_s3_class(col_4(), "shiny.tag")
  expect_s3_class(col_3(), "shiny.tag")
  expect_s3_class(col_2(), "shiny.tag")
  expect_s3_class(col_1(), "shiny.tag")

  expect_equal(as.character(col_12()), '<div class="col-sm-12"></div>')
  expect_equal(as.character(col_10()), '<div class="col-sm-10"></div>')
  expect_equal(as.character(col_8()), '<div class="col-sm-8"></div>')
  expect_equal(as.character(col_6()), '<div class="col-sm-6"></div>')
  expect_equal(as.character(col_4()), '<div class="col-sm-4"></div>')
  expect_equal(as.character(col_3()), '<div class="col-sm-3"></div>')
  expect_equal(as.character(col_2()), '<div class="col-sm-2"></div>')
  expect_equal(as.character(col_1()), '<div class="col-sm-1"></div>')
})

## Commented after Shiny 1.12.00 change to action_button
# test_that("Test make_action_button works", {
#   tmp_tag <- a(href = "#", "My super link", style = "color: lightblue;")
#   button <- make_action_button(
#     tmp_tag,
#     inputId = "mylink"
#   )
#   expect_s3_class(button, "shiny.tag")
#   expect_equal(
#     as.character(button),
#     '<a href="#" style="color: lightblue;" id="mylink" class="action-button">My super link</a>'
#   )
#   expect_error(
#     button_2 <- make_action_button(
#       unclass(tmp_tag),
#       inputId = "mylink_2"
#     )
#   )
#   expect_error(
#     button_3 <- make_action_button(
#       button,
#       inputId = "mylink_3"
#     )
#   )
#   expect_error(
#     button_4 <- make_action_button(
#       tmp_tag,
#       inputId = NULL
#     )
#   )
#   tmp_tag_2 <- tmp_tag
#   tmp_tag_2$attribs$id <- "id_already_present"
#   expect_warning(
#     button_5 <- make_action_button(
#       tmp_tag_2,
#       inputId = "mylink_5"
#     )
#   )
#   tmp_tag_3 <- tmp_tag
#   tmp_tag_3$attribs$class <- "class_already_present"
#   button_6 <- make_action_button(
#     tmp_tag_3,
#     inputId = "someID"
#   )
# })

# This file is part of the standard setup for testthat.  It is recommended that you do not modify it.  Where should you
# do additional test configuration?  Learn more about the roles of various files in: *
# https://r-pkgs.org/testing-design.html#sec-tests-files-overview *
# https://testthat.r-lib.org/articles/special-files.html

library(testthat)
library(AbSolution)

test_check("AbSolution")

#' 2_Feature_determination_1
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#' @return The return value, if any, from executing the function.
#' @import dplyr
#' @importFrom Biostrings subseq GENETIC_CODE
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
Feature_1 <- function(path_base, grouping_by) {
    input_path <- paste(path_base, "/1.Files_parsed", sep = "")
    output_path <- paste(path_base, "/2.Feature_determination", sep = "")

    ######## 2. Preparing IMGT index and adding V and J

    IMGT_parsed_index <- data.table::fread(
        file.path(input_path, "IMGT_parsed_index.txt"),
        header = TRUE, na.strings=NULL, sep = "\t"
    )
    IMGT_parsed_index[is.na(IMGT_parsed_index)] = "" ## in case NA strings doesnt work

    prefix_gene <- strsplit(IMGT_parsed_index$V_region[1], split = " ")[[1]][1]
    if (prefix_gene == IMGT_parsed_index$V_region[1]) {
        prefix_gene <- ""
    } else {
        prefix_gene <- paste0(prefix_gene, " ")
    }
    if ("D_region" %in% colnames(IMGT_parsed_index)) {
        IMGT_parsed_index <- IMGT_parsed_index %>%
            dplyr::mutate(Best_V = gsub(prefix_gene, "", .data$V_region)) %>%
            dplyr::mutate(Best_J = gsub(prefix_gene, "", .data$J_region)) %>%
            dplyr::mutate(Best_D = gsub(prefix_gene, "", .data$D_region)) %>%
            # dplyr::mutate(Best_V=gsub(' [[:graph:][:space:]]{1, }$','',Best_V)) %>%
            # What is this for ARHJ mutate(Best_J=gsub('
            # [[:graph:][:space:]]{1, }$','',Best_J)) %>% mutate(Best_D=gsub('
            # [[:graph:][:space:]]{1, }$','',Best_D)) %>%
        dplyr::mutate(Best_V = gsub(",.*", "", .data$Best_V)) %>%
            dplyr::mutate(Best_J = gsub(",.*", "", .data$Best_J)) %>%
            dplyr::mutate(Best_D = gsub(",.*", "", .data$Best_D)) %>%
            dplyr::mutate(V_and_J = paste0(.data$Best_V, "_", .data$Best_J)) %>%
            dplyr::mutate(V_and_D_and_J = paste0(.data$Best_V, "_", .data$Best_D, "_", .data$Best_J))

        IMGT_parsed_index$D_region[which(is.na(IMGT_parsed_index$D_region))] <- "NotAvailable"
        IMGT_parsed_index$D_region[which(IMGT_parsed_index$D_region == "")] <- "NotAvailable"
    } else {
        IMGT_parsed_index <- IMGT_parsed_index %>%
            dplyr::mutate(Best_V = gsub(prefix_gene, "", .data$V_region)) %>%
            dplyr::mutate(Best_J = gsub(prefix_gene, "", .data$J_region)) %>%
            dplyr::mutate(Best_V = gsub(",.*", "", .data$Best_V)) %>%
            dplyr::mutate(Best_J = gsub(",.*", "", .data$Best_J)) %>%
            # mutate(Best_V=gsub(' [[:graph:][:space:]]{1, }$','',Best_V)) %>%
            # mutate(Best_J=gsub(' [[:graph:][:space:]]{1, }$','',Best_J)) %>%
        dplyr::mutate(V_and_J = paste0(.data$Best_V, "_", .data$Best_J))
    }


    ##print("Constructing Gene Usage Table ...")  ###absolute and relative info ########## maybe its better for the Shiny part, also the Best_ thing? not really because we already decided the best V


    # colnames(IMGT_parsed_index)[which(colnames(IMGT_parsed_index)
    # %in%regions)]=NT_regions
    write.table(
        as.data.frame(IMGT_parsed_index),
        file.path(output_path, "IMGT_parsed_index_extended.txt"),
        sep = "\t", row.names = FALSE, quote = FALSE
    )

    return(
        length(
            split(
              data.table::as.data.table(IMGT_parsed_index),
                by = grouping_by
            )
        )
    )
}


#' 2_Feature_determination_2
#'
#' @description
#'    **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#' @return The return value, if any, from executing the function.
#' @import parallel
#' @import foreach
#' @import bigstatsr
#' @import bigparallelr
#' @import stringr
#' @importFrom data.table :=
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
Feature__dataset <- function(
    path_base, DF_to_parse, name_DF_to_parse, FWR1partial, FWR4partial, standard = TRUE
) {
  DF_to_parse[is.na(DF_to_parse)] = "" ## in case NA strings doesnt work

  # To avoid issues
  Sequence_type <- ID <- AA_Whole <- ORFs_match <- ORF <- NULL
  subsetDF <- d_rep <- d_recgerm <- NULL

    if (standard) {
        input_path <- paste(path_base, "/1.Files_parsed", sep = "")
        output_path <- paste(path_base, "/2.Feature_determination", sep = "")
    } else {
        input_path <- path_base
        output_path <- path_base
    }

    regions <- c("FWR1", "CDR1", "FWR2", "CDR2", "FWR3", "CDR3", "FWR4")
    NT_regions <- paste0("NT_", regions)
    AA_regions <- paste0("AA_", regions)

    ######## 3. Subdivide into smaller dfs according group and chain





    ######## 4. Iterate over each smaller df to generate DF

    ### Generating NT_Whole and removing sequences with missing regions (< 2
    ### nts) and strange characters other than std nts

    "%!in%" <- function(x, y) !(x %in% y)



    # DF_to_parse$NT_Whole= apply( DF_to_parse[ , ..NT_regions ] , 1 , paste ,
    # collapse = '' )

    if (FWR1partial) {
        init <- 2
    } else {
        init <- 1
    }

    if (FWR4partial) {
        fini <- length(NT_regions) -
            1
    } else {
        fini <- length(NT_regions)
    }

    tmp_NT_regions <- NT_regions[init:fini]

    DF_to_parse$NT_Whole <- do.call(paste, c(DF_to_parse[, tmp_NT_regions, with = FALSE], sep = ""))
    if (any(duplicated(DF_to_parse[Sequence_type != "Reconstructed_germline"]$ID))) {

        DF_to_parse=DF_to_parse[-(which(duplicated(DF_to_parse[Sequence_type != "Reconstructed_germline"]$ID))),]
    }

    ### Check absent regions (or less than 2 nts so no AA) and strange
    ### characters (there should not be strange chars)

    seqs_with_strange_chars <- which(
        unname(
            sapply(
                toupper(DF_to_parse$NT_Whole), function(x) if (any(
                  unique(strsplit(x, split = "")[[1]]) %!in%
                    c("A", "G", "C", "T")
              )) {
                  T
                } else {
                  F
                }
            )
        )
    )



    seqs_with_missing_regions <- c()
    nt_cutoff_length <- 2
    ignore_FWR1 <- c()
    ignore_FWR4 <- c()
    for (NT_region in NT_regions) {
        if (FWR1partial && NT_region == NT_regions[1]) {
            ignore_FWR1 <- c(
                ignore_FWR1, which(
                  nchar(unlist(DF_to_parse[, NT_region, with = FALSE])) <=
                    nt_cutoff_length
              )
            )
        } else if (FWR4partial && NT_region == NT_regions[7]) {
            ignore_FWR4 <- c(
                ignore_FWR4, which(
                  nchar(unlist(DF_to_parse[, NT_region, with = FALSE])) <=
                    nt_cutoff_length
              )
            )
        } else {
            seqs_with_missing_regions <- c(
                seqs_with_missing_regions, which(
                  nchar(unlist(DF_to_parse[, NT_region, with = FALSE])) <=
                    nt_cutoff_length
              )
            )

        }
    }

    to_remove <- unique(c(seqs_with_strange_chars, seqs_with_missing_regions))
    if (length(to_remove) >
        0) {
        IDs_to_remove <- DF_to_parse$ID[to_remove]
        message(
            paste0(
                "----Removed ", length(DF_to_parse$ID[to_remove]) *
                  2, " because of ", length(unique(seqs_with_missing_regions)),
                " sequences with short/missing regions and ", length(unique(seqs_with_strange_chars)),
                " with wrong chars"
            )
        )

        if (length(to_remove) >
            0) {
          data.table::fwrite(
                DF_to_parse[to_remove, ], file = paste0(output_path, "/", name_DF_to_parse, "_DISCARDED_bc_length-chars.txt"),
                sep = "\t"
            )
        }
        DF_to_parse <- DF_to_parse[!(ID %in% IDs_to_remove)]
    } else {
      message("----No removed sequences")
    }






    ## Let's try with an iterator
    cl <- parallel::makePSOCKcluster(
        max(
            1, detectCores(logical = FALSE) -
                4
        ),
        outfile = paste(output_path, "/", name_DF_to_parse, "_ORF.outfile.txt", sep = "")
    )
    doParallel::registerDoParallel(cl)

    i_DF_to_parse <- iterors::iteror(DF_to_parse, by = "row")

    parallel::clusterExport(
        cl = cl, unclass(
            lsf.str(
                envir = asNamespace("AbSolution"),
                all = TRUE
            )
        ),
        envir = as.environment(asNamespace("AbSolution"))
    )
    AA_output <- foreach(
        subsetDF = i_DF_to_parse, .export = c(
            # "NT_regions",
            "nt_changes", "subseq_func", "str_length", "translate_fun"
        ),
        .packages = c("data.table", "Biostrings"),
        .verbose = FALSE
    ) %dopar%
        {
            as.data.frame(
                t(
                  as.data.frame(
                    (unlist(
                      subseq_func(
                        as.character(subsetDF$NT_Whole),
                        IRanges::width(
                          sapply(
                            as.character(subsetDF[, NT_regions, with = FALSE]),
                            function(z) if (z ==
                              "NA" || is.na(z)) {
                              ""
                            } else {
                              z
                            }
                        )
                      ),
                        FWR1partial, FWR4partial, subsetDF$ORF_begins
                    )
                  ))
                )
              )
            )
        }
    parallel::stopCluster(cl)
    gc()
    # AA_output=lapply(AA_output, function(z) as.data.frame(z))
    AA_output <- data.table::rbindlist(AA_output)
    colnames(AA_output) <- c(
        paste0("AA_", c(regions, "Whole")),
        "ORF"
    )

    # write.table(data.frame(A=123), file=file.path(output_path,
    # paste0(name_DF_to_parse, '.sasdadasdexample')), sep='\t', quote =
    # F,row.names =F,col.names =T )

    # start.time <- Sys.time() for (j in c(1:nrow(DF_to_parse))) { if(j == 1) {
    # test=unlist(subseq_func(as.character(DF_to_parse[j, 'NT_Whole']),
    # width(sapply(as.character(DF_to_parse[j,..NT_regions]), function(z)
    # if(z=='NA'){''}else{z})),FWR1partial,FWR4partial )) } else {
    # test=rbind(test, unlist(subseq_func(as.character(DF_to_parse[j,
    # 'NT_Whole']), width(sapply(as.character(DF_to_parse[j,..NT_regions]),
    # function(z) if(z=='NA'){''}else{z})),FWR1partial,FWR4partial ))) } }
    # end.time <- Sys.time() time.taken_Linear <- end.time - start.time
    # print(time.taken_Linear)

    DF_to_parse <- cbind(DF_to_parse, AA_output)
    #print("Removing no ORF")
    message(
        paste0(
            "Removed ", length(unique(DF_to_parse[AA_Whole == "NO_GOOD_ORF"]$ID)) *
                2, " sequences because ", length(which(DF_to_parse$AA_Whole == "NO_GOOD_ORF")),
            " (", nrow(
                DF_to_parse[AA_Whole == "NO_GOOD_ORF"][Sequence_type != "Reconstructed_germline"]
            ),
            " from repertoire)", " had no good ORF"
        )
    )
    if (length(which(DF_to_parse$AA_Whole == "NO_GOOD_ORF")) >
        0) {
      data.table::fwrite(
            DF_to_parse[AA_Whole == "NO_GOOD_ORF"], file = paste0(output_path, "/", name_DF_to_parse, "_DISCARDED_bc_ORF.txt"),
            sep = "\t"
        )
    }
    DF_to_parse <- DF_to_parse[which(!(DF_to_parse$ID %in% DF_to_parse[AA_Whole == "NO_GOOD_ORF"]$ID))]

    # Check if same ORF ##in current case ofc not same orf
    data.table::setDT(DF_to_parse)[,
        ORFs_match := c("Not Match", "Match")[(data.table::uniqueN(ORF) ==
            1) + 1], ID]

    DF_to_parse$ORF=as.numeric(DF_to_parse$ORF)
    ######## 5. Iterate over each FBM

    ### Calculate num of ab features that go into the FBM


    #print("Calculating and storing Mutations and  Levenshtein distance...")  ######### its 0 for reconstructed



    ID_tmp <- DF_to_parse$ID[1]

    tmp <- DF_to_parse[ID == ID_tmp]

    regions <- c("FWR1", "CDR1", "FWR2", "CDR2", "FWR3", "CDR3", "FWR4", "Whole")
    NT_regions <- paste0("NT_", regions)
    AA_regions <- paste0("AA_", regions)
    NGly_motif <- "N[^P]{1}[S|T]"
    names(NGly_motif) <- "NGly"
    NGly_motif_pos <- c(1)
    names(NGly_motif_pos) <- names(NGly_motif)
    hot_cold_mtfs <- edited_hot_cold_mtfs()
    mut_points_mtfs <- mut_points_hot_cold_mtfs(hot_cold_mtfs)



    test <- full_analysis(
        tmp, AA_regions, NT_regions, hot_cold_mtfs, mut_points_mtfs, NGly_motif,
        NGly_motif_pos, FWR1partial, FWR4partial
    )


    tmp_example <- (as.matrix(rbind(test[["Matrix"]][["Germ"]], test[["Matrix"]][["Rep"]])))
    colnames(tmp_example) <- names(test$Matrix$Germ)
    write.table(
        tmp_example, file = file.path(output_path, paste0(name_DF_to_parse, ".example")),
        sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE
    )

    # tmp_FBM=FBM(nrow=length(test$Matrix$Rep),ncol=nrow(DF_to_parse),
    # is_read_only = F)
    tmp_FBM <- FBM(
        ncol = length(test$Matrix$Rep),
        nrow = nrow(DF_to_parse),
        is_read_only = FALSE, backingfile = file.path(output_path, name_DF_to_parse)
    )
    # block.size=2*(round(block_size(ncol(tmp_FBM), ncores = nb_cores())/2))

    #print("Feature calculation...")

    # end=0 step_size=20000 tmp_info_df=list() ###reduce DF_to_parse size while
    # ((end + 1) < ncol(tmp_FBM)) { start=end + 1 print(paste(start,
    # paste0(hour(Sys.time()), ':', minute(Sys.time())))) end = end + step_size
    # subset_DF_to_parse=DF_to_parse[c(start:min(end, ncol(tmp_FBM))),]
    # block.size=2*(round(block_size(ncol(subset_DF_to_parse), ncores =
    # nb_cores())/2)) block.size=2*(round(block_size(ncol(DF_to_parse), ncores
    # = nb_cores())/2)) tmp_tmp_info_df= big_applyAb(tmp_FBM, function(X, ind,
    # full_analysis, DF_to_parse,ind_vector, name_DF_to_parse,output_path,
    # AA_regions,
    # NT_regions,hot_cold_mtfs,mut_points_mtfs,NGly_motif,NGly_motif_pos,
    # FWR1partial, FWR4partial) { tmp_df=data.frame(matrix(NA, nrow = nrow(X),
    # ncol = length(ind))) library(data.table) library(bigassertr)
    # source('Scripts/0_Global_shiny.R') position=0 for (ID_tmp in
    # unique(DF_to_parse$ID[ind])) { position=position+1
    # results=full_analysis(DF_to_parse[ID==ID_tmp], AA_regions,
    # NT_regions,hot_cold_mtfs,mut_points_mtfs,NGly_motif,NGly_motif_pos,
    # FWR1partial, FWR4partial) if(position == 1) {
    # results_df=data.frame(matrix(NA, nrow = length(ind), ncol =
    # length(results$DF$Germ))) } tmp_df[, position] <- results$Matrix$Germ
    # results_df[position, ] <- unname(results$DF$Germ) position=position+1
    # tmp_df[, position] <- results$Matrix$Rep results_df[position, ] <-
    # unname(results$DF$Rep) } X[, ind_vector[ind]] <- tmp_df if (1 %in%
    # ind_vector[ind]) { tmp_example=t(as.matrix(tmp_df[,1:2]))
    # colnames(tmp_example)=names(results$Matrix$Germ) write.table(tmp_example,
    # file=file.path(output_path, paste0(name_DF_to_parse, '.example')),
    # sep='\t', quote = F,row.names =F,col.names =T ) } # if (ind %% 1000 == 0)
    # { # fwrite(paste(paste0(hour(Sys.time()), ':', minute(Sys.time())),ind,
    # sep=' '), file = paste0(output_path, '/', name_DF_to_parse,
    # '_feature_outfile.txt'), sep='\t') # }
    # colnames(results_df)=colnames(data.frame(as.list(results$DF$Germ)))
    # results_df }, full_analysis = full_analysis,
    # DF_to_parse=subset_DF_to_parse,
    # name_DF_to_parse=name_DF_to_parse,output_path=output_path,
    # AA_regions=AA_regions,
    # NT_regions=NT_regions,hot_cold_mtfs=hot_cold_mtfs,mut_points_mtfs=mut_points_mtfs,NGly_motif=NGly_motif,NGly_motif_pos=NGly_motif_pos,
    # FWR1partial=FWR1partial, FWR4partial=FWR4partial, a.combine = NULL,
    # ncores = max(1,nb_cores()-4), block.size=block.size,
    # ind_vector=c(start:min(end, ncol(tmp_FBM))), ind=
    # c(1:length(c(start:min(end, ncol(tmp_FBM))))) )
    # #min(ceiling((ncol(tmp_FBM)/block.size)), nb_cores()) print('done')
    # tmp_info_df[[length(tmp_info_df)+1]]=rbindlist(tmp_tmp_info_df)
    # print('rbindlist done') } info_df=rbindlist(tmp_info_df)
    # rm(tmp_tmp_info_df) rm(tmp_info_df) print('Feature calculation... Done')


    cl <- parallel::makePSOCKcluster(
        max(1, detectCores() - 5),
        outfile = paste(output_path, "/", name_DF_to_parse, "_feature_outfile.txt", sep = "")
    )
    doParallel::registerDoParallel(cl)
    parallel::clusterExport(
        cl = cl, unclass(
            lsf.str(
                envir = asNamespace("AbSolution"),
                all = TRUE
            )
        ),
        envir = as.environment(asNamespace("AbSolution"))
    )
    ### perhaps use flock package better to update on-the-go the FBM but still
    ### I would need the DF back
    end <- 0
    step_size <- 2000

    tmp_info_df <- list()
    ### reduce DF_to_parse size
    while ((end + 1) < nrow(tmp_FBM)) {
        start <- end + 1
        #print(
        #     paste(
        #         start, paste0(
        #           hour(Sys.time()),
        #           ":", minute(Sys.time())
        #       )
        #     )
        # )
        end <- end + step_size

        subset_DF_to_parse <- DF_to_parse[c(start:min(end, nrow(tmp_FBM))),
            ]

        i_rep <- iterors::iteror(
            subset_DF_to_parse[seq(
                from = 1, to = nrow(subset_DF_to_parse),
                by = 2
            ),
                ], by = "row"
        )
        i_recgerm <- iterors::iteror(
            subset_DF_to_parse[seq(
                from = 2, to = nrow(subset_DF_to_parse),
                by = 2
            ),
                ], by = "row"
        )

        results <- foreach(
            d_rep = i_rep, d_recgerm = i_recgerm, .export = c("nt_changes", "lv_changes", "length_of_region",
                "count_elements", "locate_motif", "group_freqs", "alakazam_properties",
                "Peptides_properties", "compare_sequences", "instaIndex_improved",
                "full_analysis",
                # "NT_regions", "AA_regions", "hot_cold_mtfs", "mut_points_mtfs",
                # "NGly_motif", "NGly_motif_pos", "FWR1partial", "FWR4partial",
                "stringdist"
            ),
            .packages = c("alakazam", "Peptides", "Biostrings", "BiocGenerics", "data.table", "stringr"),
            .verbose = FALSE
        ) %dopar%
            {
                full_analysis(
                  data.table::as.data.table(rbind(d_rep, d_recgerm)),
                  AA_regions, NT_regions, hot_cold_mtfs, mut_points_mtfs, NGly_motif,
                  NGly_motif_pos, FWR1partial, FWR4partial
              )
            }



        tmp_tmp_info_df <- do.call(
            rbind, lapply(results, function(z) rbind(z[["DF"]][["Germ"]], z[["DF"]][["Rep"]]))
        )

        tmp_info_df[[length(tmp_info_df) +
            1]] <- as.data.frame(tmp_tmp_info_df)

        results_df <- do.call(
            rbind, lapply(
                results, function(z) rbind(
                  unname(z[["Matrix"]][["Germ"]]),
                  unname(z[["Matrix"]][["Rep"]])
              )
            )
        )


        # tmp_FBM[,c(start:min(end, ncol(tmp_FBM)))] = results_df
        tmp_FBM[c(start:min(end, nrow(tmp_FBM))),
            ] <- results_df
        rm(results_df)
    }

    parallel::stopCluster(cl)

    info_df <- data.table::rbindlist(tmp_info_df)

    info_df <- cbind(DF_to_parse, info_df)
    write.table(
        info_df, file = file.path(output_path, paste0(name_DF_to_parse, ".info")),
        sep = "\t", quote = FALSE, row.names = FALSE, col.names = TRUE
    )
    rm(info_df)
    gc()

    # def_FBM <-big_transpose(tmp_FBM, backingfile = file.path(output_path,
    # name_DF_to_parse)) def_FBM$save()
    tmp_FBM$save()
}


#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#'
#' @noRd
full_analysis <- function(
    tmp, AA_regions, NT_regions, hot_cold_mtfs, mut_points_mtfs, NGly_motif, NGly_motif_pos,
    FWR1partial, FWR4partial
) {
    germline_index <- which(tmp$Sequence_type == "Reconstructed_germline")
    repertoire_index <- which(tmp$Sequence_type != "Reconstructed_germline")

    if (length(c(germline_index, repertoire_index)) !=
        2) {
        #print(paste0("[ERROR] More sequences than foreseen with ID ", unique(tmp$ID)))
    }

    list_row_values <- list(Rep = c(), Germ = c())
    list_row_values_df <- list(Rep = c(), Germ = c())
    list_lengths <- list(Rep_NT = c(), Rep_AA = c(), Germ_NT = c(), Germ_AA = c())
    values_not_to_diff <- c()

    for (index in c(repertoire_index, germline_index)) {
        row_values <- c()
        row_values_df <- c()
        length_regions_index <- c()
        for (NT_region in NT_regions) {
            if ((NT_region == "NT_FWR1" && FWR1partial) || (NT_region == "NT_FWR4" &&
                FWR4partial)) {
                orig_NT_region <- NT_region
                NT_region <- "NT_FWR2"
                seq_rec.germline <- (as.character(tmp[germline_index, NT_region, with = FALSE]))
                seq_repertoire <- (as.character(tmp[repertoire_index, NT_region, with = FALSE]))
                sequence <- (as.character(tmp[index, NT_region, with = FALSE]))
                if (index == repertoire_index) {
                  values_not_to_diff <- c(
                    values_not_to_diff, nt_changes("", "", orig_NT_region)
                )
                }
                seq_rec.germline <- toupper(seq_rec.germline)
                seq_repertoire <- toupper(seq_repertoire)
                sequence <- toupper(sequence)
                tmp_partial <- length_of_region(sequence, NT_region)
                tmp_partial_value <- rep(0, length(tmp_partial))
                names(tmp_partial_value) <- gsub(NT_region, orig_NT_region, names(tmp_partial))
                length_regions_index <- c(length_regions_index, tmp_partial_value)

                tmp_partial <- count_elements(sequence, NT_region)
                tmp_partial_value <- rep(0, length(tmp_partial))
                names(tmp_partial_value) <- gsub(NT_region, orig_NT_region, names(tmp_partial))
                count_el <- tmp_partial_value

            } else {
              seq_rec.germline <- (as.character(tmp[germline_index, NT_region, with = FALSE]))
              seq_repertoire <- (as.character(tmp[repertoire_index, NT_region, with = FALSE]))
              sequence <- (as.character(tmp[index, NT_region, with = FALSE]))
                if (index == repertoire_index) {
                  values_not_to_diff <- c(values_not_to_diff, nt_changes(sequence, seq_rec.germline, NT_region))
                }
                seq_rec.germline <- toupper(seq_rec.germline)
                seq_repertoire <- toupper(seq_repertoire)
                sequence <- toupper(sequence)
                length_regions_index <- c(length_regions_index, length_of_region(sequence, NT_region))
                count_el <- count_elements(sequence, NT_region)
            }


            row_values <- c(row_values, length_regions_index[length(length_regions_index)])
            row_values <- c(row_values, count_el)
        }
        if (index == repertoire_index) {
            list_lengths$Rep_NT <- length_regions_index
        } else {
            list_lengths$Germ_NT <- length_regions_index
        }


        sequence <- toupper(as.character(tmp[index, "NT_Whole"]))
        for (motif_i in c(1:length(hot_cold_mtfs))) {
            tmp_results <- locate_motif(
                sequence, length_regions_index, hot_cold_mtfs[motif_i], names(hot_cold_mtfs[motif_i]),
                motif_ini = mut_points_mtfs[which(
                  names(mut_points_mtfs) ==
                    names(hot_cold_mtfs[motif_i])
              )],
                NT_regions
            )
            row_values_df <- c(row_values_df, tmp_results$index_locis)
            row_values <- c(row_values, tmp_results$motif_count, tmp_results$perc_locis)
        }


        length_regions_index <- c()
        for (AA_region in AA_regions) {

            if ((AA_region == "AA_FWR1" && FWR1partial) || (AA_region == "AA_FWR4" &&
                FWR4partial)) {
                orig_AA_region <- AA_region
                AA_region <- "AA_FWR2"
                seq_rec.germline <- as.character(tmp[germline_index, AA_region, with = FALSE ])
                seq_repertoire <- as.character(tmp[repertoire_index, AA_region, with = FALSE ])
                sequence <- as.character(tmp[index, AA_region, with = FALSE ])
                # if (index == repertoire_index) {
                #   values_not_to_diff <- c(values_not_to_diff, aa_changes(seq_repertoire, seq_rec.germline, AA_region))
                # }

                tmp_partial <- length_of_region(sequence, AA_region)
                tmp_partial_value <- rep(0, length(tmp_partial))
                names(tmp_partial_value) <- gsub(AA_region, orig_AA_region, names(tmp_partial))
                length_regions_index <- c(length_regions_index, tmp_partial_value)
                row_values <- c(row_values, length_regions_index[length(length_regions_index)])

                tmp_partial <- c(
                  count_elements(sequence, AA_region),
                  group_freqs(sequence, AA_region),
                  alakazam_properties(sequence, AA_region),
                  Peptides_properties(sequence, AA_region)
              )
                tmp_partial_value <- rep(0, length(tmp_partial))
                names(tmp_partial_value) <- gsub(AA_region, orig_AA_region, names(tmp_partial))

                row_values <- c(row_values, tmp_partial_value)
            } else {
                seq_rec.germline <- as.character(tmp[germline_index, AA_region, with = FALSE ])
                seq_repertoire <- as.character(tmp[repertoire_index, AA_region, with = FALSE ])
                sequence <- as.character(tmp[index, AA_region, with = FALSE ])
                # if (index == repertoire_index) {
                #   values_not_to_diff <- c(values_not_to_diff, aa_changes(seq_repertoire, seq_rec.germline, AA_region))
                # }
                length_regions_index <- c(length_regions_index, length_of_region(sequence, AA_region))
                row_values <- c(row_values, length_regions_index[length(length_regions_index)])
                row_values <- c(row_values, count_elements(sequence, AA_region))
                row_values <- c(row_values, group_freqs(sequence, AA_region))
                row_values <- c(row_values, alakazam_properties(sequence, AA_region))
                row_values <- c(row_values, Peptides_properties(sequence, AA_region))
            }

        }
        if (index == repertoire_index) {
            list_lengths$Rep_AA <- length_regions_index
        } else {
            list_lengths$Germ_AA <- length_regions_index
        }

        sequence <- as.character(tmp[index, "AA_Whole"])
        tmp_results <- locate_motif(
            sequence, length_regions_index, NGly_motif, names(NGly_motif),
            motif_ini = NGly_motif_pos, AA_regions
        )
        row_values_df <- c(row_values_df, tmp_results$index_locis)
        row_values <- c(row_values, tmp_results$motif_count, tmp_results$perc_locis)

        if ((AA_region == "AA_FWR1" && FWR1partial) || (AA_region == "NT_FWR4" &&
            FWR4partial)) {
            row_values[which(is.nan(row_values))] <- 0
        }

        if (index == repertoire_index) {
            list_row_values$Rep <- row_values
            list_row_values_df$Rep <- row_values_df
        } else {
            list_row_values$Germ <- row_values
            list_row_values_df$Germ <- row_values_df
        }
    }



    seq_aa_rec.germline <- as.character(tmp[germline_index, "AA_Whole"])
    seq_aa_repertoire <- as.character(tmp[repertoire_index, "AA_Whole"])
    values_not_to_diff <- c(
        values_not_to_diff, compare_sequences(
            rep_seq = seq_aa_repertoire, germ_seq = seq_aa_rec.germline, mode = "AA",
            aa_rep_lengths = list_lengths$Rep_AA, aa_germ_lengths = list_lengths$Germ_AA,
            AA_regions = AA_regions,
            aant_rep_seq=as.character(tmp[repertoire_index, "NT_Whole"]),
            aant_germ_seq=as.character(tmp[germline_index, "NT_Whole"])
        )
    )

    seq_nt_rec.germline <- (as.character(tmp[germline_index, "NT_Whole"]))
    seq_nt_repertoire <- (as.character(tmp[repertoire_index, "NT_Whole"]))
    values_not_to_diff <- c(
        values_not_to_diff, compare_sequences(
            rep_seq = seq_nt_repertoire, germ_seq = seq_nt_rec.germline, mode = "NT",
            nt_rep_lengths = list_lengths$Rep_NT, nt_germ_lengths = list_lengths$Germ_NT,
            NT_regions = NT_regions
        )
    )

    seq_nt_rec.germline <- (as.character(tmp[germline_index, "NT_Whole"]))
    seq_nt_repertoire <- (as.character(tmp[repertoire_index, "NT_Whole"]))

    codon_analysis <- compare_sequences(
        rep_seq = seq_nt_repertoire, germ_seq = seq_nt_rec.germline, mode = "Codon",
        nt_rep_lengths = list_lengths$Rep_NT, nt_germ_lengths = list_lengths$Germ_NT,
        NT_regions = NT_regions, aa_rep_lengths = list_lengths$Rep_AA, aa_germ_lengths = list_lengths$Germ_AA,
        AA_regions = AA_regions, ORF_rep = tmp$ORF[repertoire_index], ORF_germ = tmp$ORF[germline_index]
    )

    values_not_to_diff <- c(values_not_to_diff, codon_analysis$counts)
    list_row_values$Rep <- c(list_row_values$Rep, codon_analysis$rep_codon_count)
    list_row_values$Germ <- c(list_row_values$Germ, codon_analysis$germ_codon_count)

    diff_tmp <- list_row_values$Rep - list_row_values$Germ
    names(diff_tmp) <- paste0(
        names(diff_tmp),
        "_Diff"
    )
    diff_tmp_ger <- rep(0, length(diff_tmp))
    names(diff_tmp_ger) <- names(diff_tmp)
    list_row_values$Rep <- c(list_row_values$Rep, diff_tmp)
    list_row_values$Germ <- c(list_row_values$Germ, diff_tmp_ger)



    list_row_values$Rep <- c(list_row_values$Rep, values_not_to_diff)
    empty_values_not_to_diff <- rep(0, length(values_not_to_diff))
    names(empty_values_not_to_diff) <- names(values_not_to_diff)
    list_row_values$Germ <- c(list_row_values$Germ, empty_values_not_to_diff)

    return(list(DF = list_row_values_df, Matrix = list_row_values))

}

#' Run the AbSolution Shiny Application
#' Launches the interactive AbSolution Shiny application for exploring
#' B-cell and T-cell receptor repertoire data. The application provides
#' tools for loading AIRR-formatted datasets, inspecting sequence
#' characteristics, visualizing clonotype distributions, and generating
#' summary reports.
#'
#' Internally, `run_app()` calls [`shiny::shinyApp()`] to start the
#' application, using the UI defined in [\code{app_ui()}] and the server
#' logic defined in [\code{app_server()}].
#'
#' This function is the entry point that end-users should call after
#' installing the package.
#'
#' @param verbose Logical. If \code{FALSE} (default), suppresses all warnings
#' and messages during app execution. Set to \code{TRUE} to see full console
#' output for debugging.
#' @param ... arguments to pass to golem_opts.
#' See `?golem::get_golem_options` for more details.
#'
#' @inheritParams shiny::shinyApp
#'
#' @export
#' @importFrom shiny shinyApp
#' @importFrom golem with_golem_options
#' @examples
#' if (interactive()) {
#'   ### Launch the full application with default settings:
#'   run_app()
#' }
run_app <- function(verbose = FALSE, onStart = NULL, options = list(),
                    enableBookmarking = "server", uiPattern = "/", ...) {
  # set.seed(1234)

  with_golem_options(
    app = shinyApp(
      ui = app_ui,
      server = app_server,
      onStart = onStart,
      options = options,
      enableBookmarking = enableBookmarking,
      uiPattern = uiPattern
    ),
    golem_opts = list(verbose = verbose, ...)
  )
}

#' translate_fun
#'
#' @description A function to translate NT sequences into AA sequences and select
#' the optimal translation (if it exists). Prioritizes mclosest ORF to 1.
#'
#' @return A list with either the optimal-without-stop-codon translation and its
#' ORF, or a list indicating there is no a correct translation.
#' @noRd
translate_fun <- function(x, ORF_begins) {

    if (nchar(x)<=2) {
        return(list("", ""))
    } else {
      seq_to_translate= Biostrings::subseq(
         Biostrings::DNAString(as.character(x)),
        start = if(ORF_begins==1){1}else if(ORF_begins==2){3} else if(ORF_begins==0) {2})

      AA_seq=Biostrings::translate(seq_to_translate,
                                   genetic.code= Biostrings::GENETIC_CODE,
                                   no.init.codon=T)

      is_there_stop=sum(length(Biostrings::matchPattern("*", AA_seq)))
      AA_seq=as.character(AA_seq)
      if(AA_seq == ""){
            return(list("", ""))
        } else if (is_there_stop == 0) {
            return(list(AA_seq, if(ORF_begins==1){1}else if(ORF_begins==2){3} else if(ORF_begins==0) {2}))
        } else {
            return(list("NO_GOOD_ORF", 0))  #If the ORF has stop codons. This value is used for filtering afterwards.
        }
        # ORFs <- lapply(
        #     1:3, function(pos) Biostrings::subseq(
        #         Biostrings::DNAString(as.character(x)),
        #         start = pos
        #     )
        # )
        # suppressWarnings(
        #     translated_ORFs <- lapply(
        #         ORFs, Biostrings::translate, genetic.code = Biostrings::GENETIC_CODE,
        #         no.init.codon = TRUE, if.fuzzy.codon = "error"
        #     )
        # )
        #
        # # Search for the best ORF
        # Stop_per_ORF <- vector()
        # for (i in 1:3) {
        #     Stop_per_ORF[i] <- sum(length(Biostrings::matchPattern("*", unlist(translated_ORFs[[i]]))))
        # }
        #
        # # Find STOP codons per ORF and choose the ORF with the less
        # BestORF <- which.min(Stop_per_ORF)
        # aa_seq <- as.character(translated_ORFs[[BestORF]])
        #
        # if(aa_seq == ""){
        #     return(list("", ""))
        # } else if (Stop_per_ORF[BestORF] == 0) {
        #     return(list(aa_seq, BestORF))
        # } else {
        #     return(list("NO_GOOD_ORF", 0))  #If every ORF has stop codons. This value is used for filtering afterwards.
        # }
    }

}

#' Peptides_aaCheck
#'
#' @description Original code from package Peptides, internal function aaCheck.
#' Original authors: Daniel Osorio ORCID iD [aut, cre], Paola Rondon-Villarreal ORCID iD [aut, ths], Rodrigo Torres ORCID iD [aut, ths], J. Sebastian Paez ORCID iD [ctb], Luis Pedro Coelho ORCID iD [ctb], Richèl J.C. Bilderbeek ORCID iD [ctb], Florian C. Sigloch ORCID iD [ctb]
#' Original source: https://github.com/dosorio/Peptides/blob/master/R/aaCheck.R
#' Original citation: Osorio, D., Rondon-Villarreal, P. & Torres, R. Peptides: A package for data mining of antimicrobial peptides. The R Journal. 7(1), 4–14 (2015).
#' Original license: GPL-2 (see https://cran.r-project.org/package=Peptides)
#' @return A list of aminoacids in uppercase
#' @noRd
Peptides_aaCheck <- function (seq)
{
  if (!any(lengths(seq) > 1)) {
    seq <- toupper(seq)
    seq <- gsub(pattern = "[[:space:]]+", replacement = "",
                x = seq)
    seq <- strsplit(x = seq, split = "")
  }
  else {
    seq <- lapply(seq, toupper)
  }
  check <- unlist(lapply(seq, function(sequence) {
    !all(sequence %in% c("A", "C", "D", "E", "F", "G", "H",
                         "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T",
                         "V", "W", "Y", "-"))
  }))
  if (sum(check) > 0) {
    sapply(which(check == TRUE), function(sequence) {
      warning(paste0("Sequence ", sequence, " has unrecognized amino acid types. Output value might be wrong calculated"),
              call. = FALSE)
    })
  }
  return(seq)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import stringr
#' @noRd
subseq_func <- function(NT_seq, size_NT_regions, FWR1partial = FALSE, FWR4partial = FALSE, ORF_begins) {
    ########### TOCAR
    AA_info <- translate_fun(toupper(NT_seq), ORF_begins)
    AA_seq <- AA_info[[1]]
    seq_list <- list()
    ORF <- AA_info[[2]]

    if ((AA_seq == "NO_GOOD_ORF") == TRUE) {
        seq_list[1:(length(size_NT_regions) +
            2)] <- "NO_GOOD_ORF"
    } else {
        # initialize
        if (FWR1partial) {
            size_NT_regions <- size_NT_regions[2:length(size_NT_regions)]
        }
        if (FWR4partial) {
            size_NT_regions <- size_NT_regions[1:c(
                length(size_NT_regions) -
                  1
            )]
        }
        size_NT_regions[1] <- size_NT_regions[1] - ORF + 1

        size_AA_regions=c()
        off_set=0
        for(i in c(1:(length(size_NT_regions)))){
          if((size_NT_regions[i]-off_set)%%3 == 0) {
            size_AA_regions=c(size_AA_regions,( size_NT_regions[i]-off_set)/3)
            off_set=0
          } else {
            if(i != (length(size_NT_regions))) {
              onset=3-(size_NT_regions[i]-off_set)%%3

              if(nchar(size_NT_regions[i+1])>= onset) {
                if(onset==2){
                  size_AA_regions=c(size_AA_regions, floor((size_NT_regions[i]-off_set)/3))
                  off_set=-1
                } else {
                  size_AA_regions=c(size_AA_regions, ceiling((size_NT_regions[i]-off_set)/3))
                  off_set=onset
                }

              } else {
                size_AA_regions=c(size_AA_regions, floor(size_NT_regions[i]-off_set)/3)
                off_set=0
              }
            } else {
              size_AA_regions=c(size_AA_regions, floor(size_NT_regions[i]-off_set)/3)
              off_set=0
            }


          }
        }
        # size_AA_regions <- round(size_NT_regions/3)
        # size_AA_regions[length(size_AA_regions) -
        #     1] <- floor(
        #     (size_NT_regions[length(size_AA_regions) -
        #         1]/3)
        # )
        # size_AA_regions[length(size_AA_regions)] <- floor((size_NT_regions[length(size_AA_regions)]/3))
        end <- 0


        for (i in 1:(length(size_AA_regions) -
            1)) {
            start <- end + 1
            end <- start - 1 + size_AA_regions[i]
            seq_list[i] <- Biostrings::subseq(AA_seq, start, end)
        }

        seq_list[[length(size_AA_regions)]] <- Biostrings::subseq(
            AA_seq, sum(
                size_AA_regions[1:length(size_AA_regions) -
                  1]
            ) +
                1, stringr::str_length(AA_seq)
        )

        if (FWR1partial) {
            seq_list <- append(seq_list, "", after = 0)
        }

        if (FWR4partial) {
            seq_list <- append(seq_list, "", after = length(seq_list))
        }


        if(paste(unlist(seq_list), collapse="") != AA_seq){
          strange=TRUE
        } else {
          strange=FALSE
        }
        seq_list[length(seq_list) +
            1] <- AA_seq
        seq_list[length(seq_list) +
            1] <- ORF

        if(strange){
          seq_list[length(seq_list) +
                     1] <- ("MMMMMMMMMMMmmmmmmmmmmm strange")
        }




    }
    return(seq_list)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#'
#' @noRd
length_of_region <- function(sequence, region) {
    region_length <- nchar(sequence)
    names(region_length) <- paste0(region, "_length")
    return(region_length)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import stringdist
#' @noRd
lv_changes <- function(seq_repertoire, seq_rec.germline, region) {
    lv_dist <- stringdist::stringdist(seq_rec.germline, seq_repertoire, method = "lv")
    if (Biostrings::width(as.character(seq_repertoire)) ==
        0) {
        lv_dist_norm <- 0
    } else {
        lv_dist_norm <- (lv_dist/Biostrings::width(as.character(seq_repertoire))) *
            100
    }

    changes <- c(lv_dist, lv_dist_norm)
    names(changes) <- c("lv_dist", "lv_dist_norm")
    names(changes) <- paste(
        as.character(region),
        names(changes),
        sep = "_"
    )
    return(changes)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import stringdist
#' @import utils
#' @noRd
nt_changes <- function(seq_repertoire, seq_rec.germline, region) {
    # lv_info <- lv_changes(seq_repertoire, seq_rec.germline, region)
    # lv_dist <- lv_info[1]
    # osa_dist <- stringdist::stringdist(seq_rec.germline, seq_repertoire, method = "osa")
    # no.trans <- lv_dist - osa_dist
    # tab <- drop(
    #     attr(
    #         adist(seq_rec.germline, seq_repertoire, count = TRUE),
    #         "counts"
    #     )
    # )
    # no.ins <- tab[["ins"]] - no.trans
    # no.del <- tab[["del"]] - no.trans
    # no.subs <- tab[["sub"]]
    # changes <- c(no.subs, no.ins, no.del, no.trans)
    # if (sum(changes) !=
    #     0) {
    #     changes_plus <- c(
    #         changes, sum(changes),
    #         (changes[1]/sum(changes)) *
    #             100, (changes[2]/sum(changes)) *
    #             100, (changes[3]/sum(changes)) *
    #             100, (changes[4]/sum(changes)) *
    #             100
    #     )
    # } else {
    #     changes_plus <- c(
    #         changes, sum(changes),
    #         0, 0, 0, 0
    #     )
    # }
    # names(changes_plus) <- c(
    #     "Sub", "Ins", "Del", "Trasl", "SIDT_sum", "Sub_prc", "Ins_prc", "Del_prc",
    #     "Trasl_prc"
    # )
    # names(changes_plus) <- gsub(
    #     "^", paste0(
    #         as.character(region),
    #         "_"
    #     ),
    #     names(changes_plus)
    # )

  if(nchar(seq_repertoire) > 0 ) {
    insertions=as.data.frame(str_locate_all(seq_repertoire,"[a-z]+")[[1]])
    n_ins=nrow(insertions)
    if(nrow(insertions) >0) {
      insertions$length=insertions$end-insertions$start+1
      length_ins=sum(insertions$length)
    } else {
      length_ins=0
    }

    deletions=as.data.frame(str_locate_all(seq_rec.germline,"[a-z]+")[[1]])
    n_dels=nrow(deletions)
    if(nrow(deletions) >0) {
      deletions$length=deletions$end-deletions$start+1
      length_del=sum(deletions$length)
    } else {
      length_del=0
    }

  } else {
    n_ins=0
    n_dels=0
    length_ins=0
    length_del=0
  }

    changes=c(n_ins, n_dels,length_ins,length_del)
    names(changes)=c("NumIns","NumDels","LenghtIns","LenghtDels")

    names(changes) <- gsub(
        "^", paste0(
            as.character(region),
            "_"
        ),
        names(changes)
    )
    return(changes)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @noRd
show_selected_features <- function(tmp_sunburst = tmp_sunburst, region = region) {

    # for (region in regions) {
    index_regions <- which(grepl(region, tmp_sunburst, fixed = TRUE))
    tmp_sunburst <- tmp_sunburst[index_regions]
    tmp_sunburst <- gsub(
        paste(region, "_", sep = ""),
        "", tmp_sunburst, fixed = TRUE
    )

    # tmp_sunburst[index_regions]=paste(region, tmp_sunburst[index_regions],
    # sep='-') }


    tmp_sunburst <- gsub("NT_", "NT-", tmp_sunburst, fixed = TRUE)
    tmp_sunburst <- gsub("AA_", "AA-", tmp_sunburst, fixed = TRUE)
    tmp_sunburst <- gsub("_Diff", "-Diff_with_germline", tmp_sunburst, fixed = TRUE)
    tmp_index <- which(!grepl("-Diff_with_germline", tmp_sunburst))
    tmp_sunburst[tmp_index] <- paste(tmp_sunburst[tmp_index], "-Sequence_values", sep = "")




    nucleotides <- c("A", "G", "T", "C")
    peptides <- Peptides::aaList()

    tmp_index <- which(
        grepl(
            paste(
                c(
                  paste(
                    "^NT-", paste(
                      c(nucleotides),
                      "count", sep = "_"
                  ),
                    sep = ""
                ),
                  paste(
                    "^NT-", paste(
                      c(nucleotides),
                      "norm", sep = "_"
                  ),
                    sep = ""
                ),
                  paste(
                    "^AA-", paste(
                      c(peptides),
                      "count", sep = "_"
                  ),
                    sep = ""
                ),
                  paste(
                    "^AA-", paste(
                      c(peptides),
                      "norm", sep = "_"
                  ),
                    sep = ""
                ),
                  "counts"
              ),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Composition-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("^AA-", "AA-Composition-Aminoacids-", tmp_sunburst[tmp_index], perl = TRUE)
    tmp_index <- which(grepl("counts", tmp_sunburst))
    tmp_sunburst[tmp_index] <- gsub("^AA-", "AA-Composition-Codons-", tmp_sunburst[tmp_index], perl = TRUE)
    # print(tmp_sunburst[tmp_index])

    tmp_index <- which(
        grepl(
            paste(
                c("hot", "cold", "potential"),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("cold_", "cold-", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("hot_", "hot-", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_R_", "-R_", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_F_", "-F_", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_norm", "-norm", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_count", "-count", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Hot/Cold motifs-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("^AA-", "AA-Hot/Cold motifs-", tmp_sunburst[tmp_index], perl = TRUE)

    tmp_index <- which(
        grepl(
            paste(
                c("Sub", "Sub_prc", "SIDT", "SID_sum"),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Substitutions-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Substitutions-", tmp_sunburst[tmp_index])


    tmp_index <- which(
        grepl(
            paste(
                c("Ins", "Ins_prc", "SIDT", "SID_sum"),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Insertions-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Insertions-", tmp_sunburst[tmp_index])

    tmp_index <- which(
        grepl(
            paste(
                c("Del", "Del_prc", "SIDT", "SID_sum"),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Deletions-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Deletions-", tmp_sunburst[tmp_index])


    tmp_index <- which(
        grepl(
            paste(
                c("Trasl", "Trasl_prc", "SIDT"),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Translocations-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Translocations-", tmp_sunburst[tmp_index])


    tmp_index <- which(grepl("lv_dist", tmp_sunburst))
    tmp_sunburst[tmp_index] <- gsub(
        "Transitions-Transversions", "TransitionsToTransversions", tmp_sunburst[tmp_index],
        fixed = TRUE
    )
    tmp_sunburst[tmp_index] <- gsub("_norm", "-norm", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_count", "-count", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Leveshtein distance-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Leveshtein distance-", tmp_sunburst[tmp_index])


    tmp_index <- which(grepl("Transitions|Transversions", tmp_sunburst))
    tmp_sunburst[tmp_index] <- gsub(
        "Transitions-Transversions", "TransitionsToTransversions", tmp_sunburst[tmp_index],
        fixed = TRUE
    )
    tmp_sunburst[tmp_index] <- gsub("_norm", "-norm", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_count", "-count", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Transitions and transversions-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Transitions and transversions-", tmp_sunburst[tmp_index])

    tmp_index <- which(grepl("Replacement|Silent", tmp_sunburst))
    tmp_sunburst[tmp_index] <- gsub(
        "Silent-Replacement", "SilentToReplacement", tmp_sunburst[tmp_index], fixed = TRUE
    )
    tmp_sunburst[tmp_index] <- gsub("_norm", "-norm", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Replacement and silent mutations-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Replacement and silent mutations-", tmp_sunburst[tmp_index])

    tmp_index <- which(grepl("_to_", tmp_sunburst))
    tmp_sunburst[tmp_index] <- gsub("_count", "-count", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_norm", "-norm", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Mutations, from A to B-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Mutations, from A to B-", tmp_sunburst[tmp_index])


    tmp_index <- which(
        grepl(
            paste(
                c(
                  "Peptides", "alkzm", "Small", "Tiny", "Aliphatic", "Charged", "Polar",
                  "Basic", "NonPolar", "Aromatic", "Acidic"
              ),
                collapse = "|"
            ),
            tmp_sunburst
        )
    )
    tmp_sunburst[tmp_index] <- gsub("Peptides_", "Peptides-", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("alkzm_", "alkzm-", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_count", "-count", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("_norm", "-norm", tmp_sunburst[tmp_index], fixed = TRUE)
    tmp_sunburst[tmp_index] <- gsub("NT-", "NT-Peptide features-", tmp_sunburst[tmp_index])
    tmp_sunburst[tmp_index] <- gsub("AA-", "AA-Peptide features-", tmp_sunburst[tmp_index])


    sunburst_df <- data.frame(V1 = tmp_sunburst)
    sunburst_df$V2 <- 1

    return(sunburst_df)
}

#' 2_Feature_determination
#'
#' @description A utils function, not used currently
#'
#' @return The return value, if any, from executing the utility.
#' @importFrom utils adist
#' @noRd
#'
aa_changes <- function(seq_repertoire, seq_rec.germline, region) {
    tab <- drop(
        attr(
            adist(seq_rec.germline, seq_repertoire, counts = TRUE),
            "counts"
        )
    )
    no.ins <- tab[["ins"]]
    no.del <- tab[["del"]]
    no.subs <- tab[["sub"]]
    changes <- c(no.subs, no.ins, no.del)
    if (sum(changes) !=
        0) {
        changes_plus <- c(
            changes, sum(changes),
            (changes[1]/sum(changes)) *
                100, (changes[2]/sum(changes)) *
                100, (changes[3]/sum(changes)) *
                100
        )
    } else {
        changes_plus <- c(
            changes, sum(changes),
            0, 0, 0
        )
    }
    names(changes_plus) <- c("Sub", "Ins", "Del", "SID_sum", "Sub_prc", "Ins_prc", "Del_prc")
    names(changes_plus) <- gsub(
        "^", paste0(
            as.character(region),
            "_"
        ),
        names(changes_plus)
    )

    lv_info <- lv_changes(seq_repertoire, seq_rec.germline, region)
    return(c(changes_plus, lv_info))
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import stringr
#' @noRd
count_elements <- function(sequence, region) {
    if (startsWith(region, "NT")) {
        elements <- c("A", "G", "C", "T")
    } else if (startsWith(region, "AA")) {
        aa_names <- c(
            "Phe", "Met", "Leu", "Ile", "Val", "Pro", "Tyr", "Trp", "Cys", "Ala",
            "Gly", "Ser", "Thr", "His", "Glu", "Gln", "Asp", "Asn", "Lys", "Arg"
        )
        elements <- seqinr::a(aa_names)
    } else {
        # print("[ERROR] Wrong region included")
    }
    length_seq <- stringr::str_length(sequence)
    results <- c()

    for (i in elements) {
        counts <- str_count(sequence, i)
        previous_names <- names(results)
        if (length_seq == 0) {
            results <- c(results, counts, 0)
        } else {
            results <- c(results, counts, 100 * counts/length_seq)
        }

        names(results) <- c(
            previous_names, paste0(region, "_", i, "_count"),
            paste0(region, "_", i, "_norm")
        )
    }
    return(results)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import Peptides
#' @noRd
instaIndex_improved <- function(seq) {
    #### fake instability index of 0 to single aminoacids changed to be able to
    #### work with dipeptides
    guruprasad <- c(
        WW = 1, WC = 1, WM = 24.68, WH = 24.68, WY = 1, WF = 1, WQ = 1, WN = 13.34,
        WI = 1, WR = 1, WD = 1, WP = 1, WT = -14.03, WK = 1, WE = 1, WV = -7.49,
        WS = 1, WG = -9.37, WA = -14.03, WL = 13.34, CW = 24.68, CC = 1, CM = 33.6,
        CH = 33.6, CY = 1, CF = 1, CQ = -6.54, CN = 1, CI = 1, CR = 1, CD = 20.26,
        CP = 20.26, CT = 33.6, CK = 1, CE = 1, CV = -6.54, CS = 1, CG = 1, CA = 1,
        CL = 20.26, MW = 1, MC = 1, MM = -1.88, MH = 58.28, MY = 24.68, MF = 1, MQ = -6.54,
        MN = 1, MI = 1, MR = -6.54, MD = 1, MP = 44.94, MT = -1.88, MK = 1, ME = 1,
        MV = 1, MS = 44.94, MG = 1, MA = 13.34, ML = 1, HW = -1.88, HC = 1, HM = 1,
        HH = 1, HY = 44.94, HF = -9.37, HQ = 1, HN = 24.68, HI = 44.94, HR = 1, HD = 1,
        HP = -1.88, HT = -6.54, HK = 24.68, HE = 1, HV = 1, HS = 1, HG = -9.37, HA = 1,
        HL = 1, YW = -9.37, YC = 1, YM = 44.94, YH = 13.34, YY = 13.34, YF = 1, YQ = 1,
        YN = 1, YI = 1, YR = -15.91, YD = 24.68, YP = 13.34, YT = -7.49, YK = 1,
        YE = -6.54, YV = 1, YS = 1, YG = -7.49, YA = 24.68, YL = 1, FW = 1, FC = 1,
        FM = 1, FH = 1, FY = 33.6, FF = 1, FQ = 1, FN = 1, FI = 1, FR = 1, FD = 13.34,
        FP = 20.26, FT = 1, FK = -14.03, FE = 1, FV = 1, FS = 1, FG = 1, FA = 1,
        FL = 1, QW = 1, QC = -6.54, QM = 1, QH = 1, QY = -6.54, QF = -6.54, QQ = 20.26,
        QN = 1, QI = 1, QR = 1, QD = 20.26, QP = 20.26, QT = 1, QK = 1, QE = 20.26,
        QV = -6.54, QS = 44.94, QG = 1, QA = 1, QL = 1, NW = -9.37, NC = -1.88, NM = 1,
        NH = 1, NY = 1, NF = -14.03, NQ = -6.54, NN = 1, NI = 44.94, NR = 1, ND = 1,
        NP = -1.88, NT = -7.49, NK = 24.68, NE = 1, NV = 1, NS = 1, NG = -14.03,
        NL = 1, IW = 1, IC = 1, IM = 1, IH = 13.34, IY = 1, IF = 1, IQ = 1, IN = 1,
        II = 1, IR = 1, ID = 1, IP = -1.88, IT = 1, IK = -7.49, IE = 44.94, IV = -7.49,
        IS = 1, IG = 1, IA = 1, IL = 20.26, RW = 58.28, RC = 1, RM = 1, RH = 20.26,
        RY = -6.54, RF = 1, RQ = 20.26, RN = 13.34, RI = 1, RR = 58.28, RD = 1, RP = 20.26,
        RT = 1, RK = 1, RE = 1, RV = 1, RS = 44.94, RG = -7.49, RA = 1, RL = 1, DW = 1,
        DC = 1, DM = 1, DH = 1, DY = 1, DF = -6.54, DQ = 1, DN = 1, DI = 1, DR = -6.54,
        DD = 1, DP = 1, DT = -14.03, DK = -7.49, DE = 1, DV = 1, DS = 20.26, DG = 1,
        DA = 1, DL = 1, PW = -1.88, PC = -6.54, PM = -6.54, PH = 1, PY = 1, PF = 20.26,
        PQ = 20.26, PN = 1, PI = 1, PR = -6.54, PD = -6.54, PP = 20.26, PT = 1, PK = 1,
        PE = 18.38, PV = 20.26, PS = 20.26, PG = 1, PA = 20.26, PL = 1, TW = -14.03,
        TC = 1, TM = 1, TH = 1, TY = 1, TF = 13.34, TQ = -6.54, TN = -14.03, TI = 1,
        TR = 1, TD = 1, TP = 1, TT = 1, TK = 1, TE = 20.26, TV = 1, TS = 1, TG = -7.49,
        TA = 1, TL = 1, KW = 1, KC = 1, KM = 33.6, KH = 1, KY = 1, KF = 1, KQ = 24.68,
        KN = 1, KI = -7.49, KR = 33.6, KD = 1, KP = -6.54, KT = 1, KK = 1, KE = 1,
        KV = -7.49, KS = 1, KG = -7.49, KA = 1, KL = -7.49, EW = -14.03, EC = 44.94,
        EM = 1, EH = -6.54, EY = 1, EF = 1, EQ = 20.26, EN = 1, EI = 20.26, ER = 1,
        ED = 20.26, EP = 20.26, ET = 1, EK = 1, EE = 33.6, EV = 1, ES = 20.26, EG = 1,
        EA = 1, EL = 1, VW = 1, VC = 1, VM = 1, VH = 1, VY = -6.54, VF = 1, VQ = 1,
        VN = 1, VI = 1, VR = 1, VD = -14.03, VP = 20.26, VT = -7.49, VK = -1.88,
        VE = 1, VV = 1, VS = 1, VG = -7.49, VA = 1, VL = 1, SW = 1, SC = 33.6, SM = 1,
        SH = 1, SY = 1, SF = 1, SQ = 20.26, SN = 1, SI = 1, SR = 20.26, SD = 1, SP = 44.94,
        ST = 1, SK = 1, SE = 20.26, SV = 1, SS = 20.26, SG = 1, SA = 1, SL = 1, GW = 13.34,
        GC = 1, GM = 1, GH = 1, GY = -7.49, GF = 1, GQ = 1, GN = -7.49, GI = -7.49,
        GR = 1, GD = 1, GP = 1, GT = -7.49, GK = -7.49, GE = -6.54, GV = 1, GS = 1,
        GG = 13.34, GA = -7.49, GL = 1, AW = 1, AC = 44.94, AM = 1, AH = -7.49, AY = 1,
        AF = 1, AQ = 1, AN = 1, AI = 1, AR = 1, AD = -7.49, AP = 20.26, AT = 1, AK = 1,
        AE = 1, AV = 1, AS = 1, AG = 1, AA = 1, AL = 1, LW = 24.68, LC = 1, LM = 1,
        LH = 1, LY = 1, LF = 1, LQ = 33.6, LN = 1, LI = 1, LR = 20.26, LD = 1, LP = 20.26,
        LT = 1, LK = -7.49, LE = 1, LV = 1, LS = 1, LG = 1, LA = 1, LL = 1, `NA` = 1
    )
    aa <- Peptides_aaCheck(seq)
    dp <- lapply(
        aa, function(aa) {
            if (length(aa) ==
                1) {
                0
            } else if (length(aa) ==
                2) {
                unname(
                  apply(
                    t(
                      as.data.frame(
                        stats::embed(aa, 2)[,
                          c(2:1)]
                    )
                  ),
                    1, paste0, collapse = ""
                )
              )
            } else {
                apply(
                  stats::embed(aa, 2)[,
                    2:1], 1, paste0, collapse = ""
              )
            }
        }
    )
    gp <- lapply(
        dp, function(dp) {
            (10/(length(dp) +
                1)) * sum(guruprasad[dp], na.rm = TRUE)
        }
    )
    return(unlist(gp))
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @importFrom stringr str_split
#' @noRd
group_freqs <- function(peptide, region) {
    # define groups
    group_list <- list()
    group_list[[1]] <- c("A", "C", "G", "S", "T")
    group_list[[2]] <- c("A", "B", "C", "D", "G", "N", "P", "S", "T", "V")
    group_list[[3]] <- c("A", "I", "L", "V")
    group_list[[4]] <- c("F", "H", "W", "Y")
    group_list[[5]] <- c("A", "C", "F", "G", "I", "L", "M", "P", "V", "W", "Y")
    group_list[[6]] <- c("D", "E", "H", "K", "N", "Q", "R", "S", "T", "Z")
    group_list[[7]] <- c("B", "D", "E", "H", "K", "R", "Z")
    group_list[[8]] <- c("H", "K", "R")
    group_list[[9]] <- c("B", "D", "E", "Z")
    aa_groups <- c(
        "Tiny", "Small", "Aliphatic", "Aromatic", "NonPolar", "Polar", "Charged",
        "Basic", "Acidic"
    )
    names(group_list) <- aa_groups
    # Prepare peptide
    split_peptide <- unlist(str_split(peptide, ""))
    number <- c()
    perc <- c()
    for (group in aa_groups) {
        number <- c(
            number, sum(
                table(split_peptide)[group_list[[group]]],
                na.rm = TRUE
            )
        )
        if (length(split_peptide) ==
            0) {
            perc <- c(perc, 0)
        } else {
            perc <- c(
                perc, (sum(
                  table(split_peptide)[group_list[[group]]],
                  na.rm = TRUE
              )/length(split_peptide)) *
                  100
            )
        }

    }
    names(number) <- paste0(region, "_", aa_groups, "_count")
    names(perc) <- paste0(region, "_", aa_groups, "_norm")
    stats <- c(number, round(perc, 3))
    return(stats)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @noRd
alakazam_properties <- function(peptide, region) {
    bulkiness <- alakazam::bulk(peptide)
    hydrophobicity <- alakazam::gravy(peptide)
    aliphatic_index <- alakazam::aliphatic(peptide)
    avg_polarity <- alakazam::polar(peptide)
    charge <- alakazam::charge(peptide)
    normalized_charge <- alakazam::charge(peptide, normalize = TRUE)
    alkzm_prprts <- c(
        bulkiness, hydrophobicity, aliphatic_index, avg_polarity, charge, normalized_charge
    )
    names(alkzm_prprts) <- c(
        "bulkiness", "hydrophobicity", "aliphatic_index", "avg_polarity", "charge",
        "normalized_charge"
    )
    names(alkzm_prprts) <- gsub(
        "^", paste0(region, "_", "alkzm_"),
        names(alkzm_prprts)
    )
    return(alkzm_prprts)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @noRd
Peptides_properties <- function(peptide, region) {
    aliphatic_index <- Peptides::aIndex(peptide)
    boman_index <- Peptides::boman(peptide)  #A protein has high binding potential if the index value is higher than 2.48
    charge <- Peptides::charge(peptide, pH = 7.4, pKscale = "EMBOSS")  #ph blood 7.4
    hmoment <- Peptides::hmoment(peptide)
    hydrophobicity <- Peptides::hydrophobicity(peptide)
    instability_index <- instaIndex_improved(peptide)
    mol_weight <- Peptides::mw(peptide)
    isoelectric_point <- Peptides::pI(peptide)

    # these need names
    names(aliphatic_index) <- "aliphatic_index"
    names(boman_index) <- "boman_index"
    names(charge) <- "charge"
    names(hmoment) <- "hmoment"
    names(hydrophobicity) <- "hydrophobicity"
    names(instability_index) <- "instability_index"
    names(mol_weight) <- "mol_weight"
    names(isoelectric_point) <- "isoelectric_point"

    # these are named already
    cruciani_properties <- unlist(Peptides::crucianiProperties(peptide))
    fasgai_vecs <- unlist(Peptides::fasgaiVectors(peptide))
    blosum_indicces <- unlist(Peptides::blosumIndices(peptide))
    kidera_factors <- unlist(Peptides::kideraFactors(peptide))
    mswhim_scores <- unlist(Peptides::mswhimScores(peptide))
    st_scales <- unlist(Peptides::stScales(peptide))
    t_scales <- unlist(Peptides::tScales(peptide))
    vhse_scales <- unlist(Peptides::vhseScales(peptide))
    z_scales <- unlist(Peptides::zScales(peptide))

    pptds_prprts <- c(
        aliphatic_index, boman_index, charge, hmoment, hydrophobicity, instability_index,
        mol_weight, isoelectric_point, cruciani_properties, fasgai_vecs, blosum_indicces,
        kidera_factors, mswhim_scores, st_scales, t_scales, vhse_scales, z_scales
    )
    names(pptds_prprts) <- gsub(
        "^", paste0(region, "_", "Peptides_"),
        names(pptds_prprts)
    )
    return(pptds_prprts)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#'
#' @noRd
edited_hot_cold_mtfs <- function() {
    hot_1_F <- "WRC"
    hot_1_R <- "GYW"
    hot_2_F <- "WA"
    hot_2_R <- "TW"
    cold_F <- "SYC"
    cold_R <- "GRS"
    potential_1_F <- "CRCY"
    potential_1_R <- "RGYG"
    potential_2_F <- "ATCT"
    potential_2_R <- "AGAT"
    hot_cold_motif_vec <- c(
        hot_1_F, hot_1_R, hot_2_F, hot_2_R, cold_F, cold_R, potential_1_F, potential_1_R,
        potential_2_F, potential_2_R
    )
    edit_mtf <- function(mtf) {
        mtf <- gsub("S", "[GC]", mtf)
        mtf <- gsub("W", "[AT]", mtf)
        mtf <- gsub("R", "[GA]", mtf)
        mtf <- gsub("Y", "[CT]", mtf)
        return(mtf)
    }
    edited_hot_cold_mtfs <- sapply(hot_cold_motif_vec, edit_mtf)
    names(edited_hot_cold_mtfs) <- paste0(
        c(
            rep("hot_", 4),
            rep("cold_", 2),
            rep("potential_hot_", 4)
        ),
        hot_cold_motif_vec
    )
    names(edited_hot_cold_mtfs) <- paste0(
        names(edited_hot_cold_mtfs),
        c(
            rep("_1", 2),
            rep("_2", 2),
            rep("", 2),
            rep("_1", 2),
            rep("_2", 2)
        )
    )
    names(edited_hot_cold_mtfs) <- paste0(
        names(edited_hot_cold_mtfs),
        c("_F", "_R")
    )
    return(edited_hot_cold_mtfs)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#'
#' @noRd
mut_points_hot_cold_mtfs <- function(edited_hot_cold_mtfs) {
    hot_1_F <- 3
    hot_1_R <- 1
    hot_2_F <- 2
    hot_2_R <- 1
    cold_F <- 3
    cold_R <- 1
    potential_1_F <- 3
    potential_1_R <- 2
    potential_2_F <- 3
    potential_2_R <- 2
    mut_points <- c(
        hot_1_F, hot_1_R, hot_2_F, hot_2_R, cold_F, cold_R, potential_1_F, potential_1_R,
        potential_2_F, potential_2_R
    )
    names(mut_points) <- names(edited_hot_cold_mtfs)
    return(mut_points)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @importFrom stringr str_locate_all
#' @noRd
locate_motif <- function(full_sequence, lengths, motif, motif_ID, motif_ini, regions) {
    loci <- unlist(data.frame(str_locate_all(full_sequence, motif))["start"])
    loci_mut_point <- loci + (motif_ini - 1)
    index_locis <- rep(
        "", length(lengths) -
            1
    )
    motif_count <- rep(
        0, length(lengths) -
            1
    )
    if (length(loci) >
        0) {
        for (i in c(
            1:(length(lengths) -
                1)
        )) {
            if (i != 1) {
                min <- sum(lengths[1:(i - 1)])
            } else {
                min <- 0
            }
            index_locis[i] <- paste(
                loci[which(loci_mut_point > min & loci_mut_point <= min + lengths[i])] -
                  min, collapse = "/"
            )
            motif_count[i] <- length(which(loci_mut_point > min & loci_mut_point <= min + lengths[i]))
        }
    }

    index_locis <- c(index_locis, paste(loci, collapse = "/"))
    names(index_locis) <- paste0(regions, "_", motif_ID, "_Pos")

    motif_count <- c(motif_count, sum(motif_count))
    names(motif_count) <- paste0(regions, "_", motif_ID, "_count")
    perc_locis <- 100 * motif_count/lengths
    names(perc_locis) <- paste0(regions, "_", motif_ID, "_norm")

    perc_locis[which(is.nan(perc_locis))] <- 0

    results <- list(index_locis = index_locis, motif_count = motif_count, perc_locis = perc_locis)
    return(results)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @import stringr
#' @import utils
#' @noRd
compare_sequences <- function(
    rep_seq, germ_seq, mode, nt_rep_lengths = NULL, aa_rep_lengths = NULL, nt_germ_lengths = NULL,
    aa_germ_lengths = NULL, NT_regions = NULL, AA_regions = NULL, ORF_rep = NULL,
    ORF_germ = NULL, aant_rep_seq=NULL, aant_germ_seq=NULL
) {

  ##lets find the indel coordinates and their respective ones
  if(mode=="AA") {
    germ_indel_seq=aant_germ_seq
    rep_indel_seq=aant_rep_seq
  } else {
    if(mode=="Codon") {
      germ_seq=substr(germ_seq, ORF_germ, nchar(germ_seq))
      rep_seq=substr(rep_seq, ORF_rep, nchar(rep_seq))
    }
    germ_indel_seq=germ_seq
    rep_indel_seq=rep_seq
  }

  indel=F
  indel_g=F
  indel_r=F
  indel_table=data.frame()

  indels_germ=as.data.frame(str_locate_all(germ_indel_seq, "[a-z]+")[[1]])
  if(nrow(indels_germ)>0){
    indels_germ=cbind(indels_germ, rep("Germ", nrow(indels_germ)))
    colnames(indels_germ)[3]="Seq"
    indel_g=T
  }

  indels_rep=as.data.frame(str_locate_all(rep_indel_seq, "[a-z]+")[[1]])
  if(nrow(indels_rep)>0){
    indels_rep=cbind(indels_rep, rep("Rep", nrow(indels_rep)))
    colnames(indels_rep)[3]="Seq"
    indel_r=T
  }
  if(indel_g && indel_r) {
    indel_table=rbind(indels_rep, indels_germ)
    indel=T
  } else if(indel_g){
    indel_table=indels_germ
    indel=T
  } else if (indel_r) {
    indel_table=indels_rep
    indel=T
  }

  if(indel){
    indel_table=indel_table[order(indel_table[,"start"]),]
    indel_table$length=indel_table$end-indel_table$start+1

    if(indel_g) {
      for (row in  which(indel_table[,"Seq"]=="Germ")){
        index_lower_start=intersect(which(indel_table$start < indel_table[row, "start"]),
                                    which(indel_table[,"Seq"]!="Germ"))
        indel_table[row, c("start", "end")]=indel_table[row, c("start", "end")]-sum(indel_table$length[index_lower_start])
      }

    }

    indel_positions=data.frame(Seq=as.character(), Position=as.numeric(),AA_Codon_position=as.numeric())

    for(row in c(1:nrow(indel_table))) {
      start=indel_table$start[row]
      for(nt in c(1:indel_table$length[row])) {
        indel_positions=rbind(indel_positions, data.frame(Seq=indel_table$Seq[row], Position=start,AA_Codon_position=ceiling(start/3)))
        start=start+1
      }
    }
    indel_positions$AA_Codon_full=sapply(c(1:nrow(indel_positions)), function(z) length(intersect(which(indel_positions$Seq ==indel_positions$Seq[z]),
                                                                                                  which(indel_positions$AA_Codon_position ==indel_positions$AA_Codon_position[z]))) )
  }

  if(mode=="Codon") {

    if(indel) {
      mut_germ_seq=strsplit(germ_seq, split="")[[1]]
      mut_rep_seq=strsplit(rep_seq, split="")[[1]]
      mut_germ_seq=mut_germ_seq[which(c(1:length(mut_germ_seq)) %!in% indel_positions[which(indel_positions[,"Seq"]=="Germ"),"Position"])  ]

      for(i in indel_positions[which(indel_positions[,"Seq"]!="Germ"),"Position"])  {
        mut_germ_seq <- append(mut_germ_seq, mut_rep_seq[i], i-1)
      }

      mut_germ_seq=toupper(paste(mut_germ_seq, collapse=""))
      mut_rep_seq=toupper(paste(mut_rep_seq, collapse=""))
    }
  }

  rep_seq <- toupper(rep_seq)
  germ_seq <- toupper(germ_seq)


  if (mode == "AA") {
    regions <- AA_regions
    aa_names <- c(
      "Phe", "Met", "Leu", "Ile", "Val", "Pro", "Tyr", "Trp", "Cys", "Ala",
      "Gly", "Ser", "Thr", "His", "Glu", "Gln", "Asp", "Asn", "Lys", "Arg"
    )
    elements <- seqinr::a(aa_names)

    group_list <- list()
    group_list[[1]] <- c("A", "C", "G", "S", "T")
    group_list[[2]] <- c("A", "B", "C", "D", "G", "N", "P", "S", "T", "V")
    group_list[[3]] <- c("A", "I", "L", "V")
    group_list[[4]] <- c("F", "H", "W", "Y")
    group_list[[5]] <- c("A", "C", "F", "G", "I", "L", "M", "P", "V", "W", "Y")
    group_list[[6]] <- c("D", "E", "H", "K", "N", "Q", "R", "S", "T", "Z")
    group_list[[7]] <- c("B", "D", "E", "H", "K", "R", "Z")
    group_list[[8]] <- c("H", "K", "R")
    group_list[[9]] <- c("B", "D", "E", "Z")
    aa_groups <- c(
      "Tiny", "Small", "Aliphatic", "Aromatic", "NonPolar", "Polar", "Charged",
      "Basic", "Acidic"
    )
    names(group_list) <- aa_groups

    rep_seq=strsplit(rep_seq, split="")[[1]]
    germ_seq=strsplit(germ_seq, split="")[[1]]

    rep_lengths <- aa_rep_lengths
    germ_lengths <- aa_germ_lengths

  } else if (mode == "NT") {
    regions <- NT_regions
    elements <- c("A", "C", "G", "T")
    transition_muts <- c("A_to_G", "G_to_A", "T_to_C", "C_to_T")
    rep_lengths <- nt_rep_lengths
    germ_lengths <- nt_germ_lengths

    rep_seq=strsplit(rep_seq, split="")[[1]]
    germ_seq=strsplit(germ_seq, split="")[[1]]

  } else if (mode == "Codon") {
    elements <- sort(names(Biostrings::GENETIC_CODE))
    regions <- AA_regions

    rep_seq=as.character(Biostrings::codons(Biostrings::DNAString(rep_seq)))
    germ_seq=as.character(Biostrings::codons(Biostrings::DNAString(germ_seq)))

    rep_codon_count <- c()
    germ_codon_count <- c()


    rep_lengths <- aa_rep_lengths
    germ_lengths <- aa_germ_lengths
  } else {
    # print("There is an error with the mode")
  }


  ###coordinates of each region
  coord_rep_regions=c(1:rep_lengths[8])
  coord_germ_regions=c(1:germ_lengths[8])

  tmp_rep_lengths=rep_lengths[1:7]
  tmp_germ_lengths=germ_lengths[1:7]
  names(tmp_rep_lengths)=gsub("_length","", names(tmp_rep_lengths))
  names(tmp_germ_lengths)=gsub("_length","", names(tmp_germ_lengths))
  for(i in c(2:7)) {
    tmp_rep_lengths[i]=tmp_rep_lengths[i]+tmp_rep_lengths[i-1]
    tmp_germ_lengths[i]=tmp_germ_lengths[i]+tmp_germ_lengths[i-1]
  }
  names(coord_rep_regions)=sapply(coord_rep_regions, function(z) names(tmp_rep_lengths)[ which(tmp_rep_lengths>=z)[1]])
  names(coord_germ_regions)=sapply(coord_germ_regions, function(z) names(tmp_germ_lengths)[ which(tmp_germ_lengths>=z)[1]])

  coord_regions=list(coord_rep_regions, coord_germ_regions)
  counts_per_sequence=list(Rep=c(), Germ=c())
  ###count codons
  if (mode == "Codon") {
    for (region in regions) {


      if(region == "AA_Whole") {
        location=c(regions)
      } else {
        location=region
      }

      for(i in c(1:2)) {
        coords_region=which(names(coord_regions[[i]]) %in% location)
        codons_present=table(rep_seq[coords_region])

        tmp_counts=rep(0, length(elements[which(!(elements %in% names(codons_present)))]))
        names(tmp_counts)= elements[which(!(elements %in% names(codons_present)))]
        tmp_counts=c(tmp_counts, codons_present)
        tmp_counts=tmp_counts[sort(names(tmp_counts))]
        names(tmp_counts)=paste(region, names(tmp_counts), sep="_")

        if(length(coords_region)!=0) {
          tmp_counts_norm=tmp_counts/length(coords_region)
        } else {
          tmp_counts_norm=tmp_counts
        }
        names(tmp_counts)=paste(names(tmp_counts), "counts", sep="_")
        names(tmp_counts_norm)=paste(names(tmp_counts), "norm", sep="_")


        counts_per_sequence[[i]]=c(counts_per_sequence[[i]], tmp_counts, tmp_counts_norm)
      }

    }

  }

  ##confirmed with only tmp_counts        sum(counts_per_sequence[[i]][which(startsWith(names(counts_per_sequence[[i]]), "AA_Whole"))])


  ## NT to NT
  if (mode == "NT" || mode =="AA" || mode =="Codon") {

    counts=c()
    for (region in regions) {


      if(region == "AA_Whole" || region == "NT_Whole") {
        location=c(regions)
      } else {
        location=region
      }
      coords_region=which(names(coord_regions[[1]]) %in% location)

      coords_rep=which(names(coord_regions[[1]]) %in% location)
      coords_germ=which(names(coord_regions[[2]]) %in% location)


      if(indel) {
        if (mode == "NT") {
          coords_rep=setdiff(coords_rep,indel_positions$Position[which(indel_positions$Seq=="Rep")] )
          coords_germ=setdiff(coords_germ,indel_positions$Position[which(indel_positions$Seq=="Germ")] )
        } else if (mode == "AA"){
          coords_rep=setdiff(coords_rep,
                             unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Rep"),
                                                                                which(indel_positions$AA_Codon_full>1))]))
          coords_germ=setdiff(coords_germ,
                              unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Germ"),
                                                                                 which(indel_positions$AA_Codon_full>1))]))
        } else if (mode=="Codon") {
          coords_rep=setdiff(coords_rep,
                             unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Rep"),
                                                                                which(indel_positions$AA_Codon_full>1))]))
          coords_germ=setdiff(coords_germ,
                              unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Germ"),
                                                                                 which(indel_positions$AA_Codon_full>1))]))
        }
      }




      # if(length(coords_rep) != length(coords_germ)) {
      #   print("Unusual")
      # }

      differences=which(rep_seq[coords_rep] != germ_seq[coords_germ])

      differences=differences[1:(length(differences)-length(which(c(is.na(rep_seq[coords_rep][differences]), is.na(germ_seq[coords_germ][differences])))))]
      tmp_counts=compare_diffs(elements_rep=rep_seq[coords_rep][differences],
                               elements_germ=germ_seq[coords_germ][differences],
                               elements=elements,
                               region=region)



      ### lets add transversions and transitions
      if (mode == "NT") {
        if (sum(tmp_counts) != 0) {
          transition_counts <- sum(
            tmp_counts[which(
              grepl(paste(transition_muts, collapse="|") , names(tmp_counts))
            )]
          )
          transversion_counts <- sum(tmp_counts) - transition_counts
          if (transversion_counts == 0) {
            Ratio <- 10
          } else {
            Ratio <- transition_counts/transversion_counts
          }

        } else {
          transition_counts <- 0
          transversion_counts <- 0
          Ratio <- 0
        }
        tr_tmp <- c(transition_counts, transversion_counts, Ratio)
        names(tr_tmp) <- paste(
          region, c("Transitions", "Transversions", "Ratio_Transitions-Transversions"),
          sep = "_"
        )

        if(length(coords_rep)!=0 && length(coords_germ)!=0) {
          tmp_counts_norm=tmp_counts/length(coords_rep)
        } else {
          tmp_counts_norm=tmp_counts
        }

        names(tmp_counts) =paste(names(tmp_counts), "count", sep="_")
        names(tmp_counts_norm) =paste(names(tmp_counts_norm), "norm", sep="_")


        counts <- c(counts, tmp_counts, tmp_counts_norm, tr_tmp)

      } else if(mode =="AA") {
        tmp_counts_2=tmp_counts
        combinations <- expand.grid(names(group_list), names(group_list))
        combinations <- combinations[which(combinations$Var1 != combinations$Var2),
        ]

        combinations$Counts=0
        for (i in which(tmp_counts_2 !=0)) {
          from=strsplit(names(tmp_counts_2)[i], split="_")[[1]][5]
          to=strsplit(names(tmp_counts_2)[i], split="_")[[1]][3]

          from_type=names(group_list)[grep(from, group_list)]
          to_type=names(group_list)[grep(to, group_list)]

          indexes_combinations=intersect(which(combinations$Var1 %in% from_type),
                                         which(combinations$Var2 %in% to_type))
          combinations[indexes_combinations, "Counts"]=combinations[indexes_combinations, "Counts"]+tmp_counts_2[i]



        }
        tmp_type_counts=combinations$Counts
        names(tmp_type_counts)=paste(region, paste(combinations$Var1, combinations$Var2, sep="_to_"), sep="_")

        tmp_type_counts=tmp_type_counts[sort(names(tmp_type_counts))]

        if(length(coords_region)!=0) {
          tmp_counts_norm=tmp_counts/length(coords_region)
          tmp_type_counts_norm=tmp_type_counts/length(coords_rep)
        } else {
          tmp_counts_norm=tmp_counts
          tmp_type_counts_norm=tmp_type_counts
        }
        names(tmp_counts)=paste(names(tmp_counts), "count", sep="_")
        names(tmp_counts_norm)=paste(names(tmp_counts_norm), "norm", sep="_")
        names(tmp_type_counts)=paste(names(tmp_type_counts), "count", sep="_")
        names(tmp_type_counts_norm)=paste(names(tmp_type_counts_norm), "norm", sep="_")
        # counts <- c(counts, tmp_counts, tmp_counts_norm, tmp_type_counts, tmp_type_counts_norm)

        counts <- c(counts, tmp_counts, tmp_type_counts)
      }  else if(mode =="Codon") {

        coords_rep=which(names(coord_regions[[1]]) %in% location)
        coords_germ=which(names(coord_regions[[2]]) %in% location)

        if(indel){
          # coords_rep=setdiff(coords_rep,
          #                    unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Rep"),
          #                                                                       which(indel_positions$AA_Codon_full>1))]))
          # coords_germ=setdiff(coords_germ,
          #                     unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Germ"),
          #                                                                        which(indel_positions$AA_Codon_full>1))]))
          #
          # index_1_rep=c(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Rep"),
          #                                                           which(indel_positions$AA_Codon_full==1))],
          #               coords_rep[which(coords_germ==unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Germ"),
          #                                                                                     which(indel_positions$AA_Codon_full==1))]))]
          # )
          # index_1_germ=c(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Germ"),
          #                                                           which(indel_positions$AA_Codon_full==1))],
          #                coords_germ[which(coords_rep==unique(indel_positions$AA_Codon_position[intersect(which(indel_positions$Seq=="Rep"),
          #                                                                                     which(indel_positions$AA_Codon_full==1))]))]
          # )
          #
          # coords_rep=setdiff(coords_rep,
          #                    index_1_rep)
          # coords_germ=setdiff(coords_germ,
          #                     index_1_germ)

          rep_seq=as.character(Biostrings::codons(Biostrings::DNAString(mut_rep_seq)))
          germ_seq=as.character(Biostrings::codons(Biostrings::DNAString(mut_germ_seq)))
          coords_germ=coords_rep
        }



        # if(length(coords_rep) != length(coords_germ)) {
        #   print("Unusual")
        # }

        differences=which(rep_seq[coords_rep] != germ_seq[coords_germ])

        if(length(differences)==0) {
          silent=0
          replacement=0
          Ratio=0
          total=0
        } else {
          differences=differences[1:(length(differences)-length(which(c(is.na(rep_seq[coords_rep][differences]), is.na(germ_seq[coords_germ][differences])))))]

          type_mut=sapply(differences, function(z) if(Biostrings::GENETIC_CODE[match(rep_seq[coords_rep][z],
                                                                         names(Biostrings::GENETIC_CODE))] ==
                                                      Biostrings::GENETIC_CODE[match(germ_seq[coords_germ][z],
                                                                         names(Biostrings::GENETIC_CODE))]) {
            "Silent"
          } else {
            "Replacement"
          })
          num_muts=  sapply(differences,
                            function(z)  adist(rep_seq[coords_rep][z],germ_seq[coords_germ][z])[1,1] )

          names(num_muts)=paste(rep_seq[coords_rep][differences],germ_seq[coords_germ][differences], sep="_to_")

          if(length(which(type_mut=="Silent"))>0) {
            silent=sum(sapply(which(type_mut=="Silent"),
                              function(z)
                                sum(num_muts[z]) ))
          } else {
            silent=0
          }

          if(length(which(type_mut!="Silent"))>0) {
            replacement=sum(sapply(which(type_mut!="Silent"),
                                   function(z)
                                     sum(num_muts[z]) ))
          } else{
            replacement=0
          }


          total=replacement+silent
          if (silent == 0) {
            Ratio <- 10
          } else {
            Ratio <- replacement/silent
          }
        }


        count_tmp <- c(silent, replacement, Ratio, total)
        names(count_tmp) <- paste(
          region, c("Silent_muts", "Replacement_muts",
                    "Ratio_Replacement-Silent", "Total_muts"),
          sep = "_"
        )

        if(length(coords_rep)!=0) {
          count_tmp_norm=count_tmp/(length(coords_rep)*3)
        } else {
          count_tmp_norm=count_tmp
        }

        names(count_tmp)=paste(names(count_tmp), "counts", sep="_")
        names(count_tmp_norm)=paste(names(count_tmp_norm), "norm", sep="_")

        names(tmp_counts)=paste(names(tmp_counts), "count", sep="_")

        if(region=="AA_Whole") {
          counts <- c(counts, tmp_counts, count_tmp, count_tmp_norm)
        } else {
          counts <- c(counts, count_tmp, count_tmp_norm)
        }

      }

    }

  }



  ##to_check
  if (mode == "Codon") {
    counts_full <- list(
      counts = counts, rep_codon_count = counts_per_sequence$Rep, germ_codon_count = counts_per_sequence$Germ
    )
  } else {
    counts_full <- counts
  }
  return(counts_full)
}

#' 2_Feature_determination
#'
#' @description A utils function
#'
#' @return The return value, if any, from executing the utility.
#' @noRd
compare_diffs<- function(elements_rep, elements_germ, elements, region){

  m_vec <- paste0(elements_rep, elements_germ)
  m_vec <- m_vec[which(!grepl("NA",m_vec))]
  combinations <- expand.grid(elements, elements)
  combinations <- combinations[which(combinations$Var1 != combinations$Var2),
  ]

  counts <- rep(0, nrow(combinations))
  names(counts) <- paste(combinations[, 1], combinations[, 2], sep = "_to_")
  combinations <- paste0(combinations[, 1], combinations[, 2])

  table_counts <- table(m_vec)
  indexes <- match(combinations, names(table_counts))
  counts[which(!is.na(indexes))] <- table_counts[indexes[which(!is.na(indexes))]]
  names(counts) <- paste(
    region, names(counts),
    sep = "_"
  )

  counts=counts[sort(names(counts))]
  return(counts)

}

## TESTS: translate_fun #####

test_that("Basic translation works and prioritizes first ORF, AAA-> K", {
  expect_equal(translate_fun("AAAT", ORF_begins=1)[[1]], "K")
  expect_equal(translate_fun("AAAT", ORF_begins=1)[[2]], 1)
})

test_that("Basic translation works using the ORF_begins options", {
  expect_equal(translate_fun("TAAA", ORF_begins=1)[[1]], "NO_GOOD_ORF")
  expect_equal(translate_fun("TAAA", ORF_begins=1)[[2]], 0)
  expect_equal(translate_fun("TAAA", ORF_begins=0)[[1]], "K")
  expect_equal(translate_fun("TAAA", ORF_begins=0)[[2]], 2)
  expect_equal(translate_fun("TATAA",ORF_begins=1)[[1]], "Y")
  expect_equal(translate_fun("TATAA",ORF_begins=1)[[2]], 1)
})

test_that("Gives back empty AAs, A-> ", {
  expect_equal(translate_fun("A")[[1]], "")
  expect_equal(translate_fun("A")[[2]], "")
})

#' Inverted versions of in, is.null and is.na
#'
#' @noRd
#'
#' @examples
#' 1 %not_in% 1:10
#' not_null(NULL)
`%not_in%` <- Negate(`%in%`)

not_null <- Negate(is.null)

not_na <- Negate(is.na)

"%!in%" <- function(x, y) !(x %in%
    y)

#' Removes the null from a vector
#'
#' @noRd
#'
#' @example
#' drop_nulls(list(1, NULL, 2))
drop_nulls <- function(x) {
    x[!sapply(x, is.null)]
}

#' If x is `NULL`, return y, otherwise return x
#'
#' @param x,y Two elements to test, one potentially `NULL`
#'
#' @noRd
#'
#' @examples
#' NULL %||% 1
"%||%" <- function(x, y) {
    if (is.null(x)) {
        y
    } else {
        x
    }
}

#' If x is `NA`, return y, otherwise return x
#'
#' @param x,y Two elements to test, one potentially `NA`
#'
#' @noRd
#'
#' @examples
#' NA %|NA|% 1
"%|NA|%" <- function(x, y) {
    if (is.na(x)) {
        y
    } else {
        x
    }
}

#' Typing reactiveValues is too long
#'
#' @inheritParams reactiveValues
#' @inheritParams reactiveValuesToList
#'
#' @noRd
rv <- function(...) shiny::reactiveValues(...)
rvtl <- function(...) shiny::reactiveValuesToList(...)


#' Check parameters in 3. are properly selected
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'    A function to perform PCA over a FBM object.
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
validate_before_run <- function(work_as_categories, use_sharedVDJ, VJ_included,
                                VDJ_maximize_clones, clonal_group) {

  if ((use_sharedVDJ) && (work_as_categories)) {

    if (length(VJ_included)==0) {
      showNotification("Select minimum a VJ combination", type = "error")
      return(FALSE)
    }
    if ((VDJ_maximize_clones) && length(clonal_group)==0) {
      showNotification("Create a clonal definition", type = "error")
      return(FALSE)
    }
  }

  TRUE
}

test_that("not_in works", {
  expect_true(1 %not_in% 2:10)
  expect_false(1 %not_in% 1:10)
})

test_that("not_null works", {
  expect_true(not_null(1))
  expect_false(not_null(NULL))
})

test_that("not_na works", {
  expect_true(not_na(1))
  expect_false(not_na(NA))
})

test_that("drop_nulls works", {
  expect_equal(
    drop_nulls(
      list(x = NULL, y = 2)
    ),
    list(y = 2)
  )
})

test_that("%||% works", {
  expect_equal(
    NULL %||% 1,
    1
  )
  expect_equal(
    2 %||% 1,
    2
  )
})

test_that("%|NA|% works", {
  expect_equal(
    NA %|NA|% 1,
    1
  )
  expect_equal(
    2 %|NA|% 1,
    2
  )
})

test_that("rv and rvtl work", {
  expect_true(
    inherits(rv, "function")
  )
  expect_true(
    inherits(rvtl, "function")
  )

  rv_test_1 <- rv(a = "a", b = 2)
  rv_test_2 <- reactiveValues(a = "a", b = 2)
  shiny::reactiveConsole(TRUE)
  expect_identical(rv_test_1$a, rv_test_2$a)
  expect_identical(rv_test_1$b, rv_test_2$b)
  expect_identical(
    rvtl(rv_test_2),
    shiny::reactiveValuesToList(rv_test_1)
  )
  shiny::reactiveConsole(FALSE)
})

#' The application User-Interface
#'
#' @param request Internal parameter for `{shiny}`.
#'     DO NOT REMOVE.
#' @import shiny
#' @import plotly
#' @import shinycssloaders
#' @import shinyFiles
#' @import shinymanager
#' @import shinythemes
#' @import upsetjs
#' @import utils
#' @import shinymeta
#' @import fresh
#' @noRd
app_ui <- function(request) {
    shiny::tagList(
        golem_add_external_resources(),
        bs4Dash::bs4DashPage(
          freshTheme =fresh::create_theme(
            bs4dash_vars(
              body_bg = "#2D4552",
              navbar_light_color = "#09BC8A",
              navbar_light_active_color = "#f4a261",
              navbar_light_hover_color = "#e76f51"
            ),

            bs4dash_yiq(
              contrasted_threshold = 10,
              text_dark = "#f8f9fa",
              text_light = "#2D4552"
            ),
            bs4dash_layout(
              main_bg = "#f8f9fa"
            ),
            bs4dash_sidebar_light(
              bg = "#2D4552",
              color = "#f8f9fa",
              hover_color = "#09BC8A",
              submenu_bg = "#2D4552",
              submenu_color = "#f8f9fa",
              submenu_hover_color = "#09BC8A"
            ),
            bs4dash_status(
              primary = "#2D4552", danger = "#e76f51", light = "#f8f9fa",
              secondary = "#09BC8A", warning="#f4a261"
            ),
            bs4dash_color(
              gray_900 = "#2D4552", white = "#f8f9fa"
            ),
            bs4dash_button(
              default_background_color = "#09BC8A",
              default_color="#f8f9fa"
            )
          ),
          header= bs4Dash::bs4DashNavbar(
              title=bs4Dash::bs4DashBrand(title="Solution", color = "primary",
                                 image = "www/favicon.png", opacity = 1),
              leftUi = tagList(
                # bs4Dash::dropdownMenu(
                #   badgeStatus = "info",
                #   type = "tasks",
                #   bs4Dash::taskItem(
                #     inputId = "triggerAction3",
                #     text = "My progress",
                #     color = "orange",
                #     value = 10
                #   )
                # )
              ),
              controlbarIcon = icon("circle-plus"),
              rightUi = bs4Dash::dropdownMenu(
                badgeStatus = "danger",
                type = "messages",
                bs4Dash::messageItem(
                  inputId = "triggerAction1",
                  message = "Have fun, AbSolutely! Do not forget to check the Help tab on each step",
                  from = "Rodrigo Garcia-Valiente",
                  image = "www/img/Rodrigo.jpg",
                  time = "today",
                  color = "lime"
                )
              )
            ),
            sidebar =bs4Dash::bs4DashSidebar(
              skin = "light",
              bs4Dash::bs4SidebarMenu(
                  id = "sidebar",
                  bs4Dash::bs4SidebarMenuItem("Home",
                           tabName = "home",
                           icon = bs4Dash::ionicon(name ="home"),
                           selected=T),
                  bs4Dash::bs4SidebarMenuItem(
                    "AbSolution",
                    icon =  bs4Dash::ionicon(name ="analytics"),
                    tabName = "pipeline"
                ),
                bs4Dash::bs4SidebarMenuItem(
                  "Export results",
                  icon =  bs4Dash::ionicon(name ="download"),
                  tabName = "export_results"
                ),
                bs4Dash::bs4SidebarMenuItem(
                  "Manual",
                  icon =  bs4Dash::ionicon(name ="map"),
                  tabName = "manual"
                ),
                bs4Dash::bs4SidebarMenuItem(
                    "About us", icon = bs4Dash::ionicon(name ="information-circle"),
                    tabName = "About"
                )
              )
            ),
          body =bs4Dash::bs4DashBody(
                shinyjs::useShinyjs(),
                tags$head(tags$link(rel = "shortcut icon",
                                    href = "www/img/favicon.ico")),
                tags$style(HTML(".fade:not(.show) {  opacity: 1; !important}"),
                           '.popover ul {padding: 15px; text-align: justify;}',
                           '.popover-content {text-align: justify;}'),
                # shinyDashboardThemes(theme = "poor_mans_flatly"),
                bs4Dash::bs4TabItems(
                  bs4Dash::tabItem(
                    tabName = "home", fluidRow(
                      bs4Dash::userBox(
                        title = NULL, type = 2, src = NULL, color = "#FFA500", width = 12,
                        boxToolSize = "lg", closable = F, collapsible = F, uiOutput("AbLogo"),
                        footer = HTML(
                          "<strong>AbSolution - Interactive Immune Repertoire Feature Analysis</strong>
                              <br><br>
                            <p align='justify'>
                              AbSolution is a user-friendly interactive platform for exploring B-cell and T-cell immune repertoires based on their sequence features.
                              It allows researchers to examine a wide range of physicochemical, structural, and mutation-related properties across receptor regions, clonotypes, samples, and conditions, helping to reveal patterns of immune selection and diversity.
                              By combining robust statistical analysis, intuitive visualizations, and built-in reproducibility through the ENCORE framework, AbSolution makes immune repertoire analysis accessible, transparent, and fully reproducible for both computational and experimental immunologists.
                              <br><br>
                              Use the menu on the left to navigate through the application:
                              <br><br>
                              <strong>AbSolution</strong> - Launch the interactive application to explore your data. A dedicated <em>Help</em> subtab provides personalized, step-by-step guidance for your analysis, and additional contextual help is available via the help button in the top-right corner.
                              <br>
                              <strong>Export results</strong> - Download your analysis and set up your project in a fully reproducible, ENCORE-compliant project workspace, including data, code, results, and documentation.
                              <br>
                              <strong>Manual</strong> - Access detailed documentation, tutorials, and practical examples.
                              <br>
                              <strong>About us</strong> - Learn more about the team behind AbSolution and our work in computational immunology and reproducible research.
                          <br><br><strong>Contact</strong>: <a href='mailto:r.garciavaliente@amsterdamumc.nl'>Rodrigo Garcia-Valiente</a> and <a href='mailto:a.h.vankampen@amsterdamumc.nl'> Antoine van Kampen</a>
                          </p>"
                      ),
                    HTML("<br><br>"),
                    textInput("username", "User name", "User", width="30%"),
                    textInput("usermail", "Contact email", "Email", width="30%")
                    )
                  )
                ),
                bs4Dash::tabItem(
                    tabName = "pipeline", h2("Analyze and explore your data"),

                  bs4Dash::tabBox(id = "menutabset", width=12, collapsible = F,

                      # 0.Project information  ######
                      tabPanel(
                        "0.Project information",
                          wellPanel(
                            fluidRow(

                            column(9,
                            h3("Upload your sample information table"),
                            bs4Dash::tooltip(
                              radioButtons(
                                inputId="TCRBCR_input_file",
                                label="The repertoire to analyze is:",
                                choiceNames = list("BCR", "TCR"),
                                choiceValues = list("BCR", "TCR"),
                                selected = "BCR"
                              ),
                              title = "TCR and BCR repertoires should be analyzed separatedly.",
                              placement = "bottom"
                            ) ,
                            bs4Dash::tooltip(radioButtons(
                              inputId="radiobutton_input_file",
                              label= "The input file type is:",
                              choiceNames = list("AIRR-Seq format"),
                              choiceValues = list("airrseq"),
                              selected = "airrseq"
                               ),
                              title = "Data should follow the AIRR-Seq format https://docs.airr-community.org/en/latest/datarep/rearrangements.html",
                              placement = "top"
                            ),
                            uiOutput("Conditional_Action_Filetype_description"),
                            uiOutput("Conditional_Action_Filetype_upload"),

                            HTML("<br>"),
                            DT::dataTableOutput("raw_sample_file_out"),
                            HTML("<br><br>"),
                            textAreaInput("user_0_comments",
                                          "User comments",
                                          "",
                                          width = "80%"),
                            HTML("<br><br><h3>Select work folder</h3>"),

                           shinyDirButton(
                              id="base_folder",
                              label="Select the folder where the data will be saved",
                              title = "Please select the work folder:", buttonType = "default",
                              class = NULL, icon = icon("folder", lib = "font-awesome"),
                              multiple = F
                              ),
                            HTML("<br><br><br><h3>Next step</h3>"),
                           HTML(
                             "If you have previous files, you don't have to redo all the steps. Process and upload your files where/when needed. But do provide the sample information.<br> <br>"
                           ),
                            uiOutput("Conditional_Action_Move_to_1"),
                            HTML("<br><br>"),
                            uiOutput("Conditional_Action_Move_to_Analysis")
                        ),
                          column(3,
                                 HTML("<br><br>"),
                                 uiOutput("Demo_Analysis")
                          )
                        )
                      )
                      ),
                      # 1.AIRR-Seq conversion  ######
                      tabPanel(
                        "1.AIRR-Seq conversion", wellPanel(
                          h3("Upload your AIRR-Seq  files"),
                          HTML("Select the main folder that contains the different AIRR-Seq files.<br> <br>"),
                          shinyDirButton(
                            "preinfolder_AIRR", "Select main folder", title = "Please select the main folder:",
                            buttonType = "default", class = NULL, icon = icon("folder", lib = "font-awesome"),
                            multiple = F
                          ),
                          HTML("<br>"),
                          DT::dataTableOutput("folder_information_AIRR"),
                          HTML("<br>"),
                          h3("Set parameters for germline reconstruction"),
                          bs4Dash::tooltip(
                            shinyWidgets::materialSwitch(
                              inputId="Dgene_reconstruct_airr",
                              label="Reconstruct CDR3 using the assigned D gene sequence instead of keeping that part of the CDR3 as is in the repertoire.",
                              value = T
                            ),
                            title= paste0(
                              "D genes, located in the CDR3, are of short length",
                              " and due to NPJ additions they are",
                              " hard to idenfity. You can decide to reconstruct it",
                              " using the inferred D germline and NPJs ",
                              " or keep the fragment of the sequence as they appear",
                              " in the repertoire"
                            ),
                            placement = "right"
                          ),
                          bs4Dash::tooltip(
                            shinyWidgets::materialSwitch(
                              inputId="FWR1partial_airr",
                              label= "Is there a partial FWR1 in the sequences?",
                              value = FALSE
                            ),
                            title= paste0(
                              "Depending on the primer location, sometimes the BCR",
                              "/TCR sequence is not fully sequenced. If this ",
                              "happens, AbSolution may be biased as not all ",
                              "sequences will have the same starting point. ",
                              "Activate this option if the FWR1 is not fully sequenced",
                              ". The FWR1 will be removed from the analysis"
                            ),
                            placement = "right"
                          ),
                          bs4Dash::tooltip(
                            shinyWidgets::materialSwitch(
                              inputId="FWR4partial_airr",
                              label="Is there a partial FWR4 in the sequences?",
                              value = FALSE
                            ),
                            title= paste0(
                              "Depending on the primer location, sometimes the BCR",
                              "/TCR sequence is not fully sequenced. If this ",
                              "happens, AbSolution may be biased as not all ",
                              "sequences will have the same ending point. ",
                              "Activate this option if the FWR4 is not fully sequenced.",
                              " The FWR4 will be removed from the analysis."
                            ),
                            placement = "right"
                          ),
                          bs4Dash::tooltip(
                            shinyWidgets::materialSwitch(
                              inputId="C_region_included_airr",
                              label="Is part or totatily of the pre-FWR1 or the C region included in the sequence?",
                              value = FALSE
                            ),
                            title=paste0(
                              "Depending on the primer location, sometimes the BCR",
                              "/TCR sequence pre-FWR1 or C region (or UMIs) are included in the",
                              "sequence. If this  happens, AbSolution may be biased",
                              " due to the sequence extra composition",
                              "Activate this option if the preFWR1/C gene/UMIs are present. ",
                              "The FWR4 will be removed from the analysis."
                            ),
                            placement="right"
                          ),

                          HTML("<br> Preprocessing preview (3 sequences) - Repertoire and germline extraction"),
                          DT::dataTableOutput("DT_example_parsed_sequence"),
                          HTML("<br>"),
                          textAreaInput("user_1_comments",
                                        "User comments",
                                        "",
                                        width = "80%"),
                          h3("Process AIRR-Seq files and continue to the next step"),
                          HTML("<br>"),
                          uiOutput("Conditional_Action_Preprocess_AIRR"),
                          shinyjs::hidden(
                            div(
                              id = "pb_AIRR_vis", shinyWidgets::progressBar(
                                id = "pb_AIRR", value = 0, total = 100, title = "Processing...",
                                display_pct = TRUE
                              )
                            )
                          ),
                          uiOutput("Conditional_Action_Move_to_2_AIRR")
                        )
                      ),
                      # 2.Sequence feature determination  ######
                      tabPanel(
                        "2.Sequence feature determination", wellPanel(
                          textAreaInput("user_2_comments",
                                        "User comments",
                                        "",
                                        width = "80%"),
                          actionButton(
                            "Feature_determination", "Feature determination", icon("forward"),
                            style = "color: #fff; background-color: #18bc9c"
                          ),
                          div(
                            id = "pb_Feature_vis", shinyWidgets::progressBar(
                              id = "pb_Feature", value = 0, total = 100, title = "Processing...",
                              display_pct = TRUE
                            )
                          ),
                          HTML("<br>"),
                          uiOutput("make_dummy_2"),
                          conditionalPanel(
                            "input.dummy_dataset == true", selectizeInput(
                              "make_dummy_menu", "Make a dummy version of the dataset:",
                              choices = NULL, multiple = F, options = list(plugins = list("remove_button"))
                            )
                          ),
                          conditionalPanel(
                            "input.make_dummy_menu.length > 0 & input.dummy_dataset == true",
                            numericInput(
                              "dummy_ratio", "How many sequences per original sequence?",
                              1, min = 1, max = 10
                            ),
                            actionButton(
                              "final_dummy",
                              "Produce the dummy dataset (click and wait until the table refreshes)",
                              icon("forward"),
                              style = "color: #fff; background-color: #18bc9c"
                            )
                          ),
                          uiOutput("Conditional_Action_Move_to_3")
                        )
                      ),
                      #Steps 1&2. Select your FBM and associated files  ######
                      tabPanel(
                        "Steps 1&2. Select your FBM and associated files", wellPanel(
                          shinyDirButton(
                            "FBM_folder", "Select the folder with your .rds and .bk files",
                            title = "Please select the bigmem folder:", buttonType = "default",
                            class = NULL, icon = icon("folder", lib = "font-awesome"),
                            multiple = F
                          ),
                          HTML("<br>"),
                          DT::dataTableOutput("folder_information_AIRR_steps_1_to_3"),
                          HTML("<br><br><br>"),
                          uiOutput("make_dummy"),
                          conditionalPanel(
                            "input.dummy_dataset == true", selectizeInput(
                              "make_dummy_menu", "Make a dummy version of the dataset:",
                              choices = NULL, multiple = F, options = list(plugins = list("remove_button"))
                            )
                          ),
                          conditionalPanel(
                            "input.make_dummy_menu.length > 0 & input.dummy_dataset == true",
                            numericInput(
                              "dummy_ratio", "How many sequences per original sequence?",
                              1, min = 1, max = 10
                            ),
                            actionButton(
                              "final_dummy",
                              "Produce the dummy dataset (click and wait until the table refreshes)",
                              icon("forward"),
                              style = "color: #fff; background-color: #18bc9c"
                            )
                          ),
                          HTML("<br><br><br>"),
                          uiOutput("Conditional_Action_Move_to_Analysis_Real")
                        )
                      ),
                      # 3.Dataset exploration and variable selection ######
                      tabPanel(
                        "3.Dataset exploration and variable selection",
                        bs4Dash::tabBox(id = "3menu", width=12, collapsible = F,
                          tabPanel(
                            "Menu", fluidRow(column(
                              4, wellPanel(
                                HTML(
                                  "Explore the dataset using both plots. The dataset and variables that you will use for the feature selection process are represented in the Selection plot. <br><br>"
                                ),
                                h3("1. Select your sequences"),
                                bs4Dash::tooltip(
                                  selectizeInput(
                                    inputId="use_what",
                                    label = "Include sequences:",
                                    choices = c("Repertoire", "Reconstructed germline"),
                                    selected = c("Repertoire"),
                                    multiple = TRUE, options = list(plugins = list("remove_button"))
                                  ),
                                  title="Work with the sequenced sequences (repertoire) and/or their inferred germlines.",
                                  placement="right"
                                ),
                                bs4Dash::tooltip(
                                  selectizeInput(
                                    inputId="use_productive_or_not",
                                    label = "That are:",
                                    choices = c("Productive"),
                                    selected = c("Productive"),
                                    multiple = TRUE, options = list(plugins = list("remove_button"))
                                  ),
                                  title="Use productive and/or non-productive sequences. Currently only productive sequences are supported.",
                                  placement="right"
                                ),
                                uiOutput("Sample_selection"),
                                uiOutput("Chain_selection"),
                                uiOutput("Type_selection"),
                                bs4Dash::tooltip(
                                  sliderInput(
                                    inputId = "Rmut_filter",
                                    label = "Include only sequences with a number of replacement (R) mutations between",
                                    min = 0, max = 100, value = c(0, 100),
                                    step = 1
                                  ),
                                  title="Select minimum and maximum number of R mutations. This allows to e.g. filter naive sequences or outliers.",
                                  placement="right"
                                ),

                                uiOutput("VJ_selection"),
                                HTML("<br>"),
                                h3("2. Select your variables"),
                                HTML(
                                  "<i>NOTE: Variables with NAs/Infinite/Zero variance are automatically removed.</i><br><br>"
                                ),

                                bs4Dash::tooltip(
                                  selectizeInput(
                                    inputId="my_regions",
                                    label = "Select the region(s)",
                                    choices = c(
                                      "Whole", "FWR1", "CDR1", "FWR2", "CDR2", "FWR3",
                                      "CDR3", "FWR4"
                                    ),
                                    selected = c("CDR3"),
                                    multiple = TRUE, options = list(plugins = list("remove_button"))
                                  ),
                                  title= "Select which regions (and/or the whole Fab sequence) to include in the analysis.",
                                  placement="right"
                                ),
                                bs4Dash::tooltip(
                                  selectizeInput(
                                    inputId="my_var_elements",
                                    label = "Study features from",
                                    choices = c("NT", "AA"),
                                    selected = c("NT", "AA"),
                                    multiple = TRUE, options = list(plugins = list("remove_button"))
                                  ),
                                  title="Work with features calculated according to the AA and/or NT sequences",
                                  placement="right"
                                ),
                                bs4Dash::tooltip(
                                  selectizeInput(
                                    inputId="my_vars",
                                    label = "Select the variable(s) class(es)",
                                    choices = c(
                                      "Length", "Composition", "NGly sites", "Hot/Cold motifs",
                                      "Insertions",
                                      "Deletions", "Transitions and transversions",
                                      "Replacement and silent mutations", "Mutations from X to Y",
                                      "Peptide features"
                                    ),
                                    selected = c(
                                      "Length", "Composition", "NGly sites", "Hot/Cold motifs",
                                      "Peptide features"
                                    ),
                                    multiple = TRUE, options = list(plugins = list("remove_button"))
                                  ),
                                  title="Select or unselect groups of variables to include in the analysis.",
                                  placement="right"
                                ),
                                bs4Dash::tooltip(
                                  selectizeInput(
                                    inputId="my_vartypes",
                                    label = "Select the variable type(s)",
                                    choices = c("Baseline", "Germline diff"),
                                    selected = c("Baseline"),
                                    multiple = TRUE, options = list(plugins = list("remove_button"))
                                  ),
                                  title="Include variables measured at the sequence and/or its difference between the sequence and its germline.",
                                  placement="right"
                                ),

                                uiOutput("individual_variables"),
                                bs4Dash::tooltip(
                                  shinyWidgets::materialSwitch(
                                    inputId="use_UMAP",
                                    label = "Also represent the data in a UMAP (increases computational time)",
                                    value = FALSE, width = NULL, status = "primary"
                                  ),
                                  title="The UMAP will only be calculated if the data fits the RAM memory.",
                                  placement="right"
                                ),

                                HTML("<br>"),
                                h3("3. Define the groups to study"),


                                bs4Dash::tooltip(
                                  shinyWidgets::materialSwitch(
                                    inputId="work_as_categories",
                                    label = "I want to compare between groups",
                                    value = FALSE, width = NULL, status = "primary"
                                  ),
                                  title="Do you want to work only with certain groups of interest?",
                                  placement="right"
                                ),
                                uiOutput("Group_selection"),
                                uiOutput("Group_comparison"),
                                hr(),
                                div(
                                  style = "margin-left: 20px;",
                                  shinyWidgets::materialSwitch(
                                    inputId="use_sharedVDJ",
                                    label = "Include only sequences with VJ genes present in all the groups",
                                    value = FALSE, width = NULL, status = "primary"
                                  )
                                ),
                                div(
                                  style = "margin-left: 40px;",
                                  bs4Dash::tooltip(
                                    shinyWidgets::materialSwitch(
                                      inputId="VDJ_normalized_per_size",
                                      label = "Use the same number of sequences for each VJ combination on each group",
                                      value = T, width = NULL, status = "primary"
                                    ),
                                    title="This normalizes the feature selection and avoids overrepresentation of one group",
                                    placement="right"
                                  )
                                ),
                                div(
                                  style = "margin-left: 60px;",
                                  bs4Dash::tooltip(
                                    shinyWidgets::materialSwitch(
                                      inputId="VDJ_maximize_clones",
                                      label = "Maximize the number of clones for each VJ combination on each group",
                                      value = T, width = NULL, status = "primary"
                                    ),
                                    title="This normalizes the feature selection and avoids overrepresentation of major clones",
                                    placement="right"
                                  ),
                                  uiOutput("my_clone_def")
                                ),

                                HTML("<br>"),
                                div(
                                  style = "margin-left: 60px;",
                                  shinyWidgets::materialSwitch(
                                    inputId="VDJ_normalized_per_sample",
                                    label = "And apply these rules also to each individual sample",
                                    value = F, width = NULL, status = "primary"
                                  )
                                ),
                                div(
                                  style = "margin-left: 40px;",
                                  uiOutput("VDJ_subsetting")
                                ),
                                # selectizeInput(
                                #   inputId="my_clone_def",
                                #   label = "Select the clone definition",
                                #   choices = c(NULL),
                                #   selected = c(NULL),
                                #   multiple = F, options = list(plugins = list("remove_button"))
                                # ),
                                shinyWidgets::materialSwitch(
                                  inputId="use_univlog",
                                  label = "Include only features that are correlated to the groups (0/1)" ,
                                  value = FALSE, width = NULL, status = "primary"
                                ),
                                selectizeInput(
                                  inputId="pval_type",
                                  label = "Select the P-value to use" ,
                                  choices = c("Raw p-value",
                                              "Corrected by Bonferroni method",
                                              "Corrected by Benjamini & Hochberg method",
                                              "Corrected by Benjamini & Yekutieli method"),
                                  selected = c("Corrected by Benjamini & Yekutieli method"),
                                  multiple = F, options = list(plugins = list("remove_button"))
                                ),
                                numericInput(
                                  inputId="pval_cutoff",
                                  label = "P-value cut-off (<=):", 0.05,
                                  min = 0, max = 1
                                ),
                                numericInput(
                                  inputId="estimate_cutoff",
                                  label = "Estimate cut-off (absolute value, >=):",
                                  1, min = 0
                                ),
                                selectizeInput(
                                  inputId="number_selected_vars",
                                  label = "How many selected features to use?",
                                  choices = c(
                                    "All", "Top 10 (according to estimate)",
                                    "Top 50 (according to estimate)"
                                  ),
                                  selected = c("All"),
                                  multiple = F, options = list(plugins = list("remove_button"))
                                ),
                                h3("4. Update your changes"),
                                actionButton(
                                  "Update_selection", "Update Selection set, used for all the analyses",
                                  icon("forward"),
                                  style = "color: #fff; background-color: #18bc9c"
                                ),
                                actionButton(
                                  "Update_exploration", "Update Exploration set, to compare with Selection",
                                  icon("forward"),
                                  style = "color: #fff; background-color: #CC2936"
                                )
                              )
                            ),
                            column(
                              8, HTML("<br>"),
                              HTML("Use the lasso tool or box select to get the sequence(s) information.<br><br>"),
                              fluidRow(
                                column(
                                  4, HTML("<h2>Selection plot</h2>"),
                                  fluidRow(
                                    column(
                                      6, selectInput(
                                        inputId="Selection_plot_type",
                                        label = "Dimensionality Reduction:",
                                        c(PCA = "PCA")
                                      ),
                                      selectizeInput(
                                        inputId="Selection_plot_type_dim",
                                        label = "Select the (1 to 5) dimensions to plot",
                                        choices = c(1:5),
                                        selected = c(1, 2),
                                        multiple = TRUE, options = list(
                                          plugins = list("remove_button"),
                                          maxItems = 2
                                        )
                                      ),
                                      uiOutput("Plot_color")
                                    )
                                  )
                                ),
                                column(8, plotlyOutput("Selection_plot")%>%
                                         withSpinner(type = 6, color = "#2D4552"),
                                       uiOutput("Selection_plot_error"))
                              ),
                              fluidRow(
                                column(
                                  4, HTML("<br><br><h2>Exploration plot</h2>"),
                                  fluidRow(
                                    column(
                                      6, selectInput(
                                        inputId="Exploration_plot_type",
                                        label = "Dimensionality Reduction:",
                                        c(PCA = "PCA")
                                      ),
                                      selectizeInput(
                                        inputId="Exploration_plot_type_dim",
                                        label ="Select the (1 to 5) dimensions to plot",
                                        choices = c(1:5),
                                        selected = c(1, 2),
                                        multiple = TRUE, options = list(
                                          plugins = list("remove_button"),
                                          maxItems = 2
                                        )
                                      ),
                                      uiOutput("Plot_color_expl")
                                    )
                                  )
                                ),
                                column(8,
                                       HTML("<br><br>"),
                                       plotlyOutput("Exploration_plot")%>%
                                         withSpinner(type = 6, color = "#2D4552"),
                                       uiOutput("Exploration_plot_error"))
                              ),
                              fluidRow(
                                column(6, bs4Dash::infoBoxOutput("RAM_S_Memory_Box", width = 12)),
                                column(6, bs4Dash::infoBoxOutput("RAM_E_Memory_Box", width = 12))
                              ),
                              textAreaInput("user_3_comments",
                                            "User comments",
                                            "",
                                            width = "80%"),
                              DT::dataTableOutput("Ab_table")
                            ))
                          ),
                          tabPanel(
                            "About the variables", uiOutput("about_vars"),
                            bs4Dash::tabBox(id = "varmenu", width=12, collapsible = F,
                              tabPanel(
                                "Used variables", fluidRow(
                                  column(
                                    10, HTML(
                                      "<br>Here you can see which variables are represented on each set.<br><br>The plots can take a few minutes to load!<br><br>"
                                    ),
                                    htmlOutput("used_variables") %>%
                                      withSpinner(type = 6, color = "#2D4552")
                                  )
                                )
                              ),
                              tabPanel(
                                "Variable correlation", HTML(
                                  "<br>Here you can see which variables in the selection set are correlated >=0.8 for their non-zero values. Normalized and raw variables are not considered as correlated in the plot.<br><br>The plots can take a few minutes to load!<br><br>"
                                ),
                                htmlOutput("correlated_variables") %>%
                                  withSpinner(type = 6, color = "#2D4552")
                              ),
                              tabPanel(
                                "PCA loadings", HTML(
                                  "<br>Here you can see the PCA loadings in both PCAs.<br><br>"
                                ),
                                htmlOutput("PCA_loadings") %>%
                                  withSpinner(type = 6, color = "#2D4552")
                              )
                            )
                          ),
                          tabPanel(
                            "About the samples", HTML(
                              "<br>Here you can see information about the samples that are present on the selection set.<br><br>Remember you can have included (reconstructed) germline sequences!"
                            ),
                            htmlOutput("about_samples") %>%
                              withSpinner(type = 6, color = "#2D4552")
                          ),
                          tabPanel(
                            "Add covariables", HTML("<br>Here you can add covariables to the selection dataset!<br><br>"),
                            uiOutput("about_meh"),
                            bs4Dash::tabBox(id = "covarmenu", width=12, collapsible = F,
                              # tabPanel(
                              #   "Numerical covariables (e.g. gene expression)", HTML(
                              #     "Use column names to define the sequence ID (column name Sequence_ID) and the covariables (use your own nomenclature)<br>"
                              #   ),
                              #   fileInput(
                              #     "numerical_covariable_file", "Upload", multiple = F,
                              #     accept = c(".tab", ".csv", ".txt")
                              #   ),
                              #   uiOutput("numerical_covariables_plot")
                              # ),
                              tabPanel(
                                "Categorical covariables (e.g. your own clone IDs or new groupings).",
                                HTML(
                                  "Use column names to define the sequence ID (column name Sequence_ID) and the covariables (use your own nomenclature). Use .csv format.<br>"
                                ),
                                fileInput(
                                  "categorical_covariable_file", "Upload", multiple = F,
                                  accept = c(".csv")
                                ),
                                uiOutput("results_categorical")
                              )
                            )
                          )
                        )
                      ),
                      # 4.Feature exploration ######
                      tabPanel(
                        "4.Feature exploration", wellPanel(
                          HTML("<br>"),
                          HTML(
                            "Here you can inspect several features of BCR and TCR Fab regions per region and per group (e.g. N-glycosylation sites in the FWR3, total number of mutations along the sequence, etc.)."
                          ),
                          fluidRow(
                            column(6,
                                   HTML("<br>"),
                                   selectizeInput(
                                     inputId="plot_feature", label = NULL, choices = c(NULL),
                                     selected = NULL
                                   ),
                                   shinyWidgets::materialSwitch(
                                     inputId="show_reconstructed",
                                     label = "Include reconstructed sequences",
                                     value = FALSE, width = NULL, status = "primary"
                                   ),
                                   shinyWidgets::materialSwitch(
                                     inputId="compare_opposites",
                                     label = "Do a t-test between repertoire and
                                     reconstructed sequences if Cohen's d > 0.2",
                                     value = FALSE, width = NULL, status = "primary"
                                   ),
                                   shinyWidgets::materialSwitch(
                                     inputId="really_hide_points",
                                     label = "Do not plot points (ergo, no interaction)",
                                     value = TRUE,
                                     width = NULL, status = "primary"
                                   ),
                                   shinyWidgets::materialSwitch(
                                     inputId="hide_points",
                                     label = "Hide non-selected data points",
                                     value = TRUE,
                                     width = NULL, status = "primary"
                                   ),
                                   column(6,
                                          colourpicker::colourInput("primary_color", "Select primary colour", "#09BC8A"),
                                          uiOutput("Group_selection_for_feature"))

                            ),
                            column(6,
                                   textAreaInput("user_4_comments",
                                                 "User comments",
                                                 "",
                                                 width = "80%"))
                          ),

                          plotlyOutput("Violin_feature_plot")%>%
                            withSpinner(type = 6, color = "#2D4552"),

                        )
                      ),
                      # 5.Clonal exploration ######
                      tabPanel(
                        "5.Clonal exploration", wellPanel(
                          HTML("<br></br>"),
                          HTML(
                            "Here you can group the sequences per different criteria, see how they behave on each sample and select them so you can mark their subclones in 3."
                          ),
                          hr(), fluidRow(
                            class = "clonal_row", column(
                              6,
                              bs4Dash::tabBox(id = "clonemenu1", width=12, collapsible = F,
                                tabPanel(
                                  "Clonal definition", HTML("<br>"),
                                  uiOutput("clonal_group_output"),
                                  conditionalPanel(
                                    "input.clonal_group != ''",
                                    bs4Dash::tooltip(
                                      numericInput(
                                        inputId="dominance_threshold",
                                        label = "Dominance threshold (%)",
                                        0.5, min = 0, max = 100
                                      ),
                                      title="Relative frequency cut-off to determinate which clones are (non)-dominant.",
                                      placement="right"
                                    ),
                                    shinyWidgets::materialSwitch(
                                      inputId="really_hide_points_clones",
                                      label = "Do not plot points (ergo, no interaction)",
                                      value = TRUE,
                                      width = NULL, status = "primary"
                                    ),
                                    bs4Dash::tooltip(
                                      numericInput(
                                        inputId="filter_clonal_group",
                                        label = "Do not plot clones with a frequency lower than (TO BE IMPLEMENTED)",
                                        0, min = 0, max = 100
                                      ),
                                      title="Hide low-frequency clones to reduce loading times.",
                                      placement="right"
                                    )
                                  )
                                ),
                                tabPanel(
                                  "Calculate new clonal definition", HTML("<br>"),
                                  bs4Dash::tooltip(
                                    selectizeInput(
                                      inputId="new_clonal_group",
                                      label = "Using base definition",
                                      choices = c("VCDR3J"), #, "Reconstructed germline"
                                      selected = c("VCDR3J"),
                                      multiple = F, options = list(plugins = list("remove_button"))
                                    ),
                                    title="Definition to calculate clones. Either by same V-J combination + CDR3 length + similarity or by same VJ combination + same reconstructed germline.",
                                    placement="right"
                                  ),

                                  numericInput(
                                    inputId="identity_clonal_group",
                                    label ="with a % identity of ",
                                    100, min = 100, max = 100
                                  ),

                                  selectizeInput(
                                    inputId="clonal_region_group",
                                    label = "in the region(s)",
                                    choices = c(
                                      # "Whole", "FWR1", "CDR1", "FWR2", "CDR2", "FWR3", "FWR4",
                                      "CDR3"
                                    ),
                                    selected = c("CDR3"),
                                    multiple = F, options = list(plugins = list("remove_button"))
                                  ),
                                  selectizeInput(
                                    inputId="clonal_level_group",
                                    label ="at the sequence level of ",
                                    choices = c("AA", "NT"),
                                    selected = c("AA"),
                                    multiple = F, options = list(plugins = list("remove_button"))
                                  ),
                                  shinyWidgets::materialSwitch(
                                    inputId="calculate_shared_clones",
                                    label ="Contemplate also the possibility of shared clones between samples",
                                    value = FALSE, width = NULL, status = "primary"
                                  ),
                                  actionButton("calculate_new_clone", "Calculate clonal definition")
                                )
                              ),
                              HTML("<br><br>")
                            ),
                            column(6,
                                   textAreaInput("user_5_comments",
                                                 "User comments",
                                                 "",
                                                 width = "80%"))
                          ),
                          plotlyOutput("Violin_plot")%>%
                            withSpinner(type = 6, color = "#2D4552"),
                          bs4Dash::tabBox(id = "clonemenu2", width=12, collapsible = F,
                            tabPanel(
                              "Shared clones", HTML("<br><br>"),
                              upsetjs::upsetjsOutput("upset_plot")%>%
                                withSpinner(type = 6, color = "#2D4552"),
                              HTML("<br>"),
                              hr(), HTML(
                                "In the scatterplot(s) below, you can observe the shared and non-shared clones between the samples involved in the selected intersection of the above UpSet plot. Each comparison is made on a one-to-one basis between the samples."
                              ),
                              uiOutput("coloring_scatterplot_clones"),
                              HTML("<br>"),
                              plotlyOutput("Comparison_plot")%>%
                                withSpinner(type = 6, color = "#2D4552")
                            ),
                            tabPanel("Selected clones",
                                     DT::dataTableOutput("Clonal_table")
                            )
                            # tabPanel("Clonal diversity"),
                            # tabPanel("Clonal VDJ usage"),
                            # tabPanel("Subclonal diversity and networks"),
                            # tabPanel("Phylogenies")
                          )
                        )
                      ),
                      # Help ######
                      tabPanel(
                        "Help", wellPanel(

                          HTML(
                            "Welcome to AbSolution. The programme runs in different steps, each on a tab. For each step, this Help section will explain  what to do. <br><br>"
                          ),
                          # verbatimTextOutput("code"),
                          # h3("About the current step"),
                          uiOutput("HELP_output")

                          # img(
                          #   src = "www/img/AbSolution_scheme.png", height = "100%",
                          #   width = "100%"
                          # )



                        )
                      )
                  )
                  ),
                bs4Dash::tabItem(
                  tabName = "export_results", fluidRow(
                    tabPanel(
                      "Export results", wellPanel(
                        HTML("<br></br>"),
                        HTML(
                          '<p align="justify">
                                Set up a ready-to-use ENCORE-standard project <strong>workspace</strong> that organizes your data, code, results, and documentation in a single, well-structured collection-making your analysis transparent, well documented, and easy to reproduce. Place your input data in the <code>/Data</code> folder, use a dedicated subfolder within <code>/Processing</code> as your working directory, and, after completing a full analysis with AbSolution, export all results to <code>/Processing</code> to keep your project fully traceable and organized.
                          </p>'
                        ),
                        hr(), fluidRow(
                          class = "clonal_row", column(
                            6,
                            bs4Dash::tabBox(id = "clonemenu1", width=12, collapsible = F,
                                            tabPanel(
                                              "Set up a reproducible ENCORE Project",
                                              HTML("If you want to <strong>create a complete <a href='https://www.nature.com/articles/s41467-024-52446-8'>ENCORE</a> project compendium</strong> -a self-contained folder that brings together all parts of your research (data, code, results, and notes) in a reproducible structure-, you can download the latest template here to get started."),
                                              HTML("<br><br>"),
                                              fluidRow(
                                                column(4,
                                                       textInput("sFSS_name", label=NULL,"Project_name", width="80%"),
                                                ),
                                                column(8,
                                                       shinyDirButton(
                                                         id="sFSS_folder",
                                                         label="Select the folder where the ENCORE sFSS will be set-up",
                                                         title = "Please select the work folder:", buttonType = "default",
                                                         class = NULL, icon = icon("folder", lib = "font-awesome"),
                                                         multiple = F
                                                       )
                                                )
                                              ),


                                              HTML("<br>"),
                                              fluidRow(
                                                column(12, align = "center",
                                                       uiOutput("Conditional_Action_downloadsFSS")
                                                )
                                              )


                                            ),
                                            tabPanel(
                                              "Export analysis",
                                              HTML("    <p ><strong>After running a full analysis</strong> with the app, you can download a .zip file that follows the <a href='https://www.nature.com/articles/s41467-024-52446-8'>ENCORE</a> structure. This .zip file contains the following subfolders:</p>
                                                <ul>
                                                  <li><strong>/0_SoftwareEnvironment/R/</strong>: Contains the AbSolution package in the version used for the analysis, a Docker file, and the renv information to reproduce everything.</li>
                                                  <li><strong>/Data/Dataset/</strong>: Contains the dataset used for the analysis.<ul>
                                                  <li><strong>/Raw/</strong>: Contains the sample input files.</li>
                                                  <li><strong>/Meta/</strong>: Contains a file with information about the samples.</li>
                                                  <li><strong>/Processed/</strong>: Contains the files produced during the analysis with AbSolution (sequence and feature information). These files are used for analysis and data exploration.</li></ul></li>
                                                  <li><strong>/Notebook/</strong>: Contains the .Rmd file to generate the figures. This .Rmd file is generated directly from the app's code, capturing the domain logic using shinymeta. Additionally, once exported, it uses relative paths instead of absolute paths, making it easy to knit successfully. The chunks <code>parse_input</code> and <code>feature_calculation</code> are not evaluated unless the user sets <code>eval=TRUE</code>, as they are used to produce the Processed files. For time efficiency, this part is skipped when producing the .zip.</li>
                                                  <li><strong>/Results/</strong>: Contains the HTML file produced from the .Rmd file with the key plots from AbSolution. The HTML file includes all the system information and package versioning details.</li>
                                              </ul>"),
                                              textInput("analysis_name", "Name of the exported .zip", paste("Analysis", Sys.time(), sep="_"), width="30%"),
                                              shinyWidgets::materialSwitch(
                                                "include_data_report", "Include Data subfolder",
                                                value = F, width = "100%", status = "primary"
                                              ),
                                              shinyWidgets::materialSwitch(
                                                "include_docker_renv", "Include 0_SoftwareEnvironment subfolder",
                                                value = F, width = "100%", status = "primary"
                                              ),
                                              fluidRow(
                                                column(12, align = "center",
                                                       uiOutput("export_cond")%>%
                                                         withSpinner(type = 6, color = "#2D4552")
                                                )
                                              )

                                            )

                                            )
                            ),
                            HTML("<br><br>")
                          )
                        ),

                      )
                    )
                ),
                bs4Dash::tabItem(
                  tabName = "manual", fluidRow(
                    fluidRow(
                      tags$p(
                        "This is the AbSolution manual."
                      ),
                      tags$p(
                        "For personalized, step-by-step guidance, refer to the ",
                        tags$strong("Help"),
                        " subtab in the AbSolution tab."
                      ),
                      tags$p(
                        "Additionally, you can activate the ",
                        tags$strong("tooltip mode"),
                        " by clicking the ? at the top right corner to see explanations for each option when you hover over them."
                      ),
                      tags$iframe(
                        style="height:900px; width:100%;",
                        src="www/doc/AbSolution_guide.pdf"
                      )
                    )
                  )
                ),
                bs4Dash::tabItem(
                  tabName = "About", bs4Dash::userBox(
                    title = bs4Dash::userDescription(
                      title = "   Bioinformatics Laboratory", subtitle = "  Amsterdam UMC",
                      image = "www/img/BioinformaticsLaboratory.png", type = 2
                    ),
                    status = "orange", width = 12, closable = F, footer = HTML(
                      "<p align='justify'>
                            The Bioinformatics Laboratory (Biolab) at Amsterdam UMC is a leading research group built on two key pillars: the development and application of advanced computational immunology methods, and an unwavering commitment to reproducible research. The team specializes in techniques such as transcriptomic data analysis (including single-cell RNA sequencing) and gene regulatory network modeling to unravel complex immune system dynamics. Equally central is Biolab's focus on reproducible science: our projects follow the ENCORE framework, an in-house infrastructure that supports transparent, well-structured, and reusable computational analyses. By combining innovative immunological data science with rigorous reproducibility, the lab accelerates robust insights into disease and enables others to confidently build on our work. This dual focus has also fostered close collaborations with immunology experts, including previous work with Niek de Vries and Jeroen Guikema, underscoring the translational impact of Biolab's research.
                      </p>"
                    ),
                    shiny::actionButton(
                      inputId = "github_visit", icon = icon("github"),
                      label = "GitHub", value = "Open pop-up", onclick = "window.open('https://github.com/EDS-Bioinformatics-Laboratory/','_blank')"
                    ),
                    shiny::actionButton(
                      inputId = "web_visit", icon = icon("laptop"),
                      label = "Website", value = "Open pop-up", onclick = "window.open('https://bioinformaticslaboratory.eu/','_blank')"
                    )
                  ),
                  bs4Dash::userBox(
                    title = bs4Dash::userDescription(
                      title = "   AbSolution", subtitle = "  A practical implementation to improve reproducibility and transparency of computational research",
                      image = "www/img/AbSolution.png", type = 2
                    ),
                    status = "gray-dark", width = 12, closable = F, footer = HTML(
                      "<p align='justify'>
                              AbSolution is a user-friendly interactive platform for exploring B-cell and T-cell immune repertoires based on their sequence features.
                              It allows researchers to examine a wide range of physicochemical, structural, and mutation-related properties across receptor regions, clonotypes, samples, and conditions, helping to reveal patterns of immune selection and diversity.
                              By combining robust statistical analysis, intuitive visualizations, and built-in reproducibility through the ENCORE framework, AbSolution makes immune repertoire analysis accessible, transparent, and fully reproducible for both computational and experimental immunologists.
                      </p>"
                    ),
                    shiny::actionButton(
                      inputId = "github_ENCORE_visit", icon = icon("github"),
                      label = "GitHub", value = "Open pop-up", onclick = "window.open('https://github.com/EDS-Bioinformatics-Laboratory/AbSolution,'_blank')"
                    ),
                    shiny::actionButton(
                      inputId = "web_ENCORE_visit", icon = icon("newspaper"),
                      label = "Article 1", value = "Open pop-up", onclick = "window.open('https://media0.giphy.com/media/v1.Y2lkPTc5MGI3NjExMzVhc2lmN2NkY2V3d2JoYm9wOWg0c3JtOGdwNjN2aWpsNWFxaGN4NSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YQAuKJ7wf68qBHPw6Y/giphy.gif','_blank')"
                    ),
                    shiny::actionButton(
                      inputId = "web_ENCORE_visit", icon = icon("newspaper"),
                      label = "Article 2", value = "Open pop-up", onclick = "window.open('https://media0.giphy.com/media/v1.Y2lkPTc5MGI3NjExMzVhc2lmN2NkY2V3d2JoYm9wOWg0c3JtOGdwNjN2aWpsNWFxaGN4NSZlcD12MV9pbnRlcm5hbF9naWZfYnlfaWQmY3Q9Zw/YQAuKJ7wf68qBHPw6Y/giphy.gif','_blank')"
                    )
                  ),
                  bs4Dash::userBox(
                    title = bs4Dash::userDescription(
                      title = "   ENCORE", subtitle = "  A practical implementation to improve reproducibility and transparency of computational research",
                      image = "www/img/ENCORE.png", type = 2
                    ),
                    status = "gray-dark", width = 12, closable = F, footer = HTML(
                      "<p align='justify'>
                            <small>
                              van Kampen et al., Nature Communications (2024)
                              <a href='https://www.nature.com/articles/s41467-024-52446-8' target='_blank'>https://doi.org/10.1038/s41467-024-52446-8</a>
                            </small> <br><br>
                            ENCORE (ENhancing COmputational REproducibility) is a practical, group-developed framework that elevates transparency and reproducibility in computational research by guiding scientists on how to structure, document, and share all components of a project - from data and code to results and documentation - within a standardized, self-contained project compendium. Designed to be agnostic of programming language, data type, or infrastructure, ENCORE integrates best practices into everyday workflows, leverages modern version control (e.g., GitHub), and includes tools such as pre-defined documentation templates and an HTML-based navigator to make computational research clearer, more reliable, and easier to reuse or build upon. By embedding reproducibility at the core of every project, ENCORE helps teams produce research that is not only trustworthy but also ready for open science, collaboration, and long-term impact.
                      </p>"
                    ),
                    shiny::actionButton(
                      inputId = "github_ENCORE_visit", icon = icon("github"),
                      label = "GitHub", value = "Open pop-up", onclick = "window.open('https://github.com/EDS-Bioinformatics-Laboratory/ENCORE,'_blank')"
                    ),
                    shiny::actionButton(
                      inputId = "web_ENCORE_visit", icon = icon("newspaper"),
                      label = "Article", value = "Open pop-up", onclick = "window.open('https://www.nature.com/articles/s41467-024-52446-8','_blank')"
                    )
                  )
                )
                )
          ),
          controlbar = bs4Dash::dashboardControlbar(
            id = "controlbar",
            collapsed = T,
            overlay = F,
            skin="dark",
            width=300,
            bs4Dash::controlbarMenu(
              id = "controlbarmenu",
              bs4Dash::controlbarItem(
                title = "Main parameters",
                column(
                  width = 12,
                  align = "left",
                  numericInput(inputId="seed",
                               label = "Set random seed",
                               value = 1234,
                               min=1,
                               max=9999,
                               width="100%"),
                  verbatimTextOutput("onlynumbers"),
                  shinyWidgets::materialSwitch(
                    "colorblind_mode", "Use colorblind-friendly palettes",
                    value = F, width = NULL, status = "primary"
                  ),
                  selectInput(inputId = "img_type",
                              label = "Select img type for exporting individual plots",
                              choices = c("PNG" = "png",
                                          "SVG" = "svg"),
                              selected = "png",
                              width = "100%"),
                  numericInput(inputId="pixels_width",
                               label = "Png images width (px)",
                               value = 1400,
                               min=300,
                               max=30000,
                               width="100%"),
                  verbatimTextOutput("onlynumberswidth"),
                  numericInput(inputId="pixels_height",
                               label = "Png images height (px)",
                               value = 1000,
                               min=300,
                               max=30000,
                               width="100%"),
                  verbatimTextOutput("onlynumbersheight"),
                  numericInput(inputId="labelscale",
                               label = "Svg/png width and height size multiplier",
                               value = 1,
                               min=1,
                               max=10,
                               width="100%"),
                  verbatimTextOutput("onlynumberslabelscale"),
                  # actionButton(
                  #   "Reset_all", "Reload AbSolution", icon("rotate-left"),
                  #   style = "color: #fff; background-color: #18bc9c"
                  # )
                )
              )
            )

          ),
          footer=bs4Dash::dashboardFooter(
            left = a(
              href = "https://www.linkedin.com/in/rodrigogarciavaliente/",
              target = "_blank", "@RGarciaValiente"
            ),
            right = "2026, version 1.0.1"
          )

        )
    )
}

#' Add external Resources to the Application
#'
#' This function is internally used to add external
#' resources inside the Shiny application.
#'
#' @import shiny
#' @importFrom golem add_resource_path activate_js favicon bundle_resources
#' @noRd
golem_add_external_resources <- function() {
    add_resource_path("www", app_sys("app/www"))
    # add_resource_path("doc", app_sys("app/doc"))
    #https://github.com/ThinkR-open/golem/issues/297
    # add_resource_path(
    #   "sbs", system.file("www", package = "shinyBS")
    # )

    options(shiny.maxRequestSize=3000*1024^2)

    tags$head(
        favicon(), bundle_resources(
            path = app_sys("app/www"),
            app_title = "AbSolution"
        )
    )

}


#' Prepare AbSolution logo
#'
#' This function is internally used to prepare the AbSolution logo
#'
#' @import dashboardthemes
#' @noRd
logo_absolution <- dashboardthemes::shinyDashboardLogoDIY(
    boldText = "Ab", mainText = "Solution", textSize = 16, badgeText = "BETA", badgeTextColor = "white",
    badgeTextSize = 2, badgeBackColor = "#cf395c", badgeBorderRadius = 3
)

#' 1_Parsing
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'
#' @return The return value, if any, from executing the function.
#' @import stringr
#'
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
parse_AIRRSeq_file <- function(
    file, group, patient, subgroup, sample, input_path, C_region_included, FWR1partial,
    FWR4partial, D_gene, repertoire, output_path, is_example=FALSE
) {


    ### Am I working with the insertions?

    AIRR_input <- read.table(
        file = paste(
            input_path, paste(file, ".tsv", sep = ""),
            sep = ""
        ), nrows=if(is_example){20} else {-1},
        sep = "\t", header = TRUE, fill = TRUE
    )

    AIRR_input[is.na(AIRR_input)] = ""

    if(is_example){
      # AIRR_input=AIRR_input[c(1:min(nrow(AIRR_input),10)),]

    }
    # Removing Ns
    AIRR_input <- AIRR_input[which(!grepl("N", AIRR_input$sequence)),
        ]
    AIRR_input <- AIRR_input[which(!grepl("n", AIRR_input$sequence)),
        ]
    AIRR_input <- AIRR_input[which(AIRR_input$sequence_alignment != ""),
        ]

    AIRR_input <- AIRR_input[which(AIRR_input$germline_alignment != ""),
    ]
    AIRR_input <- AIRR_input[which(AIRR_input$cdr2 != ""),
    ]
    AIRR_input <- AIRR_input[which(AIRR_input$cdr1 != ""),
    ]
    AIRR_input <- AIRR_input[which(AIRR_input$fwr2 != ""),
    ]
    AIRR_input <- AIRR_input[which(AIRR_input$cdr3 != ""),
    ]
    AIRR_input <- AIRR_input[which(AIRR_input$fwr3 != ""),
    ]
    AIRR_input <- AIRR_input[which(AIRR_input$fwr4 != ""),
    ]
    ##some programmes use - for deletions, others dont, so we standarize this way
    AIRR_input$sequence_alignment=sapply(AIRR_input$sequence_alignment, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$germline_alignment=sapply(AIRR_input$germline_alignment, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$fwr1=sapply(AIRR_input$fwr1, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$fwr2=sapply(AIRR_input$fwr2, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$fwr3=sapply(AIRR_input$fwr3, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$fwr4=sapply(AIRR_input$fwr4, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$cdr1=sapply(AIRR_input$cdr1, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$cdr2=sapply(AIRR_input$cdr2, function(x) gsub("-",".", x, fixed=TRUE))
    AIRR_input$cdr3=sapply(AIRR_input$cdr3, function(x) gsub("-",".", x, fixed=TRUE))

    regions <- c("NT_FWR1", "NT_CDR1", "NT_FWR2", "NT_CDR2", "NT_FWR3", "NT_CDR3", "NT_FWR4")

    ########## R: insertions are not included here lalalala
    IMGT_parsed_index <- data.frame(
        ID = AIRR_input$sequence_id, Patient = patient, Sample = sample, Group = group,
        Sequence_type = "Repertoire", Subgroup = subgroup, V_region = AIRR_input$v_call,
        J_region = AIRR_input$j_call, NT_FWR1 = AIRR_input$fwr1, NT_CDR1 = AIRR_input$cdr1,
        NT_FWR2 = AIRR_input$fwr2, NT_CDR2 = AIRR_input$cdr2, NT_FWR3 = AIRR_input$fwr3,
        NT_CDR3 = AIRR_input$cdr3, NT_FWR4 = AIRR_input$fwr4, stringsAsFactors = FALSE
    )


    if ("c_call" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$C_region <- AIRR_input$c_call
    }
    if ("d_call" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$D_region <- AIRR_input$d_call
    }

    if ("clone_id" %in% colnames(AIRR_input) &&
        any(!is.na(unique(AIRR_input$clone_id))) &&
        any(
            unique(AIRR_input$clone_id) !=
                " "
        ) &&
        any(
            unique(AIRR_input$clone_id) !=
                ""
        )) {

        IMGT_parsed_index$Clone_ID <- AIRR_input$clone_id
    } else if ("raw_clonotype_id" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$Clone_ID <- AIRR_input$raw_clonotype_id
    } else if ("clone_id" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$Clone_ID <- AIRR_input$clone_id
    }
    if ("cell_id" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$Cell_ID <- AIRR_input$cell_id
    }

    if ("chain" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$Chain <- AIRR_input$chain
    }
    if ("locus" %in% colnames(AIRR_input)) {
        IMGT_parsed_index$Chain <- AIRR_input$locus
    }



    IMGT_parsed_index$ORF_begins=sapply(c(1:nrow(AIRR_input)), function(z)
      if(any(str_locate_all(AIRR_input$sequence_alignment[z],".")[[1]][1,1]==1) || any(str_locate_all(AIRR_input$sequence_alignment[z],"-")[[1]][1,1]==1)){
        if(FWR1partial){
          ((str_locate_all(tolower(AIRR_input$sequence_alignment[z]),"a|c|g|t")[[1]][1,2] - nchar(gsub(".","",AIRR_input$fwr1[z], fixed=T)))%%3)
        } else {
          (str_locate_all(tolower(AIRR_input$sequence_alignment[z]),"a|c|g|t")[[1]][1,2]%%3)
        }

      }else {
        1
      } )



    ### even if the format says aligned, not all programmes (e.g. IMGT,
    ### 02/2024) have these fields aligned. Let's add a checkup
    if (any(grepl(".", IMGT_parsed_index$NT_FWR1, fixed = TRUE)) ||
        any(grepl(".", IMGT_parsed_index$NT_FWR2, fixed = TRUE)) ||
        any(grepl(".", IMGT_parsed_index$NT_FWR3, fixed = TRUE)) ||
        any(grepl(".", IMGT_parsed_index$NT_FWR4, fixed = TRUE)) ||
        any(grepl(".", IMGT_parsed_index$NT_CDR1, fixed = TRUE)) ||
        any(grepl(".", IMGT_parsed_index$NT_CDR2, fixed = TRUE)) ||
        any(grepl(".", IMGT_parsed_index$NT_CDR3, fixed = TRUE))) {
      aligned_regions <- TRUE
    } else {
      aligned_regions <- FALSE
    }

    ########## R: insertions are not included here lalalala
    if (aligned_regions){
      or_CDR1 <- 1 + nchar(AIRR_input$fwr1)
      or_FWR2 <- or_CDR1 + nchar(AIRR_input$cdr1)
      or_CDR2 <- or_FWR2 + nchar(AIRR_input$fwr2)
      or_FWR3 <- or_CDR2 + nchar(AIRR_input$cdr2)
      or_CDR3 <- or_FWR3 + nchar(AIRR_input$fwr3)
      or_FWR4 <- or_CDR3 + nchar(AIRR_input$cdr3)
    } else {
      # or_CDR1 <- 1 + as.numeric(sapply(AIRR_input$fwr1_end, function(z) if(z ==""){0}else{z}))
      # or_FWR2 <- 1 + as.numeric(sapply(AIRR_input$cdr1_end, function(z) if(z ==""){0}else{z}))
      # or_CDR2 <- 1 + as.numeric(sapply(AIRR_input$fwr2_end, function(z) if(z ==""){0}else{z}))
      # or_FWR3 <- 1 + as.numeric(sapply(AIRR_input$cdr2_end, function(z) if(z ==""){0}else{z}))
      # or_CDR3 <- 1 + as.numeric(sapply(AIRR_input$fwr3_end, function(z) if(z ==""){0}else{z}))
      # or_FWR4 <- 1 + as.numeric(sapply(AIRR_input$cdr3_end, function(z) if(z ==""){0}else{z}))


      or_CDR1 <- c()
      or_FWR2 <- c()
      or_CDR2 <- c()
      or_FWR3 <- c()
      or_CDR3 <- c()
      or_FWR4 <- c()
      end_FWR4 <- c()
      for (numrow in c(1:nrow(AIRR_input))) {
        numbers=c(1:nchar(AIRR_input$sequence_alignment[numrow]))
        characters=strsplit(AIRR_input$sequence_alignment[numrow], split="")[[1]]
        gap_positions=str_locate_all(gsub(".","!",AIRR_input$sequence_alignment[numrow], fixed=T),"!")[[1]][,1]
        characters=characters[which(numbers %!in% gap_positions)]
        names(characters)=numbers[which(numbers %!in% gap_positions)]

        tmp_pos=c()
        init_char=1
        for(region in c("fwr1", "cdr1","fwr2","cdr2","fwr3","cdr3", "fwr4")) {
          char_pos=str_locate( paste(characters[init_char:length(characters)], collapse=""),AIRR_input[numrow,region])[1,2] + 1
          if(length(tmp_pos)>0 && char_pos==2) {

            char_pos=1
          } else if (region == "fwr4") {
            char_pos=char_pos-1
          } else if (region == "fwr1" && AIRR_input[numrow,region]=="") {
            char_pos=1
          }
          tmp_pos=c(tmp_pos,as.numeric(names(characters[init_char:length(characters)])[char_pos]))

          init_char=c(init_char:length(characters))[char_pos]


        }
        or_CDR1 <- c(or_CDR1, tmp_pos[1])
        or_FWR2 <- c(or_FWR2, tmp_pos[2])
        or_CDR2 <- c(or_CDR2, tmp_pos[3])
        or_FWR3 <- c(or_FWR3,tmp_pos[4] )
        or_CDR3 <- c(or_CDR3, tmp_pos[5])
        or_FWR4 <- c(or_FWR4, tmp_pos[6])
        end_FWR4 <- c(end_FWR4, tmp_pos[7] )
      }
    }



    AIRR_input$germline_alignment_nospace <- gsub(".", "", AIRR_input$germline_alignment, fixed = T)

    AIRR_input$germline_alignment_nospace[which(startsWith(AIRR_input$sequence_alignment,"."))]=
      sapply(which(startsWith(AIRR_input$sequence_alignment,".")),
             function(z)
               gsub(".", "",
               paste(strsplit(AIRR_input$germline_alignment[z],
                              split="")[[1]][str_locate_all(tolower(AIRR_input$sequence_alignment[z]),
                                                            "a|c|g|t")[[1]][1,2]:
                                               nchar(AIRR_input$germline_alignment[z])],
               collapse=""),
              fixed = T)
             )


    if (aligned_regions) {
        NT_CDR3_germ <- strsplit(
            substring(AIRR_input$germline_alignment, or_CDR3, or_CDR3 - 1 + nchar(AIRR_input$cdr3)),
            split = ""
        )
    } else {
        NT_CDR3_germ <- strsplit(
            substring(
                AIRR_input$germline_alignment, or_CDR3, or_FWR4 - 1
            ),
            split = ""
        )
    }
    # NT_CDR3_germ=substring(AIRR_input$sequence_alignment,or_CDR3,or_CDR3-1+nchar(AIRR_input$cdr3))


    for (index_cdr3 in c(1:length(NT_CDR3_germ))) {
        tmp <- NT_CDR3_germ[[index_cdr3]]
        index_n <- c(
            which(tmp == "N"),
            which(tmp == "n")
        )
        if (D_gene == F) {
          if(length(index_n)!=0) {
            index_n <- c(min(index_n):max(index_n))
          }
        }
        if (aligned_regions) {
            tmp[index_n] <- strsplit(
                substring(
                  AIRR_input$sequence_alignment[index_cdr3], or_CDR3[index_cdr3],
                  or_CDR3[index_cdr3] - 1 + nchar(AIRR_input$cdr3[index_cdr3])
              ),
                split = ""
            )[[1]][index_n]
        } else {
            tmp[index_n] <- strsplit(
                substring(
                  AIRR_input$sequence_alignment[index_cdr3],
                  or_CDR3[index_cdr3], or_FWR4[index_cdr3] - 1
              ),
                split = ""
            )[[1]][index_n]
        }
        NT_CDR3_germ[[index_cdr3]] <- paste(tmp, collapse = "")
    }



    if (aligned_regions) {
        reconstructed_IMGT_parsed_index <- data.frame(
            ID = AIRR_input$sequence_id, Patient = patient, Sample = sample, Group = group,
            Sequence_type = "Reconstructed_germline", Subgroup = subgroup, V_region = AIRR_input$v_call,
            J_region = AIRR_input$j_call, NT_FWR1 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, 1, nchar(AIRR_input$fwr1)),
                  fixed = T
              ),
                fixed = T
            ),
            NT_CDR1 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, or_CDR1, or_CDR1 - 1 + nchar(AIRR_input$cdr1)),
                  fixed = T
              ),
                fixed = T
            ),
            NT_FWR2 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, or_FWR2, or_FWR2 - 1 + nchar(AIRR_input$fwr2)),
                  fixed = T
              ),
                fixed = T
            ),
            NT_CDR2 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, or_CDR2, or_CDR2 - 1 + nchar(AIRR_input$cdr2)),
                  fixed = T
              ),
                fixed = T
            ),
            NT_FWR3 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, or_FWR3, or_FWR3 - 1 + nchar(AIRR_input$fwr3)),
                  fixed = T
              ),
                fixed = T
            ),
            NT_CDR3 = unlist(NT_CDR3_germ),
            NT_FWR4 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, or_FWR4, or_FWR4 - 1 + nchar(AIRR_input$fwr4)),
                  fixed = T
              ),
                fixed = T
            ),
            stringsAsFactors = F
        )
    } else {
        reconstructed_IMGT_parsed_index <- data.frame(
            ID = AIRR_input$sequence_id, Patient = patient, Sample = sample, Group = group,
            Sequence_type = "Reconstructed_germline", Subgroup = subgroup, V_region = AIRR_input$v_call,
            J_region = AIRR_input$j_call, NT_FWR1 = gsub(
                "-", "", gsub(
                  ".", "", substring(AIRR_input$germline_alignment, 1, or_CDR1-1),
                  fixed = T
              ),
                fixed = T
            ),
            NT_CDR1 = gsub(
                "-", "", gsub(
                  ".", "", substring(
                    AIRR_input$germline_alignment, or_CDR1, or_FWR2 - 1
                ),
                  fixed = T
              ),
                fixed = T
            ),
            NT_FWR2 = gsub(
                "-", "", gsub(
                  ".", "", substring(
                    AIRR_input$germline_alignment, or_FWR2, or_CDR2 - 1
                ),
                  fixed = T
              ),
                fixed = T
            ),
            NT_CDR2 = gsub(
                "-", "", gsub(
                  ".", "", substring(
                    AIRR_input$germline_alignment, or_CDR2, or_FWR3 - 1
                ),
                  fixed = T
              ),
                fixed = T
            ),
            NT_FWR3 = gsub(
                "-", "", gsub(
                  ".", "", substring(
                    AIRR_input$germline_alignment, or_FWR3, or_CDR3 - 1
                ),
                  fixed = T
              ),
                fixed = T
            ),
            NT_CDR3 = unlist(NT_CDR3_germ),
            NT_FWR4 = gsub(
                "-", "", gsub(
                  ".", "", substring(
                    AIRR_input$germline_alignment, or_FWR4, end_FWR4
                ),
                  fixed = T
              ),
                fixed = T
            ),
            stringsAsFactors = F
        )
    }


    if ("c_call" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$C_region <- AIRR_input$c_call
    }
    if ("d_call" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$D_region <- AIRR_input$d_call
    }

    if ("raw_clonotype_id" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$Clone_ID <- AIRR_input$raw_clonotype_id
    }
    if ("clone_id" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$Clone_ID <- IMGT_parsed_index$Clone_ID
    }
    if ("cell_id" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$Cell_ID <- AIRR_input$cell_id
    }
    if ("chain" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$Chain <- AIRR_input$chain
    }

    if ("locus" %in% colnames(AIRR_input)) {
        reconstructed_IMGT_parsed_index$Chain <- AIRR_input$locus
    }


    if (C_region_included) {
        index_insertions <- which(
            !(sapply(
                c(1:nrow(AIRR_input)),
                function(z) grepl(
                  gsub("-", "", gsub(".", "", AIRR_input$sequence_alignment[z], fixed = T)),
                  AIRR_input$sequence[z]
              )
            ))
        )

    } else {
        index_insertions <- which(
            (sapply(
                c(1:nrow(AIRR_input)),
                function(z) (gsub("-", "", gsub(".", "", AIRR_input$sequence_alignment[z], fixed = T)) !=
                  AIRR_input$sequence[z])
            ))
        )
    }
    index_deletions_germ <- which(
      (sapply(
        c(1:nrow(AIRR_input)),
        function(z) any(str_locate_all(gsub(".","!",AIRR_input$germline_alignment[z], fixed=T),"!")[[1]][,1] %!in%
                          str_locate_all(gsub(".","!",AIRR_input$sequence_alignment[z], fixed=T),"!")[[1]][,1])
      ))
    )

    index_deletions_rep <- which(
      (sapply(
        c(1:nrow(AIRR_input)),
        function(z) any(str_locate_all(gsub(".","!",AIRR_input$sequence_alignment[z], fixed=T),"!")[[1]][,1] %!in%
                          str_locate_all(gsub(".","!",AIRR_input$germline_alignment[z], fixed=T),"!")[[1]][,1])
      ))
    )

    insertion_sizes <- sapply(
        index_insertions, function(z) drop(
            attr(
                adist(
                  gsub("-", "", gsub(".", "", AIRR_input$sequence_alignment[z], fixed = T)),
                  AIRR_input$sequence[z], counts = TRUE
              ),
                "counts"
            )
        )[which(
            names(
                drop(
                  attr(
                    adist(
                      gsub("-", "", gsub(".", "", AIRR_input$sequence_alignment[z], fixed = T)),
                      AIRR_input$sequence[z], counts = TRUE
                  ),
                    "counts"
                )
              )
            ) ==
                "ins"
        )]
    )


    for (region in regions) {
      IMGT_parsed_index[, region] <- toupper(
        gsub(
          "-", "", gsub(".", "", IMGT_parsed_index[, region], fixed = T),
          fixed = T
        )
      )
      reconstructed_IMGT_parsed_index[, region] <- toupper(
        gsub(
          "-", "", gsub(".", "", reconstructed_IMGT_parsed_index[, region], fixed = T),
          fixed = T
        )
      )
    }


    for (deletion_seq in index_deletions_germ) {
      all_germ_located=str_locate_all(gsub(".","!",AIRR_input$germline_alignment[deletion_seq], fixed=T),"!")[[1]][,1]
      all_rep_located= str_locate_all(gsub(".","!",AIRR_input$sequence_alignment[deletion_seq], fixed=T),"!")[[1]][,1]
      deletion_pos=which(all_germ_located %!in%all_rep_located)
      germ_located=all_germ_located[deletion_pos]


      for (pos in germ_located) {

        curr_pos=0
        previous_dels=length(which(all_rep_located<pos)) ############ THINK RODRIGO THINK

        is_this_on=T
        reg_number=1
        while(is_this_on) {
          region=regions[reg_number]
          old_curr_pos=curr_pos
          curr_pos=curr_pos+nchar(IMGT_parsed_index[deletion_seq, region])
          if(pos<=(curr_pos+previous_dels) && pos >= (old_curr_pos+previous_dels)){
            # print("Y")
            # break
            tmp_del=strsplit(IMGT_parsed_index[deletion_seq, region], split="")[[1]]
            tmp_del[pos-old_curr_pos-previous_dels]=tolower(tmp_del[pos-old_curr_pos-previous_dels])
            IMGT_parsed_index[deletion_seq, region]=paste(tmp_del, collapse="")
            is_this_on=F
          }
          reg_number=reg_number+1

        }
        is_this_on=T

      }
    }

    for (deletion_seq in index_deletions_rep) {
      all_germ_located=str_locate_all(gsub(".","!",AIRR_input$germline_alignment[deletion_seq], fixed=T),"!")[[1]][,1]
      all_rep_located= str_locate_all(gsub(".","!",AIRR_input$sequence_alignment[deletion_seq], fixed=T),"!")[[1]][,1]
      deletion_pos=which(all_rep_located %!in% all_germ_located)
      germ_located=all_rep_located[deletion_pos]

      for (pos in germ_located) {

        curr_pos=0
        previous_dels=length(which(all_germ_located<pos))

        is_this_on=T
        reg_number=1
        while(is_this_on) {

          region=regions[reg_number]
          old_curr_pos=curr_pos
          curr_pos=curr_pos+nchar(reconstructed_IMGT_parsed_index[deletion_seq, region])
          if(pos<=(curr_pos+previous_dels) && pos >= (old_curr_pos+previous_dels)){


            tmp_del=strsplit(reconstructed_IMGT_parsed_index[deletion_seq, region], split="")[[1]]
            tmp_del[pos-old_curr_pos-previous_dels]=tolower(tmp_del[pos-old_curr_pos-previous_dels])
            reconstructed_IMGT_parsed_index[deletion_seq, region]=paste(tmp_del, collapse="")
            is_this_on=F
          }
          reg_number=reg_number+1


        }
        is_this_on=T

      }
    }


    ### PARTIAL FWRs
    if (FWR1partial) {
      IMGT_parsed_index$NT_FWR1 <- ""
    }

    if (FWR4partial) {
      IMGT_parsed_index$NT_FWR4 <- ""
    }

    reconstructed_IMGT_parsed_index$ORF_begins=IMGT_parsed_index$ORF_begins

    ### PARTIAL FWRs
    if (FWR1partial) {
      reconstructed_IMGT_parsed_index$NT_FWR1 <- sapply(
        c(1:nrow(reconstructed_IMGT_parsed_index)),
        function(z) if (nchar(
          gsub(
            "-", "", gsub(".", "", IMGT_parsed_index$NT_FWR1[z], fixed = T),
            fixed = T
          )
        ) ==
        0) {
          ""
        } else {
          str_sub(
            gsub(
              "-", "", gsub(".", "", reconstructed_IMGT_parsed_index$NT_FWR1[z], fixed = T),
              fixed = T
            ),
            -nchar(
              gsub(
                "-", "", gsub(".", "", IMGT_parsed_index$NT_FWR1[z], fixed = T),
                fixed = T
              )
            ),
            -1
          )
        }
      )
      reconstructed_IMGT_parsed_index$NT_FWR1 <- ""

      tmp_lengths_pre=sapply(reconstructed_IMGT_parsed_index$NT_CDR1, function(x) nchar(gsub(
        "-", "", gsub(".", "", x, fixed = T),
        fixed = T
      )))

      tmp_ref=reconstructed_IMGT_parsed_index$NT_CDR1
      reconstructed_IMGT_parsed_index$NT_CDR1 <- sapply(
        c(1:nrow(reconstructed_IMGT_parsed_index)),
        function(z) if (nchar(
          gsub(
            "-", "", gsub(".", "", IMGT_parsed_index$NT_CDR1[z], fixed = T),
            fixed = T
          )
        ) ==
        0) {
          ""
        } else {
          str_sub(
            gsub(
              "-", "", gsub(".", "", reconstructed_IMGT_parsed_index$NT_CDR1[z], fixed = T),
              fixed = T
            ),
            -nchar(
              gsub(
                "-", "", gsub(".", "", IMGT_parsed_index$NT_CDR1[z], fixed = T),
                fixed = T
              )
            )-length(str_match_all(sub("^.*?([A-Z])", "\\1", reconstructed_IMGT_parsed_index$NT_CDR1[z]),
                                   "[a-z]")[[1]])+
              length(str_match_all(IMGT_parsed_index$NT_CDR1[z],
                                   "[a-z]")[[1]]),
            -1
          )
        }
      )

      tmp_lengths_pro=sapply(reconstructed_IMGT_parsed_index$NT_CDR1, function(x) nchar(x))

      tmp_ref_orf=reconstructed_IMGT_parsed_index$ORF_begins
      # reconstructed_IMGT_parsed_index$ORF_begins=(reconstructed_IMGT_parsed_index$ORF_begins+(tmp_lengths_pre-tmp_lengths_pro)%%3)%%3
    }

    if (FWR4partial) {
      reconstructed_IMGT_parsed_index$NT_FWR4 <- sapply(
        c(1:nrow(reconstructed_IMGT_parsed_index)),
        function(z) if (nchar(
          gsub(
            "-", "", gsub(".", "", IMGT_parsed_index$NT_FWR4[z], fixed = T),
            fixed = T
          )
        ) ==
        0) {
          ""
        } else {
          str_sub(
            gsub(
              "-", "", gsub(".", "", reconstructed_IMGT_parsed_index$NT_FWR4[z], fixed = T),
              fixed = T
            ),
            1, nchar(
              gsub(
                "-", "", gsub(".", "", IMGT_parsed_index$NT_FWR4[z], fixed = T),
                fixed = T
              )
            )
          )
        }
      )
      reconstructed_IMGT_parsed_index$NT_FWR4 <- ""

      reconstructed_IMGT_parsed_index$NT_CDR3 <- sapply(
        c(1:nrow(reconstructed_IMGT_parsed_index)),
        function(z) if (nchar(
          gsub(
            "-", "", gsub(".", "", IMGT_parsed_index$NT_CDR3[z], fixed = T),
            fixed = T
          )
        ) ==
        0) {
          ""
        } else {
          str_sub(
            gsub(
              "-", "", gsub(".", "", reconstructed_IMGT_parsed_index$NT_CDR3[z], fixed = T),
              fixed = T
            ),
            1, nchar(
              gsub(
                "-", "", gsub(".", "", IMGT_parsed_index$NT_CDR3[z], fixed = T),
                fixed = T
              )
            )
          )
        }
      )
    }

    for (index_ins in index_insertions) {


        tmp_alignment_object <- pwalign::pairwiseAlignment(
            pattern = toupper(AIRR_input[index_ins, ]$sequence),
            subject = toupper(
              gsub( "-", "",
                    gsub(".", "",
                         AIRR_input[index_ins, ]$sequence_alignment,
                         fixed = T),
                    fixed = T)
            )
        )
        # tmp_alignment_seq=Biostrings::pairwiseAlignment(pattern=AIRR_input[index_ins,]$sequence,
        # subject=sequence) ##no es fiable

        tmp_align <- list(pattern = as.character(), subject = as.character())
        tmp_align$pattern <- as.character(pwalign::alignedPattern(tmp_alignment_object))
        tmp_align$subject <- as.character(pwalign::alignedSubject(tmp_alignment_object))

        gap_1 <- stringr::str_locate_all(
            as.character(tmp_align$pattern),
            "-"
        )[[1]][,
            1]
        gap_2 <- stringr::str_locate_all(
            as.character(tmp_align$subject),
            "-"
        )[[1]][,
            1]
        splitted_gap_1 <- split(
            gap_1, cumsum(
                c(
                  1, diff(gap_1) !=
                    1
              )
            )
        )
        if(length(splitted_gap_1)>0){

        }
        splitted_gap_2 <- split(
            gap_2, cumsum(
                c(
                  1, diff(gap_2) !=
                    1
              )
            )
        )

        num_gap <- 0
        diff_with_unfiltered_presequence=0
        for (gap in splitted_gap_2) {
            init <- T
            finit <- T
            num_gap <- num_gap + 1


            if (C_region_included && num_gap == length(splitted_gap_2) &&
                (gap[length(gap)] == nchar(as.character(tmp_align$subject)) ||
                 startsWith(paste(strsplit(tmp_align$pattern, split="")[[1]][splitted_gap_2[[num_gap]]],
                                  collapse=""),
                            paste(strsplit(tmp_align$subject, split="")[[1]][(splitted_gap_2[[num_gap]][1]+1):nchar(tmp_align$subject)],
                                  collapse=""))
                 )

                )
              {

            } else if (C_region_included && num_gap == 1 &&
                       (gap[1]==1 ||
                        endsWith(paste(strsplit(tmp_align$pattern, split="")[[1]][splitted_gap_2[[1]]], collapse=""),
                                 paste(strsplit(tmp_align$subject, split="")[[1]][1:(splitted_gap_2[[1]][1]-1)], collapse=""))
                        ))
              {
                diff_with_unfiltered_presequence=nchar(paste(strsplit(tmp_align$pattern, split="")[[1]][splitted_gap_2[[1]]], collapse=""))
              } else if (FWR1partial && num_gap == 1 && any(gap == 1)) {

            } else {
                for (region in regions) {
                  tmp_length <- sum(
                    nchar(
                      gsub(
                        "-", "", gsub(
                          ".", "", IMGT_parsed_index[index_ins, regions[c(1:(which(regions == region)))]],
                          fixed = T
                      ),
                        fixed = T
                    )
                  )
                ) +
                    1
                  sequence <- gsub(
                    "-", "", gsub(".", "", IMGT_parsed_index[index_ins, region], fixed = T),
                    fixed = T
                )


                  # if (!init && finit){ pos_init=0 }



                  if (gap[1] <= (tmp_length+diff_with_unfiltered_presequence) && init) {
                    if (region == "NT_FWR1") {
                      pos_init <- gap[1] - 1
                    } else {
                      pos_init <- gap[1] - sum(
                        nchar(
                          gsub(
                            "-", "", gsub(
                              ".", "", IMGT_parsed_index[index_ins, regions[c(
                                1:(which(regions == region) -
                                  1)
                            )]],
                              fixed = T
                          ),
                            fixed = T
                        )
                      )
                    ) -
                        1-diff_with_unfiltered_presequence
                    }

                    what_to_insert <- strsplit(tmp_align$pattern, split = "")[[1]][gap[1]:gap[length(gap)]]
                    what_to_insert <- tolower(what_to_insert)
                    init <- F

                    test <- strsplit(sequence, split = "")[[1]]
                    test <- append(test, what_to_insert, pos_init)

                    IMGT_parsed_index[index_ins, region] <- paste(test, collapse = "")
                  }

                  # if(gap[length(gap)]<=tmp_length && finit) { finit=F }

                }
            }



        }

    }


    IMGT_parsed_index <- rbind(IMGT_parsed_index, reconstructed_IMGT_parsed_index)

    IMGT_parsed_index <- IMGT_parsed_index[with(IMGT_parsed_index,
                                                order(ID, Sequence_type)),
                                           ]



    IMGT_parsed_index$Repertoire <- repertoire
    if(!is_example){
      suppressWarnings(
        write.table(
          IMGT_parsed_index, file = paste(output_path, "IMGT_parsed_index.txt", sep = ""),
          append = T, row.names = F,
          col.names = !file.exists(paste(output_path, "IMGT_parsed_index.txt", sep = "")),
          sep = "\t", quote = F
        )
      )
    } else {
      return(IMGT_parsed_index[c(1:8),])
    }


}

test_that("Parsing AIRR-Seq works", {

  test_parse=data.frame(
    sequence_id="AAAGCAAAGTGGTAAT-1_contig_2",
    sequence="GGGGAGCTCTGGGAGACGAGCCCAGCACTGGAAGTCGCCGGTGTTTCCATTCGGTGATCAGCACTGAACACAGAGGACTCACCATGGAGTTTGGGCTGAGCTGGGTTTTCCTCGTTGCTCTTTTAAGAGGTGTCCAGTGTCAGGTGCAGCTGGTGGAGTCTGGGGGAGGCGTGGTCCAGCCTGGGAGGTCCCTGAGACTCTCCTGTGCAGCCTCTGGATTCACCTTCAGTAGCTATGCTATGCACTGGGTCCGCCAGGCTCCAGGCAAGGGGCTGGAGTGGGTGGCAGTTATATCATATGATGGAAGTAATAAATACTACGTAGACTCCGTGAAGGGCCGATTCACCATCTCCAGAGACAATTCCAAGAACACGCTGTATCTGCAAATGAACAGCCTGAGAGCTGAGGACACGGCTGTGTATTACTGTGCGAGAGATATCTCGCAGGCTGGTAAGCACTACTACTACGGTATGGACGTCTGGGGCCAAGGGACCACGGTCACCGTCTCCTCAGCACCCACCAAGGCTCCGGATGTGTTCCCCATCATATCAGGGTGCAGACACCCAAAGGATAACAGCCCTGTGGTCCTGGCATGCTTGATAACTGGGTACCACC",
    sequence_alignment="CAGGTGCAGCTGGTGGAGTCTGGGGGA...GGCGTGGTCCAGCCTGGGAGGTCCCTGAGACTCTCCTGTGCAGCCTCTGGATTCACCTTC............AGTAGCTATGCTATGCACTGGGTCCGCCAGGCTCCAGGCAAGGGGCTGGAGTGGGTGGCAGTTATATCATATGAT......GGAAGTAATAAATACTACGTAGACTCCGTGAAG...GGCCGATTCACCATCTCCAGAGACAATTCCAAGAACACGCTGTATCTGCAAATGAACAGCCTGAGAGCTGAGGACACGGCTGTGTATTACTGTGCGAGAGATATCTCGCAGGCTGGTAAGCACTACTACTACGGTATGGACGTCTGGGGCCAAGGGACCACGGTCACCGTCTCCTCA",
    germline_alignment="CAGGTGCAGCTGGTGGAGTCTGGGGGA...GGCGTGGTCCAGCCTGGGAGGTCCCTGAGACTCTCCTGTGCAGCCTCTGGATTCACCTTC............AGTAGCTATGCTATGCACTGGGTCCGCCAGGCTCCAGGCAAGGGGCTGGAGTGGGTGGCAGTTATATCATATGAT......GGAAGTAATAAATACTACGCAGACTCCGTGAAG...GGCCGATTCACCATCTCCAGAGACAATTCCAAGAACACGCTGTATCTGCAAATGAACAGCCTGAGAGCTGAGGACACGGCTGTGTATTACTGTGCGAGAGANNNNNNNNNGGCTGGTANNNACTACTACTACGGTATGGACGTCTGGGGCCAAGGGACCACGGTCACCGTCTCCTCA",
    v_call="IGHV3-30*04,IGHV3-30-3*03",
    j_call="IGHJ6*02",
    fwr1="CAGGTGCAGCTGGTGGAGTCTGGGGGA...GGCGTGGTCCAGCCTGGGAGGTCCCTGAGACTCTCCTGTGCAGCCTCT",
    cdr1="GGATTCACCTTC............AGTAGCTATGCT",
    fwr2="ATGCACTGGGTCCGCCAGGCTCCAGGCAAGGGGCTGGAGTGGGTGGCAGTT",
    cdr2="ATATCATATGAT......GGAAGTAATAAA",
    fwr3="TACTACGTAGACTCCGTGAAG...GGCCGATTCACCATCTCCAGAGACAATTCCAAGAACACGCTGTATCTGCAAATGAACAGCCTGAGAGCTGAGGACACGGCTGTGTATTACTGT",
    cdr3="GCGAGAGATATCTCGCAGGCTGGTAAGCACTACTACTACGGTATGGACGTC",
    fwr4="TGGGGCCAAGGGACCACGGTCACCGTCTCCTCA"
  )

  tmpf <- tempfile(fileext = "")
  write.table(test_parse, file = paste0(tmpf,".tsv"),
              sep = "\t", row.names = FALSE, col.names = TRUE,
              quote = FALSE)

  AbSolution:::parse_AIRRSeq_file(
    file=basename(tmpf),
    group="Test_group",
    patient="Test_patient",
    subgroup="Test_subgroup",
    sample="Test_sample",
    input_path=paste0(dirname(tmpf),"/"),
    C_region_included=T,
    FWR1partial=T,
    FWR4partial=F,
    D_gene=F,
    repertoire="BCR",
    output_path=paste0(dirname(tmpf),"/"),
    is_example=F
  )

  outfile=paste(paste0(dirname(tmpf),"/"), "IMGT_parsed_index.txt", sep = "")
  expect_true(file.exists(outfile))
  expect_gt(file.info(outfile)$size, 0)

  df <- read.table(outfile, sep = "\t", header = TRUE, stringsAsFactors = FALSE)

  expect_true(all(c("ID", "V_region", "J_region", "NT_CDR3") %in% colnames(df)))
  expect_true(all(df$Group == "Test_group"))

  expect_equal(c("TACTACGCAGACTCCGTGAAGGGCCGATTCACCATCTCCAGAGACAATTCCAAGAACACGCTGTATCTGCAAATGAACAGCCTGAGAGCTGAGGACACGGCTGTGTATTACTGT"),
               df$NT_FWR3[1])
  expect_equal(c("TACTACGTAGACTCCGTGAAGGGCCGATTCACCATCTCCAGAGACAATTCCAAGAACACGCTGTATCTGCAAATGAACAGCCTGAGAGCTGAGGACACGGCTGTGTATTACTGT"),
               df$NT_FWR3[2])
})

# Suppress R CMD check notes for data.table NSE and reactive variables
utils::globalVariables(c(
  # data.table
  ".", ".N", ".SD", "N",
  # columnas creadas on-the-fly
  "Clone", "Clone_ID", "color",
  # reactiveValues usados en módulos
  "Selection_values", "Big_mem_values"
))

#' buildRmdBundle_alt
#'
#' @description Modified shinymeta function to build a Rmd bundle following
#' ENCORE guidelines.
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/report.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return NULL
#' @import shinymeta
#' @noRd
buildRmdBundle_alt<-function (report_template, output_zip_path, vars = list(), include_files = list(),
                              render = TRUE, render_args = list(), golem_renv=FALSE)
{
  force(report_template)
  force(vars)
  shinymeta_with_progress_obj(function(progress) {
    progress$set(value = 0)
    progress$set(message = "Generating code")
    if (is.list(vars)) {
      vars <- lapply(vars, function(x) {
        if (is.language(x)) {
          paste(formatCode(x), collapse = "\n")
        }
        else {
          x
        }
      })
    }
    progress$set(value = 0.1)
    progress$set(message = "Expanding Rmd template")
    rmd_source <- shinymeta_knit_expand_safe(report_template, vars = vars)
    rmd_filename <- shinymeta_template_rename(report_template, "Rmd")
    build_bundle_alt(rmd_source, rmd_filename, output_zip_path,
                     include_files = include_files, render = render, render_args = render_args,
                     progress = progress,  golem_renv= golem_renv)
  })
}


#' build_bundle_alt
#'
#' @description Modified shinymeta function to build a Rmd bundle following
#' ENCORE guidelines.
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/report.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return NULL
#' @import shinymeta
#' @import attachment
#' @importFrom dockerfiler dock_from_desc
#' @import golem
#' @noRd
build_bundle_alt <-function (input_src, input_filename, output_zip_path, include_files = list(),
                             render = TRUE, render_args = list(), progress, golem_renv=FALSE)
{
  force(input_src)
  force(input_filename)
  force(output_zip_path)
  force(include_files)
  force(render)
  force(render_args)
  progress$set(value = 0.2)
  progress$set(message = "Adding items to zip archive")
  x <-  shinymeta_zip_archive()
  dest_filename_full <- fs::path(shinymeta_archive_basedir(x), input_filename)
  writeLines(input_src, dest_filename_full)
  shinymeta_add_items(x, !!!include_files)
  progress$set(value = 0.3)
  if (render) {
    progress$set(message = "Rendering report")
    shinymeta_render_with_args(dest_filename_full, render_args)
  }
  newdir=fs::path(shinymeta_archive_basedir(x), "Notebook")
  dir.create(newdir, showWarnings = FALSE)
  file.copy(dest_filename_full, fs::path(newdir, input_filename))
  file.remove(dest_filename_full)
  if(golem_renv) {
    progress$set(value = 0.8)
    progress$set(message = "Exporting package and renv")
    newdir=fs::path(shinymeta_archive_basedir(x), "0_SoftwareEnvironment/R")
    dir.create(newdir, showWarnings = FALSE, recursive = TRUE)
    suppressWarnings(
      suppressMessages(
        utils::capture.output(
          golem::add_dockerfile_with_renv(output_dir = newdir),
          type = "output"
        )
      )
    )
    file.rename(from=paste0(newdir, "/renv.lock.prod"),
                to=paste0(newdir, "/renv.lock"))
  }
  progress$set(value = 0.9)
  progress$set(message = "Compressing bundle")
  archive <-  shinymeta_build_archive(x, output_zip_path)
  progress$set(value = 1)
  archive
}

#' shinymeta_zip_archive
#'
#' @description Original code from package Shinymeta, internal function zip_archive.
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/archive.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return structure with dir for zip file
#' @noRd
shinymeta_zip_archive <- function(temp_dir = NULL) {
  if (!is.null(temp_dir) && (!is.character(temp_dir) || length(temp_dir) != 1)) {
    stop("temp_dir must be a single-element character vector")
  }

  if (is.null(temp_dir)) {
    temp_dir <- tempfile("archive")
    fs::dir_create(temp_dir, mode = "u=rwx,go=")
  } else if (!dir.exists(temp_dir)) {
    stop("temp_dir directory does not exist")
  }

  structure(
    list(basedir = temp_dir),
    class = "pending_zip_archive"
  )
}

#' shinymeta_add_items
#'
#' @description Original code from package Shinymeta, internal function add_items.
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/archive.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return structure with dir for zip file
#' @noRd
shinymeta_add_items <- function(x, ...) {
  stopifnot(inherits(x, "pending_zip_archive"))

  include_files <- rlang::dots_list(..., .homonyms = "last", .check_assign = TRUE)

  if (is.null(names(include_files))) {
    names(include_files) <- as.character(include_files)
  }

  mapply(names(include_files), include_files, FUN = function(to, from) {
    if (nchar(from) == 0) {
      from <- to
    }
    shinymeta_add_item(x, from, to)
    NULL
  })

  x
}


#' shinymeta_archive_basedir
#'
#' @description Adapted code from package Shinymeta, internal function archive_basedir
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/archive.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return baseline dir for zip file
#' @noRd
shinymeta_archive_basedir <- function(x) {
  stopifnot(inherits(x, "pending_zip_archive"))

  x[["basedir"]]
}


#' shinymeta_template_rename
#'
#' @description Original code from package Shinymeta, internal function template_rename
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/report.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return filename
#' @noRd
shinymeta_template_rename <- function(input_template, extension = "Rmd") {
  stopifnot(is.character(extension) && length(extension) == 1 && identical(TRUE, nzchar(extension)))

  filename <- fs::path_ext_remove(fs::path_file(input_template))
  if (tolower(fs::path_ext(filename)) == tolower(extension)) {
    filename
  } else {
    paste0(filename, ".", extension)
  }
}

#' shinymeta_build_archive
#'
#' @description Adapted code from package Shinymeta, internal function build_archive
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/archive.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return NULL
#' @noRd
shinymeta_build_archive <- function(x, output_file) {

  basedir <- shinymeta_archive_basedir(x)

  olddir <- getwd()
  setwd(basedir)
  on.exit(setwd(olddir))

  utils::zip(fs::path_abs(output_file, olddir), ".",
             flags = "-rq")
  invisible(output_file)
}

#' shinymeta_with_progress_obj
#'
#' @description Original code from package Shinymeta, internal function with_progress_obj
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/report.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return session progress
#' @noRd
shinymeta_with_progress_obj <- function(callback) {
  # Note that `session` may be NULL.
  session <- shiny::getDefaultReactiveDomain()
  if (!is.null(session$userData$shinymeta_last_progress)) {
    # If the last progress object we created for this session is still visible,
    # close it. This would be in the case of an error, we never auto-dismiss in
    # that case.
    suppressWarnings(session$userData$shinymeta_last_progress$close())
  }

  progress <- shinymeta_make_progress()
  # Register our newly created progress object.
  session$userData$shinymeta_last_progress <- progress

  tryCatch(shiny::captureStackTraces({
    callback(progress)
    progress$close()
    session$userData$shinymeta_last_progress <- NULL
  }), error = function(err) {
    progress$set(value = 1, message = "An error has occurred:",
                 detail = conditionMessage(err))
    stop(err)
  })
}

#' shinymeta_make_progress
#'
#' @description Original code from package Shinymeta, internal function make_progress
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/report.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return session progress
#' @noRd
shinymeta_make_progress <- function(...) {
  session <- shiny::getDefaultReactiveDomain()
  if (!is.null(session)) {
    shiny::Progress$new(session = session, ...)
  } else {
    # Return a dummy progress object
    nothing <- function(...) {}
    list(
      set = nothing,
      inc = nothing,
      getMin = nothing,
      getMax = nothing,
      getValue = nothing,
      close = nothing,
      clone = nothing
    )
  }
}



#' shinymeta_knit_expand_safe
#'
#' @description Original code from package Shinymeta, internal function knit_expand_safe
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/utils.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return knitr::knit_expand
#' @noRd
shinymeta_knit_expand_safe <- function(file, vars = list(), text = xfun::read_utf8(file), delim = c("{{", "}}")) {
  # The approach we take here is to detect all knitr md patterns before and
  # after expansion, and fail if anything was either added or removed. We tried
  # just testing the output of each {{expansion}} for the patterns, but, that
  # doesn't catch cases where an inline.code is started in one expansion and
  # finished in another (see test in test-report.R).

  # Code chunk delimiter regexes
  # TODO: Can we assume `md`?
  patterns <- unname(unlist(knitr::all_patterns$md))

  matches_before <- shinymeta_count_matches_by_pattern(text, patterns)

  # Create an environment that contains nothing but the variables we want to
  # make available for template expansion, plus .GlobalEnv.
  eval_envir <- list2env(vars, parent = .GlobalEnv)

  # Use a knitr hook to ensure that only the ... arguments plus stuff in the
  # global environment are available when evaluating {{/}} expressions.
  orig_eval_inline <- knitr::knit_hooks$get("evaluate.inline")
  knitr::knit_hooks$set(evaluate.inline = function(code, envir) {
    # ignore envir, as it includes the parent frame of `knit_expand` which we
    # explicitly do not want to be used for evaluation--only ... arguments to
    # knit_expand_safe should be used.
    orig_eval_inline(code, eval_envir)
  })
  on.exit(knitr::knit_hooks$set(evaluate.inline = orig_eval_inline), add = TRUE)

  res <- knitr::knit_expand(text = text, delim = delim)

  matches_after <- shinymeta_count_matches_by_pattern(xfun::split_lines(res), patterns)

  if (!identical(matches_before, matches_after)) {
    # The process of knit_expand-ing introduced new (or removed existing?) code
    # chunks
    stop("Can't build report--user input values must not contain code chunk delimiters")
  }

  res
}



#' shinymeta_render_with_args
#'
#' @description Original code from package Shinymeta, internal function render_with_args
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/report.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return NULL
#' @noRd
shinymeta_render_with_args <- function(input_file, render_args = list(), switch_dirs = TRUE, fork = TRUE) {

  if (switch_dirs) {
    old_wd <- getwd()
    setwd(fs::path_dir(input_file))
    on.exit(setwd(old_wd))
  }

  if (fork) {
    callr::r(
      function(...) rmarkdown::render(...),
      # https://github.com/rstudio/rmarkdown/issues/1204#issuecomment-344884823
      args = c(list(input_file, envir = globalenv()), render_args)
    )
  } else {
    do.call(rmarkdown::render, c(list(input_file), render_args), quote = TRUE)
  }
}


#' shinymeta_count_matches_by_pattern
#'
#' @description Original code from package Shinymeta, internal function count_matches_by_pattern
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/utils.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return vector of length(pattern)
#' @noRd
shinymeta_count_matches_by_pattern <- function(string, pattern) {
  vapply(pattern, function(regex) {
    matches <- stringr::str_locate_all(string, regex)
    sum(vapply(matches, nrow, integer(1)))
  }, integer(1), USE.NAMES = FALSE)
}


#' shinymeta_add_item
#'
#' @description Adapted code from package Shinymeta, internal function add_item
#' Original authors:  	Joe Cheng [aut], Carson Sievert ORCID iD [cre, aut], RStudio [cph]
#' Original source: https://github.com/rstudio/shinymeta/blob/main/R/archive.R
#' Original license: GPL-3 (see https://cran.r-project.org/package=shinymeta)
#' @return vector of length(pattern)
#' @noRd
shinymeta_add_item <- function(x, source_file, target_file) {
  stopifnot(inherits(x, "pending_zip_archive"))

  if (!is.character(source_file) || length(source_file) != 1) {
    stop("source_file must be a single-element character vector")
  }
  if (!is.character(target_file) || length(target_file) != 1) {
    stop("target_file must be a single-element character vector")
  }

  if (fs::is_absolute_path(target_file)) {
    stop("target_file must be a relative path")
  }

  full_src <- fs::path_abs(source_file)

  basedir <- shinymeta_archive_basedir(x)

  if (fs::dir_exists(full_src)) {
    full_dest <- fs::path(basedir, target_file)
    fs::dir_copy(full_src, full_dest)
  } else {
    if (grepl("[/\\]$", target_file)) {
      # If source is a file, but target is a directory name, ensure
      # that the file gets copied into the target, rather than as
      # the target. Without this line, fs::file_copy would treat
      # the target as a filename (it would strip off the slash).
      target_file <- fs::path(target_file, fs::path_file(source_file))
    }
    full_dest <- fs::path(basedir, target_file)

    if (!fs::path_dir(target_file) %in% c("", "."))
      fs::dir_create(fs::path_dir(full_dest), recurse = TRUE)
    fs::file_copy(full_src, full_dest)
  }

  x
}

#' Merge FBMs and meta files
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'    A function to join by row individual FBMs and .info files
#'     into a unique FBM and a unique .info file, merged_bm.
#' @param filepath Path to the individual FBMs.
#' @return A list with the merged FBM, the merged info file and the FBM header.
#' @importFrom utils  glob2rx
#' @import bigstatsr
#' @import tools
#' @import bigparallelr
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
merge_FBMs <- function(filepath) {
    basenames <- basename(list.files(path = filepath, pattern = glob2rx("*.rds")))
    prefixes <- tools::file_path_sans_ext(basenames)

    list_descriptions <- list()
    list_FBMs <- list()
    list_headers <- list()
    for (prefix in unique(prefixes)[which(
        unique(prefixes) !=
            "Merged_bm"
    )]) {
        list_descriptions[[length(list_descriptions) +
            1]] <- data.table::fread(file.path(filepath, paste0(prefix, ".info")))
        list_headers[[length(list_headers) +
            1]] <- data.table::fread(file.path(filepath, paste0(prefix, ".example")))
        if (file.exists(file.path(filepath, paste0(prefix, ".rds")))) {
            list_FBMs[[length(list_FBMs) +
                1]] <- bigstatsr::big_attach(file.path(filepath, paste0(prefix, ".rds")))
        }

    }


    merged_df <- data.table::rbindlist(list_descriptions, fill=TRUE)
    merged_df[is.na(merged_df)]<- "Undetermined"

    if("C_region" %in% colnames(merged_df)){
      merged_df$C_region[which(merged_df$C_region=="")]="Undetermined"
    }

    if("Chain" %in% colnames(merged_df)){
      merged_df$Chain[which(merged_df$Chain=="")]="Undetermined"
    }
    # merged_df[merged_df == ""] <- "Undetermined"

    merged_header <- colnames(list_headers[[1]])

    for (i in c(1:length(list_headers))) {
        header_tmp <- colnames(list_headers[[i]])
        if (all(
            length(merged_header) ==
                length(header_tmp)
        ) &&
            all(merged_header == header_tmp)) {

        } else {
            #print("[ERROR] different number of columns and/or colnames")
        }

    }



    m <- ncol(list_FBMs[[1]])  # assuming that all have the same number of columns
    n <- sum(sapply(list_FBMs, nrow))

    if (file.exists(file.path(filepath, paste0("Merged_bm", ".bin")))) {
        file.remove(file.path(filepath, paste0("Merged_bm", ".bin")))
        if (file.exists(file.path(filepath, paste0("Merged_bm", ".desc")))) {
            file.remove(file.path(filepath, paste0("Merged_bm", ".desc")))
        }

    }

    merged_del <- T
    if (file.exists(file.path(filepath, paste0("Merged_bm", ".bk")))) {

        if (file.exists(file.path(filepath, paste0("Merged_bm", ".info")))) {
            tmp_merged_df <- data.table::fread(file.path(filepath, paste0("Merged_bm", ".info")))
            if (nrow(merged_df) ==
                nrow(tmp_merged_df) &
                all(tmp_merged_df[, 1] == merged_df[, 1]) &
                all(tmp_merged_df[, 2] == merged_df[, 2]) &
                all(tmp_merged_df[, 3] == merged_df[, 3])) {
                merged_del <- F
                merged_bm <- bigstatsr::big_attach(file.path(filepath, paste0("Merged_bm", ".rds")))
                merged_df <- tmp_merged_df
                if (nrow(merged_bm) !=
                  nrow(tmp_merged_df)) {
                  merged_del <- T
                }
            }
        }
    }


    if (merged_del == T) {
        if (file.exists(file.path(filepath, paste0("Merged_bm", ".bk")))) {
            file.remove(file.path(filepath, paste0("Merged_bm", ".bk")))
            if (file.exists(file.path(filepath, paste0("Merged_bm", ".rds")))) {
                file.remove(file.path(filepath, paste0("Merged_bm", ".rds")))
            }
        }

        merged_bm <- bigstatsr::FBM(
            n, m, backingfile = file.path(filepath, "Merged_bm"),
            is_read_only = FALSE
        )
        data.table::fwrite(
            merged_df, file.path(filepath, paste0("Merged_bm", ".info")),
            sep = "\t"
        )


        ### Contribution by F. Privé in
        ### https://github.com/privefl/bigstatsr/issues/176
        ## NOTE THIS NEEDS TO SCALE UP BETTER, otherwise it occupies too much RAM
        # bigstatsr::big_apply(
        #     merged_bm, function(X, ind, list_fbm) {
        #         X[, ind] <- do.call(
        #           "rbind", lapply(
        #             list_fbm, function(fbm) fbm[,
        #               ind, drop = FALSE]
        #         )
        #       )
        #         gc()
        #         NULL
        #     }, list_fbm = list_FBMs, a.combine = "c", ncores = nb_cores()
        # )


        # bigstatsr::big_apply(
        #   merged_bm, function(X, ind, list_fbm) {
        #
        #     offset_FBM=0
        #     for (i in c(1:length(list_fbm))) {
        #       for(j in c(1:nrow(list_fbm[[i]]))) {
        #         X[j+offset_FBM, ind]=list_fbm[[i]][j,ind, drop = FALSE]
        #       }
        #       offset_FBM=offset_FBM+nrow(list_fbm[[i]])
        #     }
        #     NULL
        #   }, list_fbm = list_FBMs, a.combine = "c", ncores = nb_cores()
        # )


        #slow but RAM-friendly
        # offset_FBM=0
        # for (i in c(1:length(list_FBMs))) {
        #   print(paste(i,length(list_FBMs), sep="/"))
        #   for(j in c(1:nrow(list_FBMs[[i]]))) {
        #     merged_bm[j+offset_FBM, c(1:m)]=list_FBMs[[i]][j,c(1:m), drop = FALSE]
        #   }
        #   offset_FBM=offset_FBM+nrow(list_FBMs[[i]])
        # }



        bigstatsr::big_apply(
            merged_bm, function(X, ind, list_fbm) {
                X[, ind] <- do.call(
                  "rbind", lapply(
                    list_fbm, function(fbm) fbm[,
                      ind, drop = FALSE]
                )
              )
                gc()
                NULL
            }, list_fbm = list_FBMs, a.combine = "c", ncores = 1
        )

        merged_bm$save()

    }

    return(list(Header = merged_header, Short_DF = merged_df, Big_DF = merged_bm))

}

#' 3_Data_processing
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#' @return The return value, if any, from executing the function.
#' @import bigstatsr
#' @import bigparallelr
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
filter_merged <- function(
    FBM, merged_df, merged_header, use_rgermline, use_repertoire, use_productive,
    use_nonproductive, my_regions, my_var_elements, my_vars, my_vartypes, use_sharedVDJ,
    V_J_to_use, groups, group_A, group_B, group_C, univlog = FALSE, samples_to_keep,
    variables_to_remove, pval_type, pval_cutoff, estimate_cutoff, number_selected_vars,
    VJ_deselected, VDJ_normalized_per_size, R_mut_threshold_min, R_mut_threshold_max,
    to_compare_groups, VDJ_maximize_clones, VDJ_normalized_per_sample, my_clone_def,
    seed, chains, igsubtypes
) {
    "%!in%" <- function(x, y) !(x %in%  y)
    columns <- c(1:ncol(FBM))
    rows <- c(1:nrow(FBM))
    index_repertoire <- which(merged_df$Sequence_type == "Repertoire")


    # print(index_repertoire)

    if (!is.null(samples_to_keep) &&
        samples_to_keep[1] != "RANDOMLETTERS") {
        #print("Filtering samples")

        rows <- rows[which(rows %in% which(merged_df$Patient_Sample %in% samples_to_keep))]

    }

    if (!use_rgermline) {
        rows <- rows[which(rows %in% index_repertoire)]
    }
    if (!use_repertoire) {
        rows <- rows[which(rows %!in% index_repertoire)]
    }

    if(!is.null(chains) && chains[1] != "Unknown") {
      rows <- rows[which(rows %in% which(merged_df$Chain %in% chains))]
    }

    if(!is.null(igsubtypes) && igsubtypes[1] != "Unknown") {
      rows <- rows[which(rows %in% which(merged_df$C_region %in% c(igsubtypes,"")))]
    }

    rows <- rows[which(
        rows %in% intersect(
            which(
                FBM[, which(merged_header == "AA_Whole_Replacement_muts_counts")] >=
                  R_mut_threshold_min
            ),
            which(
                FBM[, which(merged_header == "AA_Whole_Replacement_muts_counts")] <=
                  R_mut_threshold_max
            )
        )
    )]

    if (length(VJ_deselected) !=
        0) {

        rows <- rows[which(rows %in% which(merged_df$V_and_J %!in% VJ_deselected))]
    }

    set.seed(seed)

    if (use_sharedVDJ && to_compare_groups && !is.null(group_B) &&
        !is.null(group_A)) {
        # rows=rows[which(rows%!in%index_repertoire)] if(is.null(groups)){
        # groups='Groups' } list_VJs=list() for(group in
        # unique(merged_df[,get(groups)])) {
        # list_VJs[[length(list_VJs)+1]]=unique(merged_df[which(merged_df[,get(groups)]
        # == group),get('V_and_J')]) } VJs_allowed=Reduce(intersect, list_VJs)
        # print('VJ reduction') print(length(VJs_allowed))
        # rows=rows[which(rows%in%which(merged_df$V_and_J %in% VJs_allowed))]
        # print(length(rows))

        #print("Filtering by VJ")

        if (VDJ_normalized_per_size) {
            tmp_index <- c()

            if (VDJ_normalized_per_sample) {

                id_samples_A <- unique(
                  merged_df$Patient_Sample[intersect(
                    which(merged_df$Patient_Sample %in% samples_to_keep),
                    which(merged_df[[groups]] %in% group_A)
                )]
              )
                n_samples_A <- length(id_samples_A)
                id_samples_B <- unique(
                  merged_df$Patient_Sample[intersect(
                    which(merged_df$Patient_Sample %in% samples_to_keep),
                    which(merged_df[[groups]] %in% group_B)
                )]
              )
                n_samples_B <- length(id_samples_B)
                id_samples_C <- unique(
                  merged_df$Patient_Sample[intersect(
                    which(merged_df$Patient_Sample %in% samples_to_keep),
                    which(merged_df[[groups]] %in% group_C)
                )]
              )
                n_samples_C <- length(id_samples_C)
            } else {
                n_samples_A <- 1
                n_samples_B <- 1
                n_samples_C <- 1
            }
            for (V_J_comb in V_J_to_use) {
                tmp_group_A <- rows[which(
                  rows %in% intersect(
                    which(merged_df$V_and_J %in% V_J_comb),
                    which(merged_df[[groups]] %in% group_A)
                )
              )]
                tmp_group_B <- rows[which(
                  rows %in% intersect(
                    which(merged_df$V_and_J %in% V_J_comb),
                    which(merged_df[[groups]] %in% group_B)
                )
              )]
                tmp_group_C <- rows[which(
                  rows %in% intersect(
                    which(merged_df$V_and_J %in% V_J_comb),
                    which(merged_df[[groups]] %in% group_C)
                )
              )]

                if (VDJ_normalized_per_sample) {
                  min_size_A <- min(
                    sapply(
                      unique(
                        merged_df$Patient_Sample[intersect(
                          which(merged_df$Patient_Sample %in% samples_to_keep),
                          which(merged_df[[groups]] %in% group_A)
                      )]
                    ),
                      function(z) length(
                        intersect(
                          which(merged_df$Patient_Sample == z),
                          tmp_group_A
                      )
                    )
                  )
                )
                  min_size_B <- min(
                    sapply(
                      unique(
                        merged_df$Patient_Sample[intersect(
                          which(merged_df$Patient_Sample %in% samples_to_keep),
                          which(merged_df[[groups]] %in% group_B)
                      )]
                    ),
                      function(z) length(
                        intersect(
                          which(merged_df$Patient_Sample == z),
                          tmp_group_B
                      )
                    )
                  )
                )


                  min_size_AB <- min(c(min_size_A * n_samples_A, min_size_B * n_samples_B))

                  if (!is.null(n_samples_C) &&
                    n_samples_C > 0) {
                    min_size_C <- min(
                      sapply(
                        unique(
                          merged_df$Patient_Sample[intersect(
                            which(merged_df$Patient_Sample %in% samples_to_keep),
                            which(merged_df[[groups]] %in% group_C)
                        )]
                      ),
                        function(z) length(
                          intersect(
                            which(merged_df$Patient_Sample == z),
                            tmp_group_C
                        )
                      )
                    )
                  )
                    min_size_ABC <- min(
                      c(
                        min_size_A * n_samples_A, min_size_B * n_samples_B, min_size_C *
                          n_samples_C
                    )
                  )
                  }

                } else {
                  min_size_A <- length(tmp_group_A)
                  min_size_B <- length(tmp_group_B)
                  min_size_C <- length(tmp_group_C)

                  min_size_AB <- min(c(min_size_A, min_size_B))

                  if (!is.null(n_samples_C)) {
                    min_size_ABC <- min(c(min_size_A, min_size_B, min_size_C))
                  }

                }





                if (VDJ_normalized_per_sample) {
                  min_size_A <- floor(min_size_AB/n_samples_A)
                  min_size_B <- floor(min_size_AB/n_samples_B)
                  if (!is.null(group_C) && length(group_C) > 0 && n_samples_C > 0) {
                    min_size_C <- floor(min_size_ABC / n_samples_C)
                  } else {
                    min_size_C <- 0
                  }
                }

                if (min_size_AB == 0) {

                } else {

                  max_clones_A <- NULL

                  if (VDJ_maximize_clones) {


                    if (VDJ_normalized_per_sample) {
                      max_clones_A <- c()
                      tmp_tmp_group_A <- c()
                      for (n in c(1:n_samples_A)) {

                        sub_tmp_group_A <- tmp_group_A[which(
                          tmp_group_A %in% intersect(
                            intersect(
                              which(merged_df$V_and_J %in% V_J_comb),
                              which(merged_df[[groups]] %in% group_A)
                          ),
                            which(merged_df$Patient_Sample %in% id_samples_A[n])
                        )
                      )]

                        sub_all_clones_A <- sub_tmp_group_A[match(
                          unique(merged_df[sub_tmp_group_A, get(my_clone_def)]),
                          merged_df[sub_tmp_group_A, get(my_clone_def)]
                      )]
                        sub_all_clones_A <- sub_all_clones_A[1:min(min_size_A, length(sub_all_clones_A))]
                        if(length(sub_tmp_group_A[which(
                          merged_df[sub_tmp_group_A, get(my_clone_def)] %!in%
                          merged_df[max_clones_A, get(my_clone_def)]
                        )] > 1)) {
                          tmp_tmp_group_A <- c(
                            tmp_tmp_group_A, c(
                              sub_all_clones_A, sample(
                                sub_tmp_group_A[which(
                                  merged_df[sub_tmp_group_A, get(my_clone_def)] %!in%
                                    merged_df[max_clones_A, get(my_clone_def)]
                                )],
                                min_size_A - length(sub_all_clones_A)
                              )
                            )
                          )
                        } else {
                          tmp_tmp_group_A <- c(
                            tmp_tmp_group_A, c(
                              sub_all_clones_A))
                        }

                      }
                      if ((min_size_AB - min_size_A * n_samples_A) > 0) {
                        if(length(tmp_group_A[which(tmp_group_A %!in% tmp_tmp_group_A)])>1){
                          tmp_tmp_group_A <- c(
                            tmp_tmp_group_A, sample(
                              tmp_group_A[which(tmp_group_A %!in% tmp_tmp_group_A)],
                              min_size_AB - min_size_A * n_samples_A
                            )
                          )
                        } else {
                          tmp_tmp_group_A <- c(tmp_tmp_group_A, tmp_group_A[which(tmp_group_A %!in% tmp_tmp_group_A)])
                        }

                      }

                      tmp_group_A <- tmp_tmp_group_A
                    } else {
                      all_clones_A <- tmp_group_A[match(
                        unique(merged_df[tmp_group_A, get(my_clone_def)]),
                        merged_df[tmp_group_A, get(my_clone_def)]
                        )]
                      if (length(all_clones_A) > 1){
                        max_clones_A <- sample(all_clones_A, min(min_size_AB, length(all_clones_A)))
                      } else {
                        max_clones_A <- all_clones_A
                      }

                      if(length(tmp_group_A[which(tmp_group_A %!in% max_clones_A)]) > 1) {
                        tmp_group_A <- c(
                          max_clones_A, sample(
                            tmp_group_A[which(tmp_group_A %!in% max_clones_A)],
                            min_size_AB - length(max_clones_A)
                          ))
                      } else {
                        if((min_size_AB - length(max_clones_A))>0) {
                          tmp_group_A <- c(
                            max_clones_A, tmp_group_A[which(tmp_group_A %!in% max_clones_A)])
                        } else {
                          tmp_group_A=max_clones_A
                        }

                      }



                    }

                  } else {

                    if (VDJ_normalized_per_sample) {

                      tmp_tmp_group_A <- c()
                      for (n in c(1:n_samples_A)) {

                        sub_tmp_group_A <- tmp_group_A[which(
                          tmp_group_A %in% intersect(
                            intersect(
                              which(merged_df$V_and_J %in% V_J_comb),
                              which(merged_df[[groups]] %in% group_A)
                          ),
                            which(merged_df$Patient_Sample %in% id_samples_A[n])
                        )
                      )]
                        tmp_tmp_group_A <- c(tmp_tmp_group_A,  sub_tmp_group_A[sample.int(length(sub_tmp_group_A), min_size_A)])
                      }
                      if ((min_size_AB - min_size_A * n_samples_A) > 0) {

                        if(length(tmp_group_A[which(tmp_group_A %!in% tmp_tmp_group_A)])>1){
                          tmp_tmp_group_A <- c(
                            tmp_tmp_group_A, sample(
                              tmp_group_A[which(tmp_group_A %!in% tmp_tmp_group_A)],
                              min_size_AB - min_size_A * n_samples_A
                            ))
                        } else {
                          tmp_tmp_group_A <- c(
                            tmp_tmp_group_A, tmp_group_A[which(tmp_group_A %!in% tmp_tmp_group_A)]
                            )
                        }


                      }


                      tmp_group_A <- tmp_tmp_group_A
                    } else {
                      if (length(tmp_group_A)>1){
                        tmp_group_A <- sample(tmp_group_A, min_size_AB)
                      }
                    }


                  }

                  max_clones_B <- NULL

                  if (VDJ_maximize_clones) {


                    if (VDJ_normalized_per_sample) {
                      max_clones_B <- c()
                      tmp_tmp_group_B <- c()
                      for (n in c(1:n_samples_B)) {

                        sub_tmp_group_B <- tmp_group_B[which(
                          tmp_group_B %in% intersect(
                            intersect(
                              which(merged_df$V_and_J %in% V_J_comb),
                              which(merged_df[[groups]] %in% group_B)
                          ),
                            which(merged_df$Patient_Sample %in% id_samples_B[n])
                        )
                      )]

                        sub_all_clones_B <- sub_tmp_group_B[match(
                          unique(merged_df[sub_tmp_group_B, get(my_clone_def)]),
                          merged_df[sub_tmp_group_B, get(my_clone_def)]
                      )]
                        sub_all_clones_B <- sub_all_clones_B[1:min(min_size_B, length(sub_all_clones_B))]

                        if(length(sub_tmp_group_B[which(
                          merged_df[sub_tmp_group_B, get(my_clone_def)] %!in%
                          merged_df[max_clones_B, get(my_clone_def)]
                        )])>1){
                          tmp_tmp_group_B <- c(
                            tmp_tmp_group_B, c(
                              sub_all_clones_B, sample(
                                sub_tmp_group_B[which(
                                  merged_df[sub_tmp_group_B, get(my_clone_def)] %!in%
                                    merged_df[max_clones_B, get(my_clone_def)]
                                )],
                                min_size_B - length(sub_all_clones_B)
                              )
                            )
                          )
                        } else {
                          if ((min_size_B - length(sub_all_clones_B))>0){
                            tmp_tmp_group_B <- c(
                              tmp_tmp_group_B, sub_all_clones_B,
                              sub_tmp_group_B[which(
                                merged_df[sub_tmp_group_B, get(my_clone_def)] %!in%
                                  merged_df[max_clones_B, get(my_clone_def)]
                              )]
                            )
                          }
                        }

                      }
                      if ((min_size_AB - min_size_B * n_samples_B) > 0) {
                        if(length(tmp_group_B[which(tmp_group_B %!in% tmp_tmp_group_B)])>1) {
                          tmp_tmp_group_B <- c(
                            tmp_tmp_group_B, sample(
                              tmp_group_B[which(tmp_group_B %!in% tmp_tmp_group_B)],
                              min_size_AB - min_size_B * n_samples_B
                            ))
                        } else {
                          tmp_tmp_group_B <- c(
                            tmp_tmp_group_B, tmp_group_B[which(tmp_group_B %!in% tmp_tmp_group_B)]
                            )
                        }
                      }

                      tmp_group_B <- tmp_tmp_group_B
                    } else {
                      all_clones_B <- tmp_group_B[match(
                        unique(merged_df[tmp_group_B, get(my_clone_def)]),
                        merged_df[tmp_group_B, get(my_clone_def)]
                      )]
                      if (length(all_clones_B) > 1){
                        max_clones_B <- sample(all_clones_B, min(min_size_AB, length(all_clones_B)))
                      } else {
                        max_clones_B <- all_clones_B
                      }

                      if(length(tmp_group_B[which(tmp_group_B %!in% max_clones_B)]) > 1) {
                        tmp_group_B <- c(
                          max_clones_B, sample(
                            tmp_group_B[which(tmp_group_B %!in% max_clones_B)],
                            min_size_AB - length(max_clones_B)
                          ))
                      } else {
                        if((min_size_AB - length(max_clones_B))>0) {
                          tmp_group_B <- c(
                            max_clones_B, tmp_group_B[which(tmp_group_B %!in% max_clones_B)])
                        } else {
                          tmp_group_B=max_clones_B
                        }

                      }
                    }

                  } else {

                    if (VDJ_normalized_per_sample) {

                      tmp_tmp_group_B <- c()
                      for (n in c(1:n_samples_B)) {

                        sub_tmp_group_B <- tmp_group_B[which(
                          tmp_group_B %in% intersect(
                            intersect(
                              which(merged_df$V_and_J %in% V_J_comb),
                              which(merged_df[[groups]] %in% group_B)
                          ),
                            which(merged_df$Patient_Sample %in% id_samples_B[n])
                        )
                      )]

                        tmp_tmp_group_B <- c(tmp_tmp_group_B, sub_tmp_group_B[sample.int(length(sub_tmp_group_B), min_size_B)])
                      }
                      if ((min_size_AB - min_size_B * n_samples_B) > 0) {
                        if(length(tmp_group_B[which(tmp_group_B %!in% tmp_tmp_group_B)])>1){
                          tmp_tmp_group_B <- c(
                            tmp_tmp_group_B, sample(
                              tmp_group_B[which(tmp_group_B %!in% tmp_tmp_group_B)],
                              min_size_AB - min_size_B * n_samples_B
                            ))
                        } else {
                          tmp_tmp_group_B <- c(tmp_tmp_group_B,
                                               tmp_group_B[which(tmp_group_B %!in% tmp_tmp_group_B)])
                        }

                      }


                      tmp_group_B <- tmp_tmp_group_B


                    } else {
                      if (length(tmp_group_B)>1){
                        tmp_group_B <- sample(tmp_group_B, min_size_AB)
                      }

                    }


                  }


                  # if(length(tmp_group_C)>0){ # tmp_index=c(tmp_index,
                  # tmp_group_C[1:min(c(length(tmp_group_A),length(tmp_group_B),
                  # length(tmp_group_C)))]) max_clones_C=NULL
                  # if(VDJ_maximize_clones) {
                  # max_clones_C=tmp_group_C[match(unique(merged_df[tmp_group_C,which(colnames(merged_df)==my_clone_def)]),
                  # merged_df[tmp_group_C,which(colnames(merged_df)==my_clone_def)])]
                  # max_clones_C=sample( tmp_group_C, min_size_ABC) }
                  # tmp_group_C= c(max_clones_C,
                  # sample(tmp_group_C[which(tmp_group_C %!in% max_clones_C)],
                  # min_size_ABC)) }

                  if (length(tmp_group_C) >
                    0) {
                    max_clones_C <- NULL

                    if (VDJ_maximize_clones) {


                      if (VDJ_normalized_per_sample) {
                        max_clones_C <- c()
                        tmp_tmp_group_C <- c()
                        for (n in c(1:n_samples_C)) {

                          sub_tmp_group_C <- tmp_group_C[which(
                            tmp_group_C %in% intersect(
                              intersect(
                                which(merged_df$V_and_J %in% V_J_comb),
                                which(merged_df[[groups]] %in% group_C)
                            ),
                              which(merged_df$Patient_Sample %in% id_samples_C[n])
                          )
                        )]

                          sub_all_clones_C <- sub_tmp_group_C[match(
                            unique(merged_df[sub_tmp_group_C, get(my_clone_def)]),
                            merged_df[sub_tmp_group_C, get(my_clone_def)]
                        )]
                          sub_all_clones_C <- sub_all_clones_C[1:min(min_size_C, length(sub_all_clones_C))]

                          if(length(sub_tmp_group_C[which(
                            merged_df[sub_tmp_group_C, get(my_clone_def)] %!in%
                            merged_df[max_clones_C, get(my_clone_def)]
                          )])>1) {
                            tmp_tmp_group_C <- c(
                              tmp_tmp_group_C, c(
                                sub_all_clones_C, sample(
                                  sub_tmp_group_C[which(
                                    merged_df[sub_tmp_group_C, get(my_clone_def)] %!in%
                                      merged_df[max_clones_C, get(my_clone_def)]
                                  )],
                                  min_size_C - length(sub_all_clones_C)
                                )
                              )
                            )
                          } else {
                            if((min_size_C - length(sub_all_clones_C))>0) {
                              tmp_tmp_group_C <- c(
                                tmp_tmp_group_C, c(
                                  sub_all_clones_C, sub_tmp_group_C[which(
                                    merged_df[sub_tmp_group_C, get(my_clone_def)] %!in%
                                      merged_df[max_clones_C, get(my_clone_def)]
                                  )]
                                )
                              )
                            }
                          }

                        }
                        if ((min_size_ABC - min_size_C * n_samples_C) > 0) {
                          tmp_tmp_group_C <- c(
                            tmp_tmp_group_C, sample(
                              tmp_group_C[which(tmp_group_C %!in% tmp_tmp_group_C)],
                              min_size_ABC - min_size_C * n_samples_C
                            )
                          )
                        } else {
                          tmp_tmp_group_C <- c(
                            tmp_tmp_group_C, c(
                              sub_all_clones_C))
                        }

                        tmp_group_C <- tmp_tmp_group_C
                      } else {
                        all_clones_C <- tmp_group_C[match(
                          unique(merged_df[tmp_group_C, get(my_clone_def)]),
                          merged_df[tmp_group_C, get(my_clone_def)]
                        )]
                        if (length(all_clones_C) > 1){
                          max_clones_C <- sample(all_clones_C, min(min_size_ABC, length(all_clones_C)))
                        } else {
                          max_clones_C <- all_clones_C
                        }

                        if(length(tmp_group_C[which(tmp_group_C %!in% max_clones_C)]) > 1) {
                          tmp_group_C <- c(
                            max_clones_C, sample(
                              tmp_group_C[which(tmp_group_C %!in% max_clones_C)],
                              min_size_ABC - length(max_clones_C)
                            ))
                        } else {
                          if((min_size_ABC - length(max_clones_C))>0) {
                            tmp_group_C <- c(
                              max_clones_C, tmp_group_C[which(tmp_group_C %!in% max_clones_C)])
                          } else {
                            tmp_group_C=max_clones_C
                          }

                        }
                      }

                    } else {

                      if (VDJ_normalized_per_sample) {

                        tmp_tmp_group_C <- c()
                        for (n in c(1:n_samples_C)) {

                          sub_tmp_group_C <- tmp_group_C[which(
                            tmp_group_C %in% intersect(
                              intersect(
                                which(merged_df$V_and_J %in% V_J_comb),
                                which(merged_df[[groups]] %in% group_C)
                            ),
                              which(merged_df$Patient_Sample %in% id_samples_C[n])
                          )
                        )]

                          tmp_tmp_group_C <- c(tmp_tmp_group_C,
                                               sub_tmp_group_C[sample.int(length(sub_tmp_group_C), min_size_C)])
                        }
                        if ((min_size_AB - min_size_C * n_samples_C) > 0) {
                          tmp_tmp_group_C <- c(
                            tmp_tmp_group_C,
                            tmp_group_C[which(tmp_group_C %!in% tmp_tmp_group_C)][sample.int(length(tmp_group_C[which(tmp_group_C %!in% tmp_tmp_group_C)]),
                                                                                             min_size_ABC - min_size_C * n_samples_C)]

                            )
                        }


                        tmp_group_C <- tmp_tmp_group_C


                      } else {
                        if (length(tmp_group_C)>1){
                          tmp_group_C <- sample(c(tmp_group_C), min_size_ABC)
                        }

                      }


                    }

                  } else {
                    tmp_group_C <- NULL
                  }

                  # No random tmp_index=c(tmp_index,
                  # c(tmp_group_A[1:min(c(length(tmp_group_A),length(tmp_group_B)))],
                  # tmp_group_B[1:min(c(length(tmp_group_A),length(tmp_group_B)))])
                  # )

                  tmp_index <- c(tmp_index, c(tmp_group_A, tmp_group_B, tmp_group_C))

                }



            }


            rows <- rows[which(rows %in% tmp_index)]
        } else {
            rows <- rows[which(rows %in% which(merged_df$V_and_J %in% V_J_to_use))]
        }


    }







    if (!is.null(my_regions)) {
        columns <- columns[which(
            columns %in% which(
                (sapply(merged_header, function(x) strsplit(x, split = "_")[[1]][2])) %in%
                  my_regions
            )
        )]
    } else {
        columns <- c()
    }

    if (!is.null(my_var_elements)) {
        columns <- columns[which(
            columns %in% which(
                grepl(
                  paste0(
                    paste("^", my_var_elements, sep = ""),
                    collapse = "|"
                ),
                  merged_header
              )
            )
        )]
    } else {
        columns <- c()
    }

    if (!is.null(my_vars)) {

        merged_header_splitted_at_3 <- sapply(merged_header, function(x) strsplit(x, split = "_")[[1]][3])
        merged_header_splitted_at_4 <- sapply(merged_header, function(x) strsplit(x, split = "_")[[1]][4])
        merged_header_splitted_at_3_and_4 <- sapply(
            merged_header, function(x) paste(
                strsplit(x, split = "_")[[1]][3],
                strsplit(x, split = "_")[[1]][4],
                sep = "_"
            )
        )

        if ("Length" %!in% my_vars) {
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "length")))]
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "length_Diff")))]
        }

        if ("Composition" %!in% my_vars) {
            nucleotides <- c("A", "G", "T", "C")
            peptides <- Peptides::aaList()
            columns <- columns[which(
                columns %!in% which(
                  merged_header_splitted_at_3_and_4 %in% unique(
                    c(
                      paste(
                        c(nucleotides, peptides),
                        "count", sep = "_"
                    ),
                      paste(
                        c(nucleotides, peptides),
                        "norm", sep = "_"
                    )
                  )
                )
              )
            )]


            columns <- columns[which(columns %!in% which(merged_header_splitted_at_4 %in% c("counts")))]
        }

        if ("Hot/Cold motifs" %!in% my_vars) {
            columns <- columns[which(
                columns %!in% which(merged_header_splitted_at_3 %in% c("hot", "cold", "potential"))
            )]
        }

        if ("Substitutions" %!in% my_vars) { ###Outdated
            columns <- columns[which(
                columns %!in% which(
                  merged_header_splitted_at_3_and_4 %in% c("Sub", "Sub_prc", "SIDT_sum", "SID_sum")
              )
            )]
            columns <- columns[which(columns %!in% which(merged_header_splitted_at_3 %in% c("Sub")))]
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "_S")))]  ##Old, remove later
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "_S_prc")))]  ##Old, remove later
        }

        if ("Insertions" %!in% my_vars) {
            columns <- columns[which(
                columns %!in% which(
                  merged_header_splitted_at_3 %in% c("NumIns", "LenghtIns")
              )
            )]
            # columns <- columns[which(columns %!in% which(merged_header_splitted_at_3 %in% c("Ins")))]
            # columns <- columns[which(columns %!in% which(endsWith(merged_header, "_I")))]  ##Old, remove later
            # columns <- columns[which(columns %!in% which(endsWith(merged_header, "_I_prc")))]  ##Old, remove later
        }
        if ("Deletions" %!in% my_vars) {
            columns <- columns[which(
                columns %!in% which(
                  merged_header_splitted_at_3 %in% c("LenghtDels", "NumDels")
              )
            )]
            # columns <- columns[which(columns %!in% which(merged_header_splitted_at_3 %in% c("Del")))]
            # columns <- columns[which(columns %!in% which(endsWith(merged_header, "_D")))]  ##Old, remove later
            # columns <- columns[which(columns %!in% which(endsWith(merged_header, "_D_prc")))]  ##Old, remove later
        }
        if ("Translocations" %!in% my_vars) { ###Outdated
            columns <- columns[which(
                columns %!in% which(merged_header_splitted_at_3_and_4 %in% c("Trasl", "Trasl_prc", "SIDT_sum"))
            )]
            columns <- columns[which(columns %!in% which(merged_header_splitted_at_3 %in% c("Trasl")))]
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "_T")))]  ##Old, remove later
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "_T_prc")))]  ##Old, remove later
        }

        if ("Leveshtein distance" %!in% my_vars) {  ###Outdated
            columns <- columns[which(columns %!in% which(merged_header_splitted_at_3 %in% c("lv")))]
        }

        if ("Transitions and transversions" %!in% my_vars) {
            columns <- columns[which(
                columns %!in% which(merged_header_splitted_at_3 %in% c("Transitions", "Transversions"))
            )]
            columns <- columns[which(
                columns %!in% which(merged_header_splitted_at_3_and_4 %in% c("Ratio_Transitions-Transversions"))
            )]
        }
        if ("Replacement and silent mutations" %!in% my_vars) {
            columns <- columns[which(
                columns %!in% which(merged_header_splitted_at_3 %in% c("Replacement", "Silent", "Total"))
            )]
            columns <- columns[which(
                columns %!in% which(merged_header_splitted_at_3_and_4 %in% c("Ratio_Replacement-Silent"))
            )]
        }

        if ("Mutations from X to Y" %!in% my_vars) {
            columns <- columns[which(columns %!in% which(merged_header_splitted_at_4 %in% c("to")))]
            # columns=columns[which(columns %!in% which(sapply(merged_header,
            # function(x) paste(strsplit(x, split='_')[[1]][3], strsplit(x,
            # split='_')[[1]][4], sep='_')) %in% c('Ratio_Silent') ))]
        }

        if ("NGly sites" %!in% my_vars) {
            columns <- columns[which(columns %!in% which(merged_header_splitted_at_3 %in% c("NGly")))]
        }
        if ("Peptide features" %!in% my_vars) {

          tmp_columns <- columns[intersect(which(
            columns %in% which(
              merged_header_splitted_at_3 %in% c(
                "Peptides", "alkzm", "Small", "Tiny", "Aliphatic", "Charged",
                "Polar", "Basic", "NonPolar", "Aromatic", "Acidic"
              )
            )
          ),which(columns %in% which(merged_header_splitted_at_4 %in% c("to"))))]

            columns <- columns[which(
                columns %!in% which(
                  merged_header_splitted_at_3 %in% c(
                    "Peptides", "alkzm", "Small", "Tiny", "Aliphatic", "Charged",
                    "Polar", "Basic", "NonPolar", "Aromatic", "Acidic"
                )
              )
            )]

            columns=c(columns, tmp_columns)
            # columns=columns[which(columns %!in% which(sapply(merged_header,
            # function(x) strsplit(x, split='_')[[1]][3]) %in% c('alkzm') ))]
            # columns=columns[which(columns %!in% which(sapply(merged_header,
            # function(x) strsplit(x, split='_')[[1]][3]) %in% c('alkzm') ))]
        }


    } else {
        columns <- c()
    }

    if (!is.null(my_vartypes)) {
        if ("Germline diff" %!in% my_vartypes) {
            columns <- columns[which(columns %!in% which(endsWith(merged_header, "_Diff")))]
        }
        if ("Baseline" %!in% my_vartypes) {
            columns <- columns[which(columns %in% which(endsWith(merged_header, "_Diff")))]
        }
    } else {
        columns <- c()
    }

    if (!is.null(variables_to_remove)) {
        columns <- columns[which(columns %!in% which(merged_header %in% variables_to_remove))]
    }



    big_summary <- bigstatsr::big_colstats(FBM, ind.row = rows, ind.col = columns, ncores = bigstatsr::nb_cores())
    vars_to_remove <- columns[unique(
        c(
            which(big_summary$sum == 0),
            which(big_summary$var == 0),
            which(is.na(big_summary$sum)),
            which(is.infinite(big_summary$sum)),
            which(is.nan(big_summary$sum)),
            which(is.na(big_summary$var)),
            which(is.infinite(big_summary$var)),
            which(is.nan(sqrt(big_summary$var)))
        )
    )]

    # if(any(is.nan(sqrt(big_summary$var)))) { print(FBM[rows,
    # columns[which(is.nan(sqrt(big_summary$var)))]]) }
    columns <- columns[which(columns %!in% vars_to_remove)]

    if (univlog && !is.null(columns) &&
        !is.null(group_B) &&
        !is.null(group_A) &&
        to_compare_groups) {

        merged_df$Binary_classif <- 4
        merged_df$Binary_classif[which(merged_df[[groups]] %in% group_A)] <- 0
        merged_df$Binary_classif[which(merged_df[[groups]] %in% group_B)] <- 1
        tmp_rows <- rows[(which(rows %in% which(merged_df[[groups]] %in% c(group_B, group_A))))]

        # print(length(tmp_rows))
        # print(table(merged_df$Binary_classif[tmp_rows]))
        # if(any(merged_df$Binary_classif[tmp_rows] %!in% c(0,1))){
        #   print("Unusual")
        #   print(which(merged_df$Binary_classif[tmp_rows] %!in% c(0,1)))
        #   print(merged_df$Binary_classif[tmp_rows][which(merged_df$Binary_classif[tmp_rows] %!in% c(0,1))])
        # }
        testuniv <- big_univLogReg(
            FBM, merged_df$Binary_classif[tmp_rows], ind.train = tmp_rows, ind.col = columns,
            ncores = nb_cores()
        )

        testuniv$p.value <- stats::predict(testuniv, log10 = FALSE)


        if (pval_type == "Corrected by Benjamini & Hochberg method") {
            testuniv$p.value <- round(
                stats::p.adjust(testuniv$p.value, "BH"),
                6
            )
        } else if (pval_type == "Corrected by Bonferroni method") {
            testuniv$p.value <- round(
                stats::p.adjust(testuniv$p.value, "bonferroni"),
                6
            )
        } else if (pval_type == "Corrected by Benjamini & Yekutieli method") {
          testuniv$p.value <- round(
            stats::p.adjust(testuniv$p.value, "BY"),
            6
          )
        }


        intersection <- intersect(
          which(
            abs(testuniv$p.value) <=
              pval_cutoff
          ),
          which(
            abs(testuniv$estim) >=
              estimate_cutoff
          )
        )
        length(intersection)

        if (number_selected_vars == "All") {
            columns <- columns[intersection]
        } else if (number_selected_vars == "Top 10 (according to estimate)") {

            columns <- columns[intersection[sort(
                abs(testuniv$estim)[intersection],
                index.return = TRUE, decreasing = TRUE
            )$ix[1:min(10, length(intersection))]]]

        } else if (number_selected_vars == "Top 50 (according to estimate)") {

            columns <- columns[intersection[sort(
                abs(testuniv$estim)[intersection],
                index.return = TRUE, decreasing = TRUE
            )$ix[1:min(50, length(intersection))]]]
        }



    }

    columns=columns[which(!is.na(columns))]




    return(list(ROWS = rows, COLUMNS = columns))
}

#' Big PCA
#'
#' @description **Internal function.** Not intended for direct use. Exported only for
#'    `shinymeta` report rendering via `::` access. Use [run_app()] instead.
#'    A function to perform PCA over a FBM object.
#' @param FBM A FBM object.
#' @param rows Index of rows of the FBM that will be used to calculate the PCA.
#' @param columns Index of columns of the FBM that will be used to calculate the PCA..
#' @return A list with the PCA scores and its explained variances
#' @import bigstatsr
#' @keywords internal
#' @export
#' @examples
#' \donttest{
#'   # Internal function exported for shinymeta :: access during report rendering.
#'   # Requires a live Shiny reactive context and real AIRR-seq data.
#'   # Use run_app() as the user-facing entry point.
#' }
big_PCA <- function(FBM, rows, columns) {

    if (length(columns) >
        0 && length(rows) >
        0) {

        PCA_error_perhaps <- tryCatch(
            {

              testing <- bigstatsr::big_randomSVD(
                FBM, fun.scaling = big_scale(), ind.row = rows, ind.col = columns, k = 5,
                verbose = FALSE
              )

              scores <- stats::predict(testing)

                var_exp <- testing$d^2/big_norm(
                  FBM, ind.row = rows, ind.col = columns, center = testing$center,
                  scale = testing$scale
              )
                variance <- signif(var_exp, 2)

                cumulative_variance <- round(
                  cumsum(var_exp),
                  3
              )
                # Contribuciones (%): para estandarizado es simplemente 100 * v^2
                contrib <- 100 * testing$v^2
                # Comprobación: cada columna debe sumar 100
                colSums(contrib)  # ~ 100

                # Nombres útiles (si tienes nombres de variables/PCs)
                # rownames(contrib) <- colnames(mat)  # si partiste de 'mat'
                colnames(contrib) <- paste0("PCA", seq_len(ncol(contrib)))

                list(Scores = scores, Variance_explained = variance,
                     Variance=contrib)
            }, error = function(e) {
                # print("Caught an error!")
                list(Scores = NULL, Variance_explained = NULL,
                     Variance=NULL)
            }, warning = function(w) {
                # print("Caught an warning!")
                list(Scores = NULL, Variance_explained = NULL,
                     Variance=NULL)
            }
        )
        return(
            list(
                Scores = PCA_error_perhaps$Scores,
                Variance_explained = PCA_error_perhaps$Variance_explained,
                Variance=PCA_error_perhaps$Variance
            )
        )

    } else {
        # print("No PCA")
        return(list(Scores = NULL, Variance_explained = NULL,
                    Variance = NULL))
    }


}

test_that("Filtering comparing groups works", {

  FBM=bigstatsr::as_FBM(matrix(c(rep(c(0:5),2),4,rep(14,4),15,rep(15,8)),
                              13)
                        )

  Short_DF=data.frame(
    Sequence_type=rep(c("Repertoire"), 13),
    Patient_Sample=c(rep(c("Pat_a","Pat_b"), each=5), "Pat_b", rep("Pat_c",2)),
    Chain=rep("IGH",13),
    V_and_J=c(rep("VJ",3),rep("VJ2",2),rep("VJ",5),"VJ2",rep("VJ",2)),
    Groupname=c(rep("A",4),"C",rep("B",6),"C","A"),
    Clonedef=c(rep("Clone_1a", 3), "Clone_2a", "Clone_3a",
               rep("Clone_1b", 3), "Clone_2b", "Clone_3b","Clone_4b",
               "Clone_1c","Clone_2c")
  )
  Short_DF=data.table::as.data.table(Short_DF)

  Header=c("AA_Whole_Replacement_muts_counts",
           "NT_CDR3_length")


  expect_equal(c(1:13),
               AbSolution:::filter_merged(
                 FBM=FBM,
                 merged_df=Short_DF,
                 merged_header=Header,
                 use_rgermline=FALSE,
                 use_repertoire=TRUE,
                 use_productive=TRUE,
                 use_nonproductive= FALSE,
                 my_regions= c("CDR3"),
                 my_var_elements=c("NT"),
                 my_vars=c(
                   "Length"
                 ),
                 my_vartypes="Baseline",
                 groups="Groupname",
                 group_A="A",
                 group_B="B",
                 group_C=c("C"),
                 univlog=T,
                 samples_to_keep=c("Pat_a",
                                   "Pat_b",
                                   "Pat_c"),
                 variables_to_remove=NULL,
                 pval_type="Bonferroni",
                 pval_cutoff=0.05,
                 estimate_cutoff=1,
                 number_selected_vars="All",
                 VJ_deselected= NULL,
                 R_mut_threshold_min=0,
                 R_mut_threshold_max=5,
                 V_J_to_use=c("VJ","VJ2"),
                 to_compare_groups=T,
                 use_sharedVDJ=F,
                 VDJ_normalized_per_size=F,
                 VDJ_maximize_clones=F,
                 VDJ_normalized_per_sample=F,
                 my_clone_def="Clonedef",
                 seed=1234,
                 chains="IGH",
                 igsubtypes=NULL
               )$ROWS)
})



test_that("Filtering used shared-VJ works", {

  FBM=bigstatsr::as_FBM(matrix(c(rep(c(0:5),2),4,rep(14,4),15,rep(15,8)),
                               13)
  )

  Short_DF=data.frame(
    Sequence_type=rep(c("Repertoire"), 13),
    Patient_Sample=c(rep(c("Pat_a","Pat_b"), each=5), "Pat_b", rep("Pat_c",2)),
    Chain=rep("IGH",13),
    V_and_J=c(rep("VJ",3),rep("VJ2",2),rep("VJ",5),"VJ2",rep("VJ",2)),
    Groupname=c(rep("A",4),"C",rep("B",6),"C","A"),
    Clonedef=c(rep("Clone_1a", 3), "Clone_2a", "Clone_3a",
               rep("Clone_1b", 3), "Clone_2b", "Clone_3b","Clone_4b",
               "Clone_1c","Clone_2c")
  )
  Short_DF=data.table::as.data.table(Short_DF)

  Header=c("AA_Whole_Replacement_muts_counts",
           "NT_CDR3_length")


  expect_equal(c(1:13),
               AbSolution:::filter_merged(
                 FBM=FBM,
                 merged_df=Short_DF,
                 merged_header=Header,
                 use_rgermline=FALSE,
                 use_repertoire=TRUE,
                 use_productive=TRUE,
                 use_nonproductive= FALSE,
                 my_regions= c("CDR3"),
                 my_var_elements=c("NT"),
                 my_vars=c(
                   "Length"
                 ),
                 my_vartypes="Baseline",
                 groups="Groupname",
                 group_A="A",
                 group_B="B",
                 group_C=c("C"),
                 univlog=T,
                 samples_to_keep=c("Pat_a",
                                   "Pat_b",
                                   "Pat_c"),
                 variables_to_remove=NULL,
                 pval_type="Bonferroni",
                 pval_cutoff=0.05,
                 estimate_cutoff=1,
                 number_selected_vars="All",
                 VJ_deselected= NULL,
                 R_mut_threshold_min=0,
                 R_mut_threshold_max=5,
                 V_J_to_use=c("VJ","VJ2"),
                 to_compare_groups=T,
                 use_sharedVDJ=T,
                 VDJ_normalized_per_size=F,
                 VDJ_maximize_clones=F,
                 VDJ_normalized_per_sample=F,
                 my_clone_def="Clonedef",
                 seed=1234,
                 chains="IGH",
                 igsubtypes=NULL
               )$ROWS)
})


test_that("Filtering used shared-VJ normalized per group works", {

  FBM=bigstatsr::as_FBM(matrix(c(rep(c(0:5),2),4,rep(14,4),15,rep(15,8)),
                               13)
  )

  Short_DF=data.frame(
    Sequence_type=rep(c("Repertoire"), 13),
    Patient_Sample=c(rep(c("Pat_a","Pat_b"), each=5), "Pat_b", rep("Pat_c",2)),
    Chain=rep("IGH",13),
    V_and_J=c(rep("VJ",3),rep("VJ2",2),rep("VJ",5),"VJ2",rep("VJ",2)),
    Groupname=c(rep("A",4),"C",rep("B",6),"C","A"),
    Clonedef=c(rep("Clone_1a", 3), "Clone_2a", "Clone_3a",
               rep("Clone_1b", 3), "Clone_2b", "Clone_3b","Clone_4b",
               "Clone_1c","Clone_2c")
  )
  Short_DF=data.table::as.data.table(Short_DF)

  Header=c("AA_Whole_Replacement_muts_counts",
           "NT_CDR3_length")


  expect_equal(c(c(1:6),c(8:13)),
               AbSolution:::filter_merged(
                 FBM=FBM,
                 merged_df=Short_DF,
                 merged_header=Header,
                 use_rgermline=FALSE,
                 use_repertoire=TRUE,
                 use_productive=TRUE,
                 use_nonproductive= FALSE,
                 my_regions= c("CDR3"),
                 my_var_elements=c("NT"),
                 my_vars=c(
                   "Length"
                 ),
                 my_vartypes="Baseline",
                 groups="Groupname",
                 group_A="A",
                 group_B="B",
                 group_C=c("C"),
                 univlog=T,
                 samples_to_keep=c("Pat_a",
                                   "Pat_b",
                                   "Pat_c"),
                 variables_to_remove=NULL,
                 pval_type="Bonferroni",
                 pval_cutoff=0.05,
                 estimate_cutoff=1,
                 number_selected_vars="All",
                 VJ_deselected= NULL,
                 R_mut_threshold_min=0,
                 R_mut_threshold_max=5,
                 V_J_to_use=c("VJ","VJ2"),
                 to_compare_groups=T,
                 use_sharedVDJ=T,
                 VDJ_normalized_per_size=T,
                 VDJ_maximize_clones=F,
                 VDJ_normalized_per_sample=F,
                 my_clone_def="Clonedef",
                 seed=1234,
                 chains="IGH",
                 igsubtypes=NULL
               )$ROWS)
})


test_that("Filtering used shared-VJ normalized per group and sample works", {

  FBM=bigstatsr::as_FBM(matrix(c(rep(c(0:5),2),4,rep(14,4),15,rep(15,8)),
                               13)
  )

  Short_DF=data.frame(
    Sequence_type=rep(c("Repertoire"), 13),
    Patient_Sample=c(rep(c("Pat_a","Pat_b"), each=5), "Pat_b", rep("Pat_c",2)),
    Chain=rep("IGH",13),
    V_and_J=c(rep("VJ",3),rep("VJ2",2),rep("VJ",5),"VJ2",rep("VJ",2)),
    Groupname=c(rep("A",4),"C",rep("B",6),"C","A"),
    Clonedef=c(rep("Clone_1a", 3), "Clone_2a", "Clone_3a",
               rep("Clone_1b", 3), "Clone_2b", "Clone_3b","Clone_4b",
               "Clone_1c","Clone_2c")
  )
  Short_DF=data.table::as.data.table(Short_DF)

  Header=c("AA_Whole_Replacement_muts_counts",
           "NT_CDR3_length")


  expect_equal(c(c(2,9),c(10,13)),
               AbSolution:::filter_merged(
                 FBM=FBM,
                 merged_df=Short_DF,
                 merged_header=Header,
                 use_rgermline=FALSE,
                 use_repertoire=TRUE,
                 use_productive=TRUE,
                 use_nonproductive= FALSE,
                 my_regions= c("CDR3"),
                 my_var_elements=c("NT"),
                 my_vars=c(
                   "Length"
                 ),
                 my_vartypes="Baseline",
                 groups="Groupname",
                 group_A="A",
                 group_B="B",
                 group_C=c("C"),
                 univlog=T,
                 samples_to_keep=c("Pat_a",
                                   "Pat_b",
                                   "Pat_c"),
                 variables_to_remove=NULL,
                 pval_type="Bonferroni",
                 pval_cutoff=0.05,
                 estimate_cutoff=1,
                 number_selected_vars="All",
                 VJ_deselected= NULL,
                 R_mut_threshold_min=0,
                 R_mut_threshold_max=5,
                 V_J_to_use=c("VJ","VJ2"),
                 to_compare_groups=T,
                 use_sharedVDJ=T,
                 VDJ_normalized_per_size=T,
                 VDJ_maximize_clones=F,
                 VDJ_normalized_per_sample=T,
                 my_clone_def="Clonedef",
                 seed=1234,
                 chains="IGH",
                 igsubtypes=NULL
               )$ROWS)
})


test_that("Filtering used shared-VJ normalized per group and clone works", {

  FBM=bigstatsr::as_FBM(matrix(c(rep(c(0:5),2),4,rep(14,4),15,rep(15,8)),
                               13)
  )

  Short_DF=data.frame(
    Sequence_type=rep(c("Repertoire"), 13),
    Patient_Sample=c(rep(c("Pat_a","Pat_b"), each=5), "Pat_b", rep("Pat_c",2)),
    Chain=rep("IGH",13),
    V_and_J=c(rep("VJ",3),rep("VJ2",2),rep("VJ",5),"VJ2",rep("VJ",2)),
    Groupname=c(rep("A",4),"C",rep("B",6),"C","A"),
    Clonedef=c(rep("Clone_1a", 3), "Clone_2a", "Clone_3a",
               rep("Clone_1b", 3), "Clone_2b", "Clone_3b","Clone_4b",
               "Clone_1c","Clone_2c")
  )
  Short_DF=data.table::as.data.table(Short_DF)

  Header=c("AA_Whole_Replacement_muts_counts",
           "NT_CDR3_length")


  expect_equal(c(c(1:7),c(9:13)),
               AbSolution:::filter_merged(
                 FBM=FBM,
                 merged_df=Short_DF,
                 merged_header=Header,
                 use_rgermline=FALSE,
                 use_repertoire=TRUE,
                 use_productive=TRUE,
                 use_nonproductive= FALSE,
                 my_regions= c("CDR3"),
                 my_var_elements=c("NT"),
                 my_vars=c(
                   "Length"
                 ),
                 my_vartypes="Baseline",
                 groups="Groupname",
                 group_A="A",
                 group_B="B",
                 group_C=c("C"),
                 univlog=T,
                 samples_to_keep=c("Pat_a",
                                   "Pat_b",
                                   "Pat_c"),
                 variables_to_remove=NULL,
                 pval_type="Bonferroni",
                 pval_cutoff=0.05,
                 estimate_cutoff=1,
                 number_selected_vars="All",
                 VJ_deselected= NULL,
                 R_mut_threshold_min=0,
                 R_mut_threshold_max=5,
                 V_J_to_use=c("VJ","VJ2"),
                 to_compare_groups=T,
                 use_sharedVDJ=T,
                 VDJ_normalized_per_size=T,
                 VDJ_maximize_clones=T,
                 VDJ_normalized_per_sample=F,
                 my_clone_def="Clonedef",
                 seed=1234,
                 chains="IGH",
                 igsubtypes=NULL
               )$ROWS)
})


test_that("Filtering used shared-VJ normalized per group, sample and clone works", {

  FBM=bigstatsr::as_FBM(matrix(c(rep(c(0:5),2),4,rep(14,4),15,rep(15,8)),
                               13)
  )

  Short_DF=data.frame(
    Sequence_type=rep(c("Repertoire"), 13),
    Patient_Sample=c(rep(c("Pat_a","Pat_b"), each=5), "Pat_b", rep("Pat_c",2)),
    Chain=rep("IGH",13),
    V_and_J=c(rep("VJ",3),rep("VJ2",2),rep("VJ",5),"VJ2",rep("VJ",2)),
    Groupname=c(rep("A",4),"C",rep("B",6),"C","A"),
    Clonedef=c(rep("Clone_1a", 3), "Clone_2a", "Clone_3a",
               rep("Clone_1b", 3), "Clone_2b", "Clone_3b","Clone_4b",
               "Clone_1c","Clone_2c")
  )
  Short_DF=data.table::as.data.table(Short_DF)

  Header=c("AA_Whole_Replacement_muts_counts",
           "NT_CDR3_length")


  expect_equal(c(c(1,6),c(9,12), 13),
               AbSolution:::filter_merged(
                 FBM=FBM,
                 merged_df=Short_DF,
                 merged_header=Header,
                 use_rgermline=FALSE,
                 use_repertoire=TRUE,
                 use_productive=TRUE,
                 use_nonproductive= FALSE,
                 my_regions= c("CDR3"),
                 my_var_elements=c("NT"),
                 my_vars=c(
                   "Length"
                 ),
                 my_vartypes="Baseline",
                 groups="Groupname",
                 group_A="A",
                 group_B="B",
                 group_C=c("C"),
                 univlog=T,
                 samples_to_keep=c("Pat_a",
                                   "Pat_b",
                                   "Pat_c"),
                 variables_to_remove=NULL,
                 pval_type="Bonferroni",
                 pval_cutoff=0.05,
                 estimate_cutoff=1,
                 number_selected_vars="All",
                 VJ_deselected= NULL,
                 R_mut_threshold_min=0,
                 R_mut_threshold_max=5,
                 V_J_to_use=c("VJ","VJ2"),
                 to_compare_groups=T,
                 use_sharedVDJ=T,
                 VDJ_normalized_per_size=T,
                 VDJ_maximize_clones=T,
                 VDJ_normalized_per_sample=T,
                 my_clone_def="Clonedef",
                 seed=1234,
                 chains="IGH",
                 igsubtypes=NULL
               )$ROWS)
})

---
title: "AbSolution"
author: "Rodrigo García-Valiente"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 4
    highlight: tango
    number_sections: true
    fig_caption: true
vignette: >
  %\VignetteIndexEntry{AbSolution}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

# A guide to `AbSolution`

`AbSolution` is an interactive tool for exploring immune repertoires and their sequence-based features. `AbSolution` has been designed under the principles of accessibility, scalability, flexibility, interactivity and reproducibility to be approachable to all users while customizable for the different AIRR-Seq studies.


This vignette offers an overview of `AbSolution`'s functionality, guides you on how to initialize the app with your own data, and how to customize the app.

## Launching `AbSolution`

This is easy. Only two lines are required.

```{r setup_AbS, warning = FALSE, message = FALSE, eval = FALSE, results="hide"}
# Load required package
library(AbSolution)

# Load Shiny app
AbSolution::run_app()

```

Make sure you have space in your hard-drive! It can very easily occupy quite a lot of gigabytes.


### Quick Help

Need customized information about your **current step**? Check the **Help subtab** in the `AbSolution` tab.

Looking for an **in-depth explanation of an option**? Activate **tooltip mode** by clicking the "?" at the top right corner to see explanations for each option when you hover over them.

![The Help subtab adjusts its content based on the current step the user is in.](img/Fig_Help_subtab.png)

![The tooltip mode is activated (top right corner) and a tooltip is shown over the hovered option.](img/Fig_Menu_top_right.png)

### Setting up the project analysis - Step 0

In the *0.Project information* tab, we will define the type of project, samples, and workspace to be used. You need to indicate:

- If your data is **TCR or BCR** repertoire-based.
- Provide details about your data, including the **required descriptive table**.
- Specify **the folder where all related files will be created**.

![Tab 0.Project_information. Setting up the project analysis.](img/Fig_Project_information.png)

### *OPTIONAL: try the test dataset*

You can **try `AbSolution` with a [test dataset](https://alakazam.readthedocs.io/en/stable/topics/Example10x)**. This is a small, down-sampled dataset of 10x Genomics Ig V(D)J sequences from CD19+ B cells isolated from the PBMCs of a healthy human donor. The data, provided by 10x Genomics under a Creative Commons Attribution license, has been processed using their Cell Ranger pipeline. It comes from the [Alakazam package](https://alakazam.readthedocs.io/).

![Button - Load the test dataset.](img/Fig_Test_option.png){ width=30% height=30% }

Just click the *Instead, load a test dataset* button and continue the workflow.

### Parsing the sequences - Step 1

In the *1.AIRR-Seq conversion* tab, we will reconstruct the repertoire germlines based on your input. `AbSolution` works with full Fab sequences or sequences with partially-sequenced FWR1/4. You need to indicate:

- The **folder where your data is located**. All files must be in the same folder. Filenames should match those indicated in the previous step. If they are found in the folder, it will say 'Yes' in Found_in_folder; otherwise, it will say 'No'. 
- Additional information to better **reconstruct the germlines**. Decide whether to keep the CDR3 D-gene segment as is or use the D gene information to reconstruct it. Indicate if the FWR1 or FWR4 are partially sequenced and if there is a pre or post-Fab sequence to be removed.

Please confirm with the pre-visualization that the sequences are correctly parsed and reconstructed. A lowercase letter in the repertoire sequence indicates an insertion, while a lowercase letter in the reconstructed germline indicates a deletion in the repertoire.

![Tab 1.AIRR-Seq conversion. Parsing sequences and reconstructing germlines.](img/Fig_AIRR-Seq_conversion.png)

### Calculating sequence-based features - Step 2

In the *2.Sequence feature determination* tab, we will calculate the features (physicochemical, composition, etc.) at the nucleotide (NT) and amino acid (AA) levels for the entire sequence (repertoire and reconstructed germline) and its individual regions.

Just click the button, and wait until it is completed. It can take quite some time depending on your dataset size!

### *ALTERNATIVE: start with previously calculated files - Skip steps 1 & 2*

For pre-calculated data, in step 0 you can select to skip steps 1 and 2. In the **1&2.Select your FBM and associated files** tab, you can load the files and proceed directly to step 3.

You need to specify the **folder where your pre-calculated FBM files are located**. All files must be in the same folder, and filenames should match those indicated in the previous step. If the files are found in the folder, it will display 'Yes' in the Found_in_folder column; otherwise, it will display 'No'.

You can also generate dummy datasets with the same number of mutations to use as a negative reference in the next steps.


### Exploring the dataset and filtering sequences and variables - Step 3

In the **3.Dataset exploration and variable selection** tab, you can customize your analysis by selecting specific sequences, variables, and groups, ensuring a tailored and comprehensive exploration of your dataset. Variables with NAs, infinite values, or zero variance are automatically removed. 


![Tab 3.Dataset exploration. PCA visualization of BCR sequences according to the filtered variables. The variable was selected after differential analysis between a manually selected group against the rest of the sequences. Sequences in the Selection set will be shown in the **Feature Exploration** and **Clonal Exploration** tabs. The Exploration set can be used to try combinations and compare with the Selection set. Selected sequences are highlighted in both visualizations and positions and behaviours can be compared.](img/Fig_Sequences_PCAs.png)





There are different steps and options inside:


1. **Select and filter the sequences.** You can choose whether to include repertoire sequences, reconstructed germline sequences, or both. You can also filter sequences based on productivity, sample, chain type, and the number of replacement (R) mutations.

![Options to select and filter sequences.](img/Fig_Sequences_Option_1.png){ width=30% height=30% }

2. **Select and filter the variables.** You can choose which regions (e.g., CDR3, FWR1, Whole Fab), types of sequences-based features (e.g., AA-based, NT-based, composition, physicochemical, etc.) to include in the analysis, or exclude specific features. You can also work with the baseline values or with the germline-difference values.

![Options to select and filter variables.](img/Fig_Sequences_Option_2.png){ width=30% height=30% }

3. **Decide if you want to compare between groups of sequences.** If so, specify which sequences and the type of comparison you want to perform. You can specify groups of sequences for comparison and normalize groups per VDJ usage or clonal usage to avoid overrepresentation. You can also apply these rules to individual samples instead of at a group-level and include only features correlated to the groups.

![Options to compare between groups.](img/Fig_Sequences_Option_3.png){ width=50% height=50% }

![additional options to compare between groups and action buttons to perform calculations on the Selection or Exploration set.](img/Fig_Sequences_Option_4.png){ width=30% height=30% }

4. **Perform calculations** for the Selection set (which will be shown in the **Feature Exploration** and **Clonal Exploration** tabs) or the Exploration set (which you can use to try combinations and compare with the Selection set).


### Exploring the variables - Step 4

In the *4.Feature exploration* tab, you can plot the filtered variables as a violin plot to observe their behavior according to the selected group in field 4 of the **Dataset Exploration and Variable Selection** tab. You can add the values as dots, select those of interest, and include the reconstructed germline sequences (if they are not already included) to compare how these values have changed since the germline sequence.

![Tab 4.Feature exploration. Options and violin plots displaying the Kidera Factor 2 CDR3 values for two BCR groups. The variable was selected after differential analysis between a manually selected group against the rest of the sequences.](img/Fig_Variables_distribution.png)


### Exploring the clones - Step 5

In the *5.Clonal exploration* tab, we will define the clonal definition to be used. You can:

- Use a pre-existing clone definition (Clone_ID) or calculate it de novo.
- Select a dominance threshold based on relative abundance.
- Identify shared clones between samples.
- Manually select clones of interest.

If you have a clonal definition assigned and the images in the clonal and feature tabs have loaded, you will be able to export results in the **Export Results** tab.

![Tab 5.Clonal exploration. Violin plots displaying the log10 relative frequency distribution of BCR clones across six samples. Highlighted in yellow, a shared clone between samples. The horizontal dashed line represents the dominance threshold. On the top, information regarding clone composition of a hovered clone.](img/Fig_Clones_distribution.png)


![Tab 5.Clonal exploration. Upset plot displaying the shared BCR clones across six samples. Highlighted in yellow, a shared clone between samples. The selected clone is highlighted accordingly in the other plots.](img/Fig_Clones_UpSet.png)



\newpage
# Reproducibility of `AbSolution`

In the *Export results* tab, after running a full analysis with the app (this means you need to have generated clonal figures in Step 5), you can download a .zip file that follows the [ENCORE](https://www.nature.com/articles/s41467-024-52446-8) structure. This .zip file contains the following subfolders:

- **/0_SoftwareEnvironment/R/**: Contains the `AbSolution` package in the version used for the analysis, a Docker file, and the renv information to reproduce everything.
- **/Data/Dataset/**: Contains the dataset used for the analysis.
- **/Raw/**: Contains the sample input files.
- **/Meta/**: Contains a file with information about the samples.
- **/Processed/**: Contains the files produced during the analysis with `AbSolution` (sequence and feature information). These files are used for analysis and data exploration.
- **/Notebook/**: Contains the .Rmd file to generate the figures. This .Rmd file is generated directly from the app's code, capturing the domain logic using shinymeta. Additionally, once exported, it uses relative paths instead of absolute paths, making it easy to knit successfully. The chunks `parse_input` and `feature_calculation` are not evaluated unless the user sets `eval=TRUE`, as they are used to produce the Processed files. For time efficiency, this part is skipped when producing the .zip. There are various fields along the pipeline where users can take notes. These notes will be included in the subsequent notebook and results.
- **/Results/**: Contains the HTML file produced from the .Rmd file with the key plots from `AbSolution`. The HTML file includes all the system information and package versioning details.

It can take some time to load and some time to generate this .zip file. Be patient!

---
title: "Report v.1.00"
author: "AbSolution"
date: "`r format(Sys.time())`"
output: 
  html_document:
    toc: true # table of content true
    toc_depth: 3  # upto three depths of headings (specified by #, ## and ###)
    toc_float: true
    number_sections: true  ## if you want number sections at each table header
    theme: flatly  # many options for theme, this one is my favorite.
    highlight: tango  # specifies the syntax highlighting style
    code_folding: hide
---




```{r setup, include = FALSE}
knitr::opts_chunk$set(
  out.width = "100%", 
  tidy = TRUE
)
{{setupRmarkdown2}}
```


**AbSolution** is a tool for BCR/TCR Fab sequence feature analysis. This report was generated for reproducibility purposes.<br>
<br>
For more details on using AbSolution see <http://rmarkdown.rstudio.com>. AbSolution follows the [ENCORE guidelines](https://www.nature.com/articles/s41467-024-52446-8). <br>

Author: `r (author)`<br>
Contact email: `r (usermail)`

# Project information

<b>User comments:</b> `r (user_0_comments)`<br>

```{r loading_info, warning = FALSE, message = FALSE}
{{loading}}

```

<b>Datasets and parameters used:</b>
```{r loading_parameters, warning = FALSE, message = FALSE, max.height='200px'}
{{parameter_setup}}
```

# Processing input 

<br><b>User comments:</b> `r (user_1_comments)`<br> `r (user_2_comments)`

```{r parse_input, warning = FALSE, message = FALSE, eval = FALSE}
{{parse_input}}
```

# Feature calculation

<br><b>User comments:</b> `r (user_2_comments)`<br> 
```{r feature_calculation, warning = FALSE, message = FALSE, eval = FALSE}
{{feature_calculation}}
```

```{r loading_file, warning = FALSE, message = FALSE}
{{file_load}}
```


# Clonal exploration

```{r clonal_explor, warning = FALSE, message = FALSE}
{{clone}}
```
<br><b>User comments:</b> `r (user_5_comments)`

# Sequence exploration

```{r sequence_selection, warning = FALSE, message = FALSE}
{{PCA_ex}}
```
<br><b>User comments:</b> `r (user_3_comments)`

# Feature exploration

```{r feature, warning = FALSE, message = FALSE}
{{feature}}
```
<br><b>User comments:</b> `r (user_4_comments)`

# Citation
## AbSolution
## ENCORE
> *van Kampen AHC, Mahamune U, Jongejan A, van Schaik BDC, Balashova D, Lashgari D, Pras-Raves M, Wever EJM, Dane AD, García-Valiente R, Moerland PD. ENCORE: a practical implementation to improve reproducibility and transparency of computational research. Nat Commun. 2024 Sep 16;15(1):8117. doi: 10.1038/s41467-024-52446-8. PMID: 39284801; PMCID: PMC11405857.*

# System and version information

## System information
```{r system_info, warning = FALSE, message = FALSE}
          osVersion
          Sys.info()[which(names(Sys.info()) %in% c("sysname", "release", "version", "machine"))]
          print(benchmarkme::get_ram(), unit_system = "iec")
          benchmarkme::get_cpu()
```

## R information
```{r r_info, warning = FALSE, message = FALSE}
          sessionInfo()

```


<br><br><br><br>

![](data:image/png;base64,[encoded data, 27704 characters])





<link rel="icon" type="image/png" sizes="16x16" href="data:image/png;base64,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" />
<link rel="shortcut icon" type="image/png" sizes="16x16" href="data:image/png;base64,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" />

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fct_2_Feature_determination.R
\name{Feature__dataset}
\alias{Feature__dataset}
\title{2_Feature_determination_2}
\usage{
Feature__dataset(
  path_base,
  DF_to_parse,
  name_DF_to_parse,
  FWR1partial,
  FWR4partial,
  standard = TRUE
)
}
\value{
The return value, if any, from executing the function.
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_3_Data_processing.R
\name{Ab_palette}
\alias{Ab_palette}
\title{Ab_palette: color ranges adapted to the BCR/TCR gene structure}
\usage{
Ab_palette(
  list_values,
  vect_genes_comb = NA,
  type_values = c("V", "D", "J", "VJ", "VDJ", "cuantitative", "cualitative"),
  colorblind = FALSE,
  seed = 1234
)
}
\arguments{
\item{list_values}{For V(D)J combinations, a list of lists of the families of
each segment and the genes within them. For V/D/J segments, a list of the
 families and the genes within them. For cuantitative or cualitative
 values, a list of values.}

\item{vect_genes_comb}{The vector of the V(D)J combinations present in the
dataset if type_values is 'VJ' or 'VDJ'. Otherwise, NA.}

\item{type_values}{One of 'V','D','J','VJ','VDJ', 'cuantitative' or
'cualitative'.  'V','D','J','VJ','VDJ',}

\item{colorblind}{If TRUE, the output is a colorblind-friendly vector of
colors from the viridis package. The similarity of the V-D-J colors is
lost.}
}
\value{
palette_colors: a (named) vector of colors
}
\description{
#' @description
   **Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
   Returns a vector of colors according to the input.
   For V/D/J/VJ/VDJ inputs, the colors are produced in a way that genes and
   combination of genes from the same family are assigned a similar color.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fct_3_Data_processing.R
\name{big_PCA}
\alias{big_PCA}
\title{Big PCA}
\usage{
big_PCA(FBM, rows, columns)
}
\arguments{
\item{FBM}{A FBM object.}

\item{rows}{Index of rows of the FBM that will be used to calculate the PCA.}

\item{columns}{Index of columns of the FBM that will be used to calculate the PCA..}
}
\value{
A list with the PCA scores and its explained variances
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
   A function to perform PCA over a FBM object.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_5_Feature_exploration.R
\name{draw_feature_violinplot}
\alias{draw_feature_violinplot}
\title{5_Feature_exploration}
\usage{
draw_feature_violinplot(
  values,
  name_values,
  sequence_info_df,
  group_info,
  additional_group_info = "Patient",
  show_reconstructed,
  compare_opposites = FALSE,
  selected_rows,
  selected_subclones = NULL,
  selected_clones = NULL,
  hide_dots = FALSE,
  really_hide_dots = FALSE,
  width = 1400,
  height = 1000,
  img_type = "png",
  scale = 4,
  seed = 1234,
  source = "feature_violinplot",
  primary_color = "#09BC8A"
)
}
\value{
The return value, if any, from executing the utility.
}
\description{
A utils function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_app}
\alias{run_app}
\title{Run the AbSolution Shiny Application
Launches the interactive AbSolution Shiny application for exploring
B-cell and T-cell receptor repertoire data. The application provides
tools for loading AIRR-formatted datasets, inspecting sequence
characteristics, visualizing clonotype distributions, and generating
summary reports.}
\usage{
run_app(
  verbose = FALSE,
  onStart = NULL,
  options = list(),
  enableBookmarking = "server",
  uiPattern = "/",
  ...
)
}
\arguments{
\item{verbose}{Logical. If \code{FALSE} (default), suppresses all warnings
and messages during app execution. Set to \code{TRUE} to see full console
output for debugging.}

\item{onStart}{A function that will be called before the app is actually run.
This is only needed for \code{shinyAppObj}, since in the \code{shinyAppDir}
case, a \code{global.R} file can be used for this purpose.}

\item{options}{Named options that should be passed to the \code{runApp} call
(these can be any of the following: "port", "launch.browser", "host", "quiet",
"display.mode" and "test.mode"). You can also specify \code{width} and
\code{height} parameters which provide a hint to the embedding environment
about the ideal height/width for the app.}

\item{enableBookmarking}{Can be one of \code{"url"}, \code{"server"}, or
\code{"disable"}. The default value, \code{NULL}, will respect the setting from
any previous calls to  \code{\link[shiny:enableBookmarking]{enableBookmarking()}}. See \code{\link[shiny:enableBookmarking]{enableBookmarking()}}
for more information on bookmarking your app.}

\item{uiPattern}{A regular expression that will be applied to each \code{GET}
request to determine whether the \code{ui} should be used to handle the
request. Note that the entire request path must match the regular
expression in order for the match to be considered successful.}

\item{...}{arguments to pass to golem_opts.
See `?golem::get_golem_options` for more details.}
}
\description{
Internally, `run_app()` calls [`shiny::shinyApp()`] to start the
application, using the UI defined in [\code{app_ui()}] and the server
logic defined in [\code{app_server()}].
}
\details{
This function is the entry point that end-users should call after
installing the package.
}
\examples{
if (interactive()) {
  ### Launch the full application with default settings:
  run_app()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fct_1_Parsing.R
\name{parse_AIRRSeq_file}
\alias{parse_AIRRSeq_file}
\title{1_Parsing}
\usage{
parse_AIRRSeq_file(
  file,
  group,
  patient,
  subgroup,
  sample,
  input_path,
  C_region_included,
  FWR1partial,
  FWR4partial,
  D_gene,
  repertoire,
  output_path,
  is_example = FALSE
)
}
\value{
The return value, if any, from executing the function.
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fct_3_Data_processing.R
\name{filter_merged}
\alias{filter_merged}
\title{3_Data_processing}
\usage{
filter_merged(
  FBM,
  merged_df,
  merged_header,
  use_rgermline,
  use_repertoire,
  use_productive,
  use_nonproductive,
  my_regions,
  my_var_elements,
  my_vars,
  my_vartypes,
  use_sharedVDJ,
  V_J_to_use,
  groups,
  group_A,
  group_B,
  group_C,
  univlog = FALSE,
  samples_to_keep,
  variables_to_remove,
  pval_type,
  pval_cutoff,
  estimate_cutoff,
  number_selected_vars,
  VJ_deselected,
  VDJ_normalized_per_size,
  R_mut_threshold_min,
  R_mut_threshold_max,
  to_compare_groups,
  VDJ_maximize_clones,
  VDJ_normalized_per_sample,
  my_clone_def,
  seed,
  chains,
  igsubtypes
)
}
\value{
The return value, if any, from executing the function.
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fct_3_Data_processing.R
\name{merge_FBMs}
\alias{merge_FBMs}
\title{Merge FBMs and meta files}
\usage{
merge_FBMs(filepath)
}
\arguments{
\item{filepath}{Path to the individual FBMs.}
}
\value{
A list with the merged FBM, the merged info file and the FBM header.
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
   A function to join by row individual FBMs and .info files
    into a unique FBM and a unique .info file, merged_bm.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/golem_utils_server.R
\name{validate_before_run}
\alias{validate_before_run}
\title{Check parameters in 3. are properly selected}
\usage{
validate_before_run(
  work_as_categories,
  use_sharedVDJ,
  VJ_included,
  VDJ_maximize_clones,
  clonal_group
)
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
   A function to perform PCA over a FBM object.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_4_Clonal_exploration.R
\name{calculate_clone}
\alias{calculate_clone}
\title{4_Clonal_exploration}
\usage{
calculate_clone(
  seq_df,
  clonotype,
  AA_or_NT = "NT",
  region = "CDR3",
  percentage = 100,
  calculate_shared_clones
)
}
\value{
The return value, if any, from executing the utility.
}
\description{
**Internal function.** Not intended for direct use. Exported only for
   `shinymeta` report rendering via `::` access. Use [run_app()] instead.
}
\examples{
\donttest{
  # Internal function exported for shinymeta :: access during report rendering.
  # Requires a live Shiny reactive context and real AIRR-seq data.
  # Use run_app() as the user-facing entry point.
}
}
\keyword{internal}
